Package: reprex
Title: Prepare Reproducible Example Code via the Clipboard
Version: 2.1.1
Authors@R: c(
    person("Jennifer", "Bryan", , "jenny@posit.co", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-6983-2759")),
    person("Jim", "Hester", role = "aut",
           comment = c(ORCID = "0000-0002-2739-7082")),
    person("David", "Robinson", , "admiral.david@gmail.com", role = "aut"),
    person("Hadley", "Wickham", , "hadley@posit.co", role = "aut",
           comment = c(ORCID = "0000-0003-4757-117X")),
    person("Christophe", "Dervieux", , "cderv@posit.co", role = "aut",
           comment = c(ORCID = "0000-0003-4474-2498")),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: Convenience wrapper that uses the 'rmarkdown' package to
    render small snippets of code to target formats that include both code
    and output.  The goal is to encourage the sharing of small,
    reproducible, and runnable examples on code-oriented websites, such as
    <https://stackoverflow.com> and <https://github.com>, or in email. The
    user's clipboard is the default source of input code and the default
    target for rendered output. 'reprex' also extracts clean, runnable R
    code from various common formats, such as copy/paste from an R
    session.
License: MIT + file LICENSE
URL: https://reprex.tidyverse.org, https://github.com/tidyverse/reprex
BugReports: https://github.com/tidyverse/reprex/issues
Depends: R (>= 3.6)
Imports: callr (>= 3.6.0), cli (>= 3.2.0), clipr (>= 0.4.0), fs, glue,
        knitr (>= 1.23), lifecycle, rlang (>= 1.0.0), rmarkdown,
        rstudioapi, utils, withr (>= 2.3.0)
Suggests: covr, fortunes, miniUI, rprojroot, sessioninfo, shiny,
        spelling, styler (>= 1.2.0), testthat (>= 3.2.1)
VignetteBuilder: knitr
Config/Needs/website: dplyr, tidyverse/tidytemplate
Config/testthat/edition: 3
Config/testthat/parallel: TRUE
Config/testthat/start-first: knitr-options, venues, reprex
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.2
SystemRequirements: pandoc (>= 2.0) - https://pandoc.org/
NeedsCompilation: no
Packaged: 2024-07-06 00:20:55 UTC; jenny
Author: Jennifer Bryan [aut, cre] (<https://orcid.org/0000-0002-6983-2759>),
  Jim Hester [aut] (<https://orcid.org/0000-0002-2739-7082>),
  David Robinson [aut],
  Hadley Wickham [aut] (<https://orcid.org/0000-0003-4757-117X>),
  Christophe Dervieux [aut] (<https://orcid.org/0000-0003-4474-2498>),
  Posit Software, PBC [cph, fnd]
Maintainer: Jennifer Bryan <jenny@posit.co>
Repository: CRAN
Date/Publication: 2024-07-06 13:50:01 UTC
Built: R 4.6.0; ; 2026-04-28 13:42:58 UTC; unix

# Generated by roxygen2: do not edit by hand

export(reprex)
export(reprex_addin)
export(reprex_clean)
export(reprex_document)
export(reprex_html)
export(reprex_invert)
export(reprex_locale)
export(reprex_r)
export(reprex_render)
export(reprex_rescue)
export(reprex_rtf)
export(reprex_selection)
export(reprex_slack)
import(fs)
import(rlang)
importFrom(glue,glue)
importFrom(glue,glue_collapse)
importFrom(lifecycle,deprecated)

# reprex <a href="https://reprex.tidyverse.org"><img src="man/figures/logo.png" align="right" height="138" alt = ""/></a>

<!-- badges: start -->
[![CRAN\_Status\_Badge](https://www.r-pkg.org/badges/version/reprex)](https://cran.r-project.org/package=reprex)
[![R-CMD-check](https://github.com/tidyverse/reprex/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/tidyverse/reprex/actions/workflows/R-CMD-check.yaml)
[![Codecov test coverage](https://codecov.io/gh/tidyverse/reprex/branch/main/graph/badge.svg)](https://app.codecov.io/gh/tidyverse/reprex?branch=main)
<!-- badges: end -->

## Overview

Prepare reprexes for posting to [GitHub
issues](https://docs.github.com/issues/tracking-your-work-with-issues/about-issues),
[StackOverflow](https://stackoverflow.com/questions/tagged/r), in Slack [messages](https://slack.com/intl/en-ca/help/articles/201457107-Send-and-read-messages) or [snippets](https://slack.com/intl/en-ca/help/articles/204145658-Create-a-snippet), or even to paste into PowerPoint or Keynote slides.
What is a `reprex`? It’s a **repr**oducible **ex**ample, as coined by
Romain Francois in a tweet from 2014.

<a href="https://media.giphy.com/media/fdLR6LGwAiVNhGQNvf/giphy.gif"><img src="man/figures/help-me-help-you.png" align="right" /></a>

Given R code on the clipboard, selected in RStudio, as an expression
(quoted or not), or in a file …

  - run it via `rmarkdown::render()`,
  - with deliberate choices re: `render()` arguments, knitr options, and
    Pandoc options.

Get resulting runnable code + output as

  - Markdown, suitable for GitHub or Stack Overflow or Slack, or as
  - R code, augmented with commented output, or as
  - Plain HTML or (experimental) Rich Text

The result is returned invisibly, written to a file and, if possible, placed on the clipboard.
Preview an HTML version in RStudio viewer or default browser.

## Installation

Install from CRAN:

``` r
install.packages("reprex")
```

or get a development version from GitHub:

``` r
# install.packages("pak")
pak::pak("tidyverse/reprex")
```

On Linux, you probably want to install
[xclip](https://github.com/astrand/xclip) or
[xsel](https://www.vergenet.net/~conrad/software/xsel/), so reprex can
access the X11 clipboard. This is 'nice to have', but not mandatory. The
usual `sudo apt-get install` or `sudo yum install` installation methods
should work for both xclip and xsel.

## Usage

Let’s say you copy this code onto your clipboard (or, on RStudio Server or Cloud, select it):

    (y <- 1:4)
    mean(y)

Then call `reprex()`, where the default target venue is GitHub:

``` r
reprex()
```

A nicely rendered HTML preview will display in RStudio's Viewer (if
you’re in RStudio) or your default browser otherwise.

![](man/figures/README-viewer-screenshot.png)

The relevant bit of GitHub-flavored Markdown is ready to be pasted from
your clipboard (on RStudio Server or Cloud, you will need to copy this yourself):

    ``` r
    (y <- 1:4)
    #> [1] 1 2 3 4
    mean(y)
    #> [1] 2.5
    ```

Here’s what that Markdown would look like rendered in a GitHub issue:

``` r
(y <- 1:4)
#> [1] 1 2 3 4
mean(y)
#> [1] 2.5
```

Anyone else can copy, paste, and run this immediately.

In addition to GitHub, this Markdown also works on Stack Overflow and Discourse. Those venues can be formally requested via `venue = "so"` and `venue = "ds"`, but they are just aliases for `venue = "gh"`.

Instead of reading from the clipboard, you can:

  - `reprex(mean(rnorm(10)))` to get code from expression.

  - `reprex(input = "mean(rnorm(10))\n")` gets code from character
    vector (detected via length or terminating newline). Leading prompts
    are stripped from input source: `reprex(input = "> median(1:3)\n")`
    produces same output as `reprex(input = "median(1:3)\n")`

  - `reprex(input = "my_reprex.R")` gets code from file

  - Use one of the RStudio add-ins to use the selected text or current
    file.

But wait, there’s more\!

  - Get slightly different Markdown, optimized for Slack messages, with
    `reprex(..., venue = "slack")`.

  - Get a runnable R script, augmented with commented output, with
    `reprex(..., venue = "R")`. This is useful for Slack code snippets, email,
    etc.

  - Get html with `reprex(..., venue = "html")`. Useful for sites that don't
    support Markdown.

  - Prepare (un)rendered, syntax-highlighted code snippets to paste into
    Keynote or PowerPoint, with `reprex(..., venue = "rtf")`. This
    feature is still experimental; see the [associated article](https://reprex.tidyverse.org/articles/articles/rtf.html) for more.

  - By default, figures are uploaded to [imgur.com](https://imgur.com/)
    and the resulting URL is dropped into an inline image tag.

  - If you really need to reprex in a specific directory, use the `wd`
    argument. For example, `reprex(wd = ".")` requests the current
    working directory.
    
  - Append session info via `reprex(..., session_info = TRUE)`.

  - Get clean, runnable code from wild-caught reprexes with
    
      - `reprex_invert()` = the opposite of `reprex()`
      - `reprex_clean()`, e.g. when you copy/paste from GitHub or Stack
        Overflow
      - `reprex_rescue()`, when you’re dealing with copy/paste from R
        Console

#' No user-supplied code found ... so we've made some up. You're welcome!

#+ fortunes, include = requireNamespace("fortunes", quietly = TRUE), eval = requireNamespace("fortunes", quietly = TRUE)
fortunes::fortune()

#+ no-fortunes, include = !requireNamespace("fortunes", quietly = TRUE)
sprintf("Happy %s!", weekdays(Sys.Date()))

# nocov start

#' Render a "prex"
#'
#' @description
#' `prex()` is like [reprex()], but much less reproducible!
#'   * Code is evaluated in the global environment of the current R session.
#'   * Current working directory is used.
#'   * `advertise = FALSE` is the default.
#' This violates many principles of a true reprex, which is why the `prex()`
#' family of functions is unexported. The main motivation is to make
#' `prex_rtf()` available for preparing code snippets that are scattered around
#' a talk and that can't necessarily be self-contained.
#'
#' Specific `reprex()` arguments do not appear as `prex()` arguments or may have
#' different capabilities (i.e., more limited).
#' * `std_out_err`: not offered in `prex()`
#'
#' @noRd
#' @keywords internal
#'
#' @examples
#' # compare and contrast to get a feel for reprex() vs prex()
#' reprex(ls())     # character(0)
#' prex(ls())       # whatever is lying around your current workspace
#'
#' reprex(search()) # won't reflect anything you attach in .Rprofile
#' prex(search())   # generally does reflect packages attached in .Rprofile
#'
#' reprex(getwd())  # a reprex directory below session temp directory
#' prex(getwd())    # current working directory
prex <- function(x = NULL,
                 input = NULL,
                 venue = c("gh", "r", "rtf", "html", "slack", "so", "ds"),

                 render = TRUE,

                 advertise       = FALSE,       # <-- different from reprex
                 session_info    = opt(FALSE),
                 style           = opt(FALSE),
                 html_preview    = opt(TRUE),
                 comment         = opt("#>"),
                 tidyverse_quiet = opt(TRUE)) {
  reprex_impl(
    x_expr = substitute(x),
    input = input,
    wd = ".",                                   # <-- different from reprex
    venue = venue,

    render = render,
    new_session = FALSE,                        # <-- different from reprex()

    advertise       = advertise,
    session_info    = session_info,
    style           = style,
    comment         = comment,
    tidyverse_quiet = tidyverse_quiet,
    std_out_err     = FALSE,                    # <-- different from reprex()
    html_preview    = html_preview
  )
}

prex_html  <- function(...) prex(..., venue = "html")
prex_r     <- function(...) prex(..., venue = "r")
prex_rtf   <- function(...) prex(..., venue = "rtf")
prex_slack <- function(...) prex(..., venue = "slack")

# these should exist for completeness, but I predict they'd never get used and
# they just clutter the auto-complete landscape
# prex_gh   <- function(...) prex(..., venue = "gh")
# prex_so   <- function(...) prex(..., venue = "so")
# prex_ds   <- function(...) prex(..., venue = "ds")

# nocov end

#' reprex output format
#'
#' @description
#' This is an R Markdown output format designed specifically for making
#' "reprexes", typically created via the [reprex()] function, which ultimately
#' renders the document with [reprex_render()]. It is a heavily modified version
#' of [rmarkdown::md_document()]. The arguments have different spheres of
#' influence:
#'   * `venue` potentially affects input preparation and [reprex_render()].
#'   * Add content to the primary input, prior to rendering:
#'     - `advertise`
#'     - `session_info`
#'     - `std_out_err` (also consulted by [reprex_render()])
#'   * Influence knitr package or chunk options:
#'     - `style`
#'     - `comment`
#'     - `tidyverse_quiet`
#'
#' RStudio users can create new R Markdown documents with the
#' `reprex_document()` format using built-in templates. Do
#' *File > New File > R Markdown ... > From Template* and choose one of:
#'   * reprex (minimal)
#'   * reprex (lots of features)
#'
#' Both include `knit: reprex::reprex_render` in the YAML, which causes the
#' RStudio "Knit" button to use `reprex_render()`. If you render these documents
#' yourself, you should do same.
#'
#' @inheritParams reprex
#' @inheritParams rmarkdown::md_document
#' @return An R Markdown output format to pass to [rmarkdown::render()].
#' @export
#' @examples
#' reprex_document()
reprex_document <- function(venue = c("gh", "r", "rtf", "html", "slack", "so", "ds"),

                            advertise       = NULL,
                            session_info    = opt(FALSE),
                            style           = opt(FALSE),
                            comment         = opt("#>"),
                            tidyverse_quiet = opt(TRUE),
                            std_out_err     = opt(FALSE),
                            pandoc_args = NULL) {
  venue <- tolower(venue)
  venue <- match.arg(venue)
  venue <- normalize_venue(venue)

  advertise       <- set_advertise(advertise, venue)
  session_info    <- arg_option(session_info)
  style           <- arg_option(style)
  style           <- style_requires_styler(style)
  comment         <- arg_option(comment)
  tidyverse_quiet <- arg_option(tidyverse_quiet)
  std_out_err     <- arg_option(std_out_err)

  stopifnot(is_bool(advertise), is_bool(session_info), is_bool(style))
  stopifnot(is.character(comment))
  stopifnot(is_bool(tidyverse_quiet), is_bool(std_out_err))

  opts_chunk <- list(
    # fixed defaults
    collapse = TRUE, error = TRUE,
    # explicitly exposed for user configuration
    comment = comment,
    R.options = list(
      tidyverse.quiet = tidyverse_quiet,
      tidymodels.quiet = tidyverse_quiet
    )
  )
  if (isTRUE(style)) {
    opts_chunk[["tidy"]] <- "styler"
  }
  opts_knit <- list(
    upload.fun = switch(venue,
      r = identity,
      knitr::imgur_upload
    )
  )

  pandoc_args <- c(
    pandoc_args,
    if (rmarkdown::pandoc_available()) "--wrap=preserve"
  )

  pre_knit <- function(input, ...) {

    # I don't know why the pre_knit hook operates on the **original** input
    # instead of the to-be-knitted (post-spinning) input, but I need to
    # operate on the latter. So I brute force the correct path.
    # This is a no-op if input starts as `.Rmd`.
    knit_input <- sub("[.]R$", ".spin.Rmd", input)
    input_lines <- read_lines(knit_input)

    input_lines <- c(rprofile_alert(venue), "", input_lines)
    input_lines <- c(reprex_opts(venue), "", input_lines)

    if (isTRUE(advertise)) {
      input_lines <- c(input_lines, "", ad(venue))
    }

    if (isTRUE(std_out_err)) {
      input_lines <- c(input_lines, "", std_out_err_stub(input, venue))
    }

    if (isTRUE(session_info)) {
      input_lines <- c(input_lines, "", si(venue))
    }

    write_lines(input_lines, knit_input)
  }

  format <- rmarkdown::output_format(
    knitr = rmarkdown::knitr_options(
      opts_knit = opts_knit,
      opts_chunk = opts_chunk
    ),
    pandoc = rmarkdown::pandoc_options(
      # https://github.com/tidyverse/reprex/issues/375
      to = paste0("gfm", if (rmarkdown::pandoc_available("2.13")) "-yaml_metadata_block"),
      from = rmarkdown::from_rmarkdown(implicit_figures = FALSE),
      ext = ".md",
      args = pandoc_args
    ),
    clean_supporting = FALSE,
    pre_knit = pre_knit,
    base_format = rmarkdown::md_document()
  )
  format
}

reprex_opts <- function(venue = "gh") {
  string <- glue('
    ```{{r reprex-options, include = FALSE}}
    options(
      keep.source = TRUE,
      rlang_backtrace_on_error_report = "full",
      crayon.enabled = FALSE,
      reprex.current_venue = "{venue}"
    )
    ```')
}

rprofile_alert <- function(venue = "gh") {
  if (venue %in% c("gh", "html", "slack")) {
    fmt <- '"*Local `.Rprofile` detected at `%s`*"'
  } else { # venue %in% c("r", "rtf")
    fmt <- '"Local .Rprofile detected at %s"'
  }
  include_eval <-
    "include = file.exists('.Rprofile'), eval = file.exists('.Rprofile')"

  c(
    glue("```{{r, results = 'asis', echo = FALSE, {include_eval}}}"),
    glue('cat(sprintf({fmt}, normalizePath(".Rprofile")))'),
    "```"
  )
}

ad <- function(venue = "gh") {
  markdown_ad <- glue('
    Created on `r Sys.Date()` with \\
    [reprex v`r utils::packageVersion("reprex")`]\\
    (https://reprex.tidyverse.org)')
  if (venue %in% c("gh", "html")) {
    glue('<sup>{markdown_ad}</sup>')
  } else if (venue == "slack") {
    markdown_ad
  } else { # venue %in% c("r", "rtf")
    glue('
      Created on `r Sys.Date()` with reprex \\
      v`r utils::packageVersion("reprex")` https://reprex.tidyverse.org')
  }
}

std_out_err_stub <- function(input, venue = "gh") {
  txt <- backtick(std_file(input))
  if (venue %in% c("gh", "html")) {
    details(txt, desc = "Standard output and standard error")
  } else { # venue %in% c("r", "rtf", "slack")
    c("#### Standard output and error", txt)
  }
}

si <- function(venue = "gh") {
  txt <- r_chunk(session_info_string())
  if (venue %in% c("gh", "html")) {
    details(txt, "Session info")
  } else { # venue %in% c("r", "rtf", "slack")
    txt
  }
}

session_info_string <- function() {
  if (rlang::is_installed("sessioninfo")) {
    "sessioninfo::session_info()"
  } else {
    "sessionInfo()"
  }
}

test_that("upload.fun responds to venue", {
  x <- reprex_document(venue = "gh")
  expect_identical(x$knitr$opts_knit$upload.fun, knitr::imgur_upload)
  x <- reprex_document(venue = "r")
  expect_identical(x$knitr$opts_knit$upload.fun, identity)
})

test_that("ad responds to venue", {
  expect_snapshot(ad("gh"))
  expect_snapshot(ad("slack"))
  expect_snapshot(ad("r"))
})

reprex_impl <- function(x_expr = NULL,
                        input  = NULL,
                        wd     = NULL,
                        venue  = c("gh", "r", "rtf", "html", "slack", "so", "ds"),

                        render = TRUE,
                        new_session = TRUE,

                        advertise       = NULL,
                        session_info    = opt(FALSE),
                        style           = opt(FALSE),
                        comment         = opt("#>"),
                        tidyverse_quiet = opt(TRUE),
                        std_out_err     = opt(FALSE),
                        html_preview    = opt(TRUE),

                        outfile = deprecated()) {

  venue <- tolower(venue)
  venue <- match.arg(venue)
  venue <- normalize_venue(venue)

  advertise       <- set_advertise(advertise, venue)
  session_info    <- arg_option(session_info)
  style           <- arg_option(style)
  style           <- style_requires_styler(style)
  html_preview    <- arg_option(html_preview)
  html_preview    <- html_preview_requires_interactive(html_preview)
  comment         <- arg_option(comment)
  tidyverse_quiet <- arg_option(tidyverse_quiet)
  std_out_err     <- arg_option(std_out_err)

  if (!is.null(input)) stopifnot(is.character(input))
  if (!is.null(wd)) stopifnot(is_string(wd))
  stopifnot(is_bool(advertise), is_bool(session_info), is_bool(style))
  stopifnot(is_bool(html_preview), is_bool(render))
  stopifnot(is.character(comment))
  stopifnot(is_bool(tidyverse_quiet), is_bool(std_out_err))

  if (lifecycle::is_present(outfile)) {
    stopifnot(is.character(outfile) || is.na(outfile))
  }

  where <- if (is.null(x_expr)) locate_input(input) else "expr"
  src <- switch(where,
    expr      = stringify_expression(x_expr),
    clipboard = ingest_clipboard(),
    path      = read_lines(input),
    input     = escape_newlines(sub("\n$", "", enc2utf8(input))),
    selection = rstudio_selection(),
    NULL
  )
  src <- ensure_not_empty(src)
  src <- ensure_not_dogfood(src)
  src <- ensure_no_prompts(src)

  reprex_files <- plan_files(
    infile = if (where == "path") input else NULL,
    # the else branch makes prex_*() functions write files below temp dir
    wd = if (new_session) wd else NULL,
    outfile = outfile
  )

  r_file <- r_file(reprex_files$filebase)
  if (would_clobber(r_file)) {
    return(invisible())
  }

  reprex_document_options <- list(
    venue = venue,
    advertise = advertise, session_info = session_info,
    style = style, comment = comment,
    tidyverse_quiet = tidyverse_quiet, std_out_err = std_out_err
  )
  src <- c(yamlify(reprex_document_options), "", src)
  if (reprex_files$chatty) {
    reprex_path("Preparing reprex as {.code .R} file:", r_file)
  }
  write_lines(src, r_file)

  if (!render) {
    return(invisible(read_lines(r_file)))
  }

  local_rprofile <- path(path_dir(path_real(r_file)), ".Rprofile")
  if (file_exists(local_rprofile)) {
    reprex_path(
      "Local {.path .Rprofile} detected in reprex directory:",
      local_rprofile,
      type = "warning"
    )
  }

  reprex_info("Rendering reprex...")
  reprex_file <- reprex_render_impl(
    r_file,
    new_session = new_session, html_preview = html_preview
  )

  if (reprex_files$chatty) {
    # TODO: be smarter about when to report full path vs. relative,
    # i.e., consider if the user provided any path info
    reprex_path("Writing reprex file:", reprex_file)
  }
  expose_reprex_output(reprex_file, rtf = (venue == "rtf"))
  invisible(read_lines(reprex_file))
}

advertise_default <- function(venue) {
  default <- c(
    gh    = TRUE,
    ds    = TRUE,
    html  = TRUE,
    so    = TRUE,
    r     = FALSE,
    rtf   = FALSE,
    slack = FALSE
  )
  default[[venue]]
}

set_advertise <- function(advertise, venue) {
  advertise %||%
    getOption("reprex.advertise") %||%
    advertise_default(venue)
}

style_requires_styler <- function(style) {
  if (!requireNamespace("styler", quietly = TRUE)) {
    reprex_danger("
      Install the {.pkg styler} package in order to use
      {.code style = TRUE}.")
    style <- FALSE
  }
  invisible(style)
}

html_preview_requires_interactive <- function(html_preview) {
  if (html_preview && !is_interactive()) {
    reprex_info(
      "Non-interactive session, setting {.code html_preview = FALSE}."
    )
    html_preview <- FALSE
  }
  invisible(html_preview)
}

# re-express reprex() args as yaml for the reprex_document() format ----
yamlify <- function(x) {
  x <- remove_defaults(x)
  if (length(x) < 1) {
    return(decorate_yaml("output: reprex::reprex_document"))
  }
  lines <- c(
    "output:",
    "  reprex::reprex_document:",
    paste0("    ", nv(x))
  )
  decorate_yaml(lines)
}

decorate_yaml <- function(x) roxygen_comment(x <- c("---", x, "---"))

remove_defaults <- function(x) {
  defaults <- list(
    venue           = "gh",
    # this is the only conditional default, i.e. that depends on venue
    advertise       = advertise_default(x[["venue"]]),
    session_info    = FALSE,
    style           = FALSE,
    comment         = "#>",
    tidyverse_quiet = TRUE,
    std_out_err     = FALSE
  )

  compare_one <- function(nm) identical(x[[nm]], defaults[[nm]])
  is_default <- vapply(names(x), compare_one, logical(1))

  novel_names <- setdiff(names(x), names(defaults))
  if (length(novel_names) > 0) {
    reprex_danger("
      {?This/These} parameter{?s} {?is/are} not recognized for the
      {.fun reprex_document} format: {.code {novel_names}}.")
  }

  x[!is_default]
}

nv <- function(x) {
  is_character <- vapply(x, is.character, logical(1))
  # dQuote didn't gain the `q` argument until R 3.6
  withr::local_options(list(useFancyQuotes = FALSE))
  x[is_character] <- vapply(x[is_character], dQuote, character(1))
  glue("{name}: {value}", name = names(x), value = x)
}

test_that("can't see environment of caller", {
  skip_on_cran()
  z <- "don't touch me"
  ret <- reprex(z)
  expect_match(ret, "object 'z' not found", all = FALSE)
})

test_that("reprex doesn't write into environment of caller", {
  skip_on_cran()
  z <- "don't touch me"
  ret <- reprex((z <- "I touched it!"), advertise = FALSE)
  expect_identical(ret[3], "#> [1] \"I touched it!\"")
  expect_identical(z, "don't touch me")

  ## concrete example I have suffered from:
  ## assign object to name of object inside reprex_impl()
  expect_match(reprex(r_file <- 0L), "r_file <- 0L", all = FALSE)
})

test_that("reprex env doesn't bear traces of reprex or its dependencies", {
  skip_on_cran()

  ret <- reprex(input = c("a <- 'a'", "ls(all.names = TRUE)"))
  ret <- ret[grepl("^#>", ret)]

  # https://github.com/r-lib/debugme/issues/50
  # styler --> tibble --> pillar --> debugme --> tickles RNG
  pkg <- "debugme"
  if (requireNamespace(pkg, quietly = TRUE)) {
    # until debugme updates on CRAN, let's tolerate .Random.seed, but not
    # require it either
    expect_match(ret, '"a"', all = FALSE)
  } else {
    expect_identical(ret, "#> [1] \"a\"")
  }
})

# https://github.com/tidyverse/reprex/issues/363
# ironically, reprex.advertise = FALSE + reprex(advertise = FALSE)
# resulted in reprex(advertise = TRUE) behaviour
test_that("reprex.advertise default detection isn't affected by the option", {
  full_list   <- list(advertise = FALSE, venue = "gh", session_info = FALSE)
  non_default <- list(advertise = FALSE)

  withr::with_options(
    list(reprex.advertise = TRUE),
    expect_equal(remove_defaults(full_list), non_default)
  )
  withr::with_options(
    list(reprex.advertise = FALSE),
    expect_equal(remove_defaults(full_list), non_default)
  )
  withr::with_options(
    list(reprex.advertise = NULL),
    expect_equal(remove_defaults(full_list), non_default)
  )
})

ingest_clipboard <- function() {
  if (reprex_clipboard()) {
    return(suppressWarnings(
      enc2utf8(clipr::read_clip() %||% character())
    ))
  }
  reprex_warning("No input provided and clipboard is not available.")
  character()
}

write_clip_windows_rtf <- function(path) {
  path <- gsub("\\s", "` ", path)
  cmd <- glue('
    powershell -Command "\\
    Add-Type -AssemblyName System.Windows.Forms | Out-Null;\\
    [Windows.Forms.Clipboard]::SetText(
    (Get-Content -Raw {path}),\\
    [Windows.Forms.TextDataFormat]::Rtf
    )"')
  res <- system(cmd)
  if (res > 0) {
    # abort("Failed to put RTF on the Windows clipboard")
    reprex_danger("Failed to put RTF on the Windows clipboard :(")
    invisible(FALSE)
  } else {
    invisible(TRUE)
  }
}

# reports clipr::clipr_available(), with some exceptions and niceties
# - if we detect RStudio server, hard FALSE
# - otherwise, if clipr_available() reports FALSE, call dr_clipr() ONCE
# - use an option to persist this finding in current session
reprex_clipboard <- function() {
  x <- getOption("reprex.clipboard", NA)
  if (length(x) != 1 || !is.logical(x)) {
    cli::cli_abort("
      The {.arg reprex.clipboard} option must be {.code TRUE}, {.code FALSE}, \\
      or (logical) {.code NA}.",
      call = NULL
    )
  }
  if (is_rstudio_server()) {
    x <- FALSE
    options(reprex.clipboard = x)
  }
  if (is.na(x)) {
    y <- clipr::clipr_available()
    if (isFALSE(y)) {
      clipr::dr_clipr()
    }
    options(reprex.clipboard = y)
  }
  getOption("reprex.clipboard")
}

test_that("reprex_clipboard() works", {
  withr::local_options(list(reprex.clipboard = FALSE))
  expect_false(reprex_clipboard())
})

test_that("reprex_clipboard() insists on length one logical", {
  withr::local_options(list(reprex.clipboard = function() "wut"))
  expect_snapshot(error = TRUE, reprex_clipboard())
})

test_that("ingest_clipboard() copes when clipboard not available", {
  withr::local_options(list(reprex.clipboard = FALSE))
  local_reprex_loud()
  msg <- capture_messages(
    out <- ingest_clipboard()
  )
  expect_equal(out, character())
  expect_match(msg, "clipboard is not available")
})

interactive <- function(...) {
  cli::cli_abort("
    Inside {.pkg reprex}, we use {.fun rlang::is_interactive}, \\
    not {.fun interactive}, for mocking reasons.",
    .internal = TRUE
  )
}

## returns TRUE if user says "no"
##         FALSE otherwise
nope <- function(..., yes = "yes", no = "no") {
  if (is_interactive()) {
    cat(paste0(..., collapse = ""))
    return(utils::menu(c(yes, no)) == 2)
  }
  FALSE
}

## returns TRUE if user says "yes"
##         FALSE otherwise
yep <- function(..., yes = "yes", no = "no") {
  if (is_interactive()) {
    cat(paste0(..., collapse = ""))
    return(utils::menu(c(yes, no)) == 1)
  }
  FALSE
}

test_that("locate_input() works", {
  with_mock(
    reprex_clipboard = function() TRUE,
    {
      expect_identical("clipboard", locate_input(NULL))
    }
  )
  with_mock(
    reprex_clipboard = function() FALSE,
    in_rstudio = function() TRUE,
    {
      expect_identical("selection", locate_input(NULL))
    }
  )
  with_mock(
    reprex_clipboard = function() FALSE,
    in_rstudio = function() FALSE,
    {
      expect_null(locate_input(NULL))
    }
  )
  expect_identical("path", locate_input(path_temp()))
  expect_identical("input", locate_input(c("a", "b")))
  expect_identical("input", locate_input("a\n"))
})

test_that("nope() defaults to 'yes' if user not available", {
  expect_false(nope())
})

test_that("yep() defaults to 'no' if user not available", {
  expect_false(yep())
})

# nocov start

#' Venue-specific shortcuts
#'
#' These are thin wrappers around `reprex()` that incorporate the target `venue`
#' as a suffix in the function name, for easier access via auto-completion.
#'
#' @param ... Passed along to [reprex()].
#'
#' @name reprex_venue
NULL

#' @export
#' @rdname reprex_venue
reprex_html <- function(...) reprex(..., venue = "html")

#' @export
#' @rdname reprex_venue
reprex_r <- function(...) reprex(..., venue = "r")

#' @export
#' @rdname reprex_venue
reprex_rtf <- function(...) reprex(..., venue = "rtf")

#' @export
#' @rdname reprex_venue
reprex_slack <- function(...) reprex(..., venue = "slack")

# these should exist for completeness, but I predict they'd never get used and
# they just clutter the auto-complete landscape
# reprex_gh <- function(...) reprex(..., venue = "gh")
# reprex_so <- function(...) reprex(..., venue = "so")
# reprex_ds <- function(...) reprex(..., venue = "ds")

# nocov ends

normalize_venue <- function(venue) {
  venue <- ds_is_gh(venue)
  venue <- so_is_gh(venue)
  venue <- rtf_requires_highlight(venue)
  venue
}

ds_is_gh <- function(venue) {
  if (venue == "ds") {
    reprex_info('
      The Discourse venue "ds" is an alias for the default GitHub venue "gh".
      There is no need to specify the venue.')
    venue <- "gh"
  }
  venue
}

so_is_gh <- function(venue) {
  if (venue == "so") {
    reprex_info('
      The Stack Overflow venue "so" is an alias for the default GitHub venue
      "gh". There is no need to specify the venue.')
    venue <- "gh"
  }
  venue
}

#' Render a reprex
#'
#' @description
#' Run a bit of R code using [rmarkdown::render()] and write the rendered result
#' to user's clipboard. If the clipboard is unavailable, the file containing
#' the rendered result is opened for manual copy. The goal is to make it easy to
#' share a small reproducible example ("reprex"), e.g., in a GitHub issue.
#' Reprex source can be
#'
#' * read from clipboard
#' * provided directly as expression, character vector, or string
#' * read from file
#' * read from current selection or active document in RStudio
#'
#' reprex can also be used for syntax highlighting (with or without rendering);
#' see below for more.
#'
#' @section Details:
#' The usual "code + commented output" is returned invisibly, written to file,
#' and, whenever possible, put on the clipboard. An HTML preview displays in
#' RStudio's Viewer pane, if available, or in the default browser, otherwise.
#' Leading `"> "` prompts, are stripped from the input code. Read more at
#' <https://reprex.tidyverse.org/>.
#'
#' reprex sets specific [knitr options](https://yihui.org/knitr/options/):
#' * Chunk options default to `collapse = TRUE`, `comment = "#>"`,
#'   `error = TRUE`. Note that `error = TRUE`, because a common use case is bug
#'   reporting.
#' * reprex also sets knitr's `upload.fun`. It defaults to
#'   [knitr::imgur_upload()] so figures produced by the reprex appear properly
#'   on GitHub, Stack Overflow, Discourse, and Slack. Note that `imgur_upload()`
#'   requires the packages httr and xml2. When `venue = "r"`, `upload.fun` is
#'   set to `identity()`, so that figures remain local. In that case, you may
#'   also want to provide a filepath to `input` or set `wd`, to control where
#'   the reprex files are written.
#' You can supplement or override these options with special comments in your
#' code (see examples).
#'

#'
#' @section Error backtraces:
#'
#' reprex sets the rlang option `rlang_backtrace_on_error_report = "full"`.
#' Combined with the knitr option `error = TRUE`, this means rlang errors are
#' displayed with a full backtrace. This basically eliminates the need to call
#' [rlang::last_error()] or [rlang::last_trace()] explicitly, although these
#' functions can be used in a reprex.
#'

#' Insert a line containing the special comment `#'` in between the
#' error-causing code and the `last_error()` or `last_trace()` call, to fulfill
#' the requirement of being in separate chunks:

#' ``` r
#' f <- function() rlang::abort('foo')
#' f()
#' #'
#' rlang::last_error()
#' rlang::last_trace()
#' ```
#'

#' Read more in rlang's documentation: [Errors in
#' RMarkdown](https://rlang.r-lib.org/reference/rlang_backtrace_on_error.html#errors-in-rmarkdown).

#'
#' @section Syntax highlighting:
#'
#' `r lifecycle::badge("experimental")`
#'
#' A secondary use case for reprex is to produce syntax highlighted code
#' snippets, with or without rendering, to paste into applications like
#' Microsoft Word, PowerPoint, or Keynote. Use `venue = "rtf"` for this.
#'
#' This feature is experimental and requires the installation of the
#' [highlight](http://www.andre-simon.de/doku/highlight/en/highlight.php)
#' command line tool. The `"rtf"` venue is documented in [its own
#' article](https://reprex.tidyverse.org/articles/articles/rtf.html)
#'
#' @param x An expression. If not given, `reprex()` looks for code in
#'   `input`. If `input` is not provided, `reprex()` looks on the clipboard.
#'
#'   When the clipboard is structurally unavailable, e.g., on RStudio Server or
#'   RStudio Cloud, `reprex()` consults the current selection instead of the
#'   clipboard.
#' @param input Character. If has length one and lacks a terminating newline,
#'   interpreted as the path to a file containing reprex code. Otherwise,
#'   assumed to hold reprex code as character vector. When `input` specifies a
#'   filepath, it also determines the reprex working directory and the location
#'   of all resulting files.
#' @param wd An optional filepath that is consulted when `input` is not a
#'   filepath. (By default, all work is done, quietly, in a subdirectory of the
#'   session temp directory.)
#'
#'   The most common use of `wd` is to set `wd = "."`, which means "reprex right
#'   HERE in the current working directory". Do this if you really must
#'   demonstrate something with local files.
#' @param venue Character. Must be one of the following (case insensitive):
#' * "gh" for [GitHub-Flavored Markdown](https://github.github.com/gfm/), the
#'   default
#' * "r" for a runnable R script, with commented output interleaved. Also useful
#'   for [Slack code snippets](https://slack.com/intl/en-ca/slack-tips/share-code-snippets);
#'   select "R" from the "Type" drop-down menu to enjoy nice syntax
#'   highlighting.
#' * "rtf" for
#'   [Rich Text Format](https://en.wikipedia.org/wiki/Rich_Text_Format)
#'   (not supported for un-reprexing)
#' * "html" for an HTML fragment suitable for inclusion in a larger HTML
#'   document (not supported for un-reprexing)
#' * "slack" for pasting into a Slack message. Optimized for people who opt out
#'   of Slack's WYSIWYG interface. Go to
#'   **Preferences > Advanced > Input options** and select "Format messages with
#'   markup". (If there is demand for a second Slack venue optimized for use
#'   with WYSIWYG, please open an issue to discuss.)
#' * "so" for
#'   [Stack Overflow Markdown](https://stackoverflow.com/editing-help#syntax-highlighting).
#'   Note: this is just an alias for "gh", since Stack Overflow started to
#'   support CommonMark-style fenced code blocks in January 2019.
#' * "ds" for Discourse, e.g.,
#'   [community.rstudio.com](https://community.rstudio.com). Note: this is
#'   currently just an alias for "gh".
#' @param advertise Logical. Whether to include a footer that describes when and
#'   how the reprex was created. If unspecified, the option `reprex.advertise`
#'   is consulted and, if that is not defined, default is `TRUE` for venues
#'   `"gh"`, `"html"`, `"so"`, `"ds"` and `FALSE` for `"r"`, `"rtf"`, `"slack"`.
#' @param session_info Logical. Whether to include
#'   [sessioninfo::session_info()], if available, or [sessionInfo()] at the end
#'   of the reprex. When `venue` is "gh", the session info is wrapped in a
#'   collapsible details tag. Read more about [opt()].
#' @param style Logical. Whether to set the knitr chunk option `tidy =
#'   "styler"`, which re-styles code with the [styler
#'   package](https://styler.r-lib.org). Read more about [opt()].
#' @param comment Character. Prefix with which to comment out output, defaults
#'   to `"#>"`. Read more about [opt()].
#' @param render Logical. Whether to call [rmarkdown::render()] on the templated
#'   reprex, i.e. whether to actually run the code. Defaults to `TRUE`. Exists
#'   primarily for the sake of internal testing.
#' @param tidyverse_quiet Logical. Sets the options `tidyverse.quiet` and
#'   `tidymodels.quiet`, which suppress (`TRUE`, the default) or include
#'   (`FALSE`) the startup messages for the tidyverse and tidymodels packages.
#'   Read more about [opt()].
#' @param std_out_err Logical. Whether to append a section for output sent to
#'   stdout and stderr by the reprex rendering process. This can be necessary to
#'   reveal output if the reprex spawns child processes or `system()` calls.
#'   Note this cannot be properly interleaved with output from the main R
#'   process, nor is there any guarantee that the lines from standard output and
#'   standard error are in correct chronological order. See [callr::r()] for
#'   more. Read more about [opt()].
#' @param html_preview Logical. Whether to show rendered output in a viewer
#'   (RStudio or browser). Always `FALSE` in a noninteractive session. Read more
#'   about [opt()].
#' @param outfile `r lifecycle::badge("deprecated")` in favor of `wd` or
#'   providing a filepath to `input`. To reprex in current working directory,
#'   use `wd = "."` now, instead of `outfile = NA`.
#' @param show `r lifecycle::badge("deprecated")` in favor of `html_preview`,
#'   for greater consistency with other R Markdown output formats.
#' @param si `r lifecycle::badge("deprecated")` in favor of `session_info`.
#'
#' @return Character vector of rendered reprex, invisibly.
#' @examples
#' \dontrun{
#' # put some code like this on the clipboard
#' # (y <- 1:4)
#' # mean(y)
#' reprex()
#'
#' # provide code as an expression
#' reprex(rbinom(3, size = 10, prob = 0.5))
#' reprex({y <- 1:4; mean(y)})
#' reprex({y <- 1:4; mean(y)}, style = TRUE)
#'
#' # note that you can include newlines in those brackets
#' # in fact, that is often a good idea
#' reprex({
#'   x <- 1:4
#'   y <- 2:5
#'   x + y
#' })
#'
#' ## provide code via character vector
#' reprex(input = c("x <- 1:4", "y <- 2:5", "x + y"))
#'
#' ## if just one line, terminate with '\n'
#' reprex(input = "rnorm(3)\n")
#'
#' ## customize the output comment prefix
#' reprex(rbinom(3, size = 10, prob = 0.5), comment = "#;-)")
#'
#' # override a default chunk option
#' reprex({
#'   #+ setup, include = FALSE
#'   knitr::opts_chunk$set(collapse = FALSE)
#'
#'   #+ actual-reprex-code
#'   (y <- 1:4)
#'   median(y)
#' })
#'
#' # add prose, use general markdown formatting
#' reprex({
#'   #' # A Big Heading
#'   #'
#'   #' Look at my cute example. I love the
#'   #' [reprex](https://github.com/tidyverse/reprex#readme) package!
#'   y <- 1:4
#'   mean(y)
#' }, advertise = FALSE)
#'
#' # read reprex from file and write resulting files to that location
#' tmp <- file.path(tempdir(), "foofy.R")
#' writeLines(c("x <- 1:4", "mean(x)"), tmp)
#' reprex(input = tmp)
#' list.files(dirname(tmp), pattern = "foofy")
#'
#' # clean up
#' file.remove(list.files(dirname(tmp), pattern = "foofy", full.names = TRUE))
#'
#' # write reprex to file AND keep figure local too, i.e. don't post to imgur
#' tmp <- file.path(tempdir(), "foofy")
#' dir.create(tmp)
#' reprex({
#'   #+ setup, include = FALSE
#'   knitr::opts_knit$set(upload.fun = identity)
#'
#'   #+ actual-reprex-code
#'   #' Some prose
#'   ## regular comment
#'   (x <- 1:4)
#'   median(x)
#'   plot(x)
#'   }, wd = tmp)
#' list.files(dirname(tmp), pattern = "foofy")
#'
#' # clean up
#' unlink(tmp, recursive = TRUE)
#'
#' ## target venue = R, also good for email or Slack snippets
#' ret <- reprex({
#'   x <- 1:4
#'   y <- 2:5
#'   x + y
#' }, venue = "R")
#' ret
#'
#' ## target venue = html
#' ret <- reprex({
#'   x <- 1:4
#'   y <- 2:5
#'   x + y
#' }, venue = "html")
#' ret
#'
#' ## include prompt and don't comment the output
#' ## use this when you want to make your code hard to execute :)
#' reprex({
#'   #+ setup, include = FALSE
#'   knitr::opts_chunk$set(comment = NA, prompt = TRUE)
#'
#'   #+ actual-reprex-code
#'   x <- 1:4
#'   y <- 2:5
#'   x + y
#' })
#'
#' ## leading prompts are stripped from source
#' reprex(input = c("> x <- 1:3", "> median(x)"))
#' }
#' @export
reprex <- function(x = NULL,
                   input = NULL, wd = NULL,
                   venue = c("gh", "r", "rtf", "html", "slack", "so", "ds"),

                   render = TRUE,

                   advertise       = NULL,
                   session_info    = opt(FALSE),
                   style           = opt(FALSE),
                   comment         = opt("#>"),
                   tidyverse_quiet = opt(TRUE),
                   std_out_err     = opt(FALSE),
                   html_preview    = opt(TRUE),

                   outfile = deprecated(),
                   show = deprecated(),
                   si = deprecated()) {
  if (lifecycle::is_present(show)) {
    html_preview <- show
    lifecycle::deprecate_warn(
      when = "1.0.0",
      what = "reprex(show)",
      with = "reprex(html_preview)"
    )
  }
  if (lifecycle::is_present(si)) {
    session_info <- si
    # I kind of regret deprecating this, so let's not make a fuss
    # I won't throw a warning.
  }

  reprex_impl(
    x_expr = substitute(x),
    input = input,
    wd = wd,
    venue = venue,

    render = render,
    new_session = TRUE,

    advertise       = advertise,
    session_info    = session_info,
    style           = style,
    html_preview    = html_preview,
    comment         = comment,
    tidyverse_quiet = tidyverse_quiet,
    std_out_err     = std_out_err,

    outfile = outfile
  )
}

test_that("pandoc does not add hard linebreak in the ad", {
  skip_on_cran()
  out <- reprex(input = "1:3\n", venue = "gh", advertise = TRUE)
  expect_match(out[length(out)], "Created on")
})

test_that("pandoc version checkers don't error", {
  skip_on_cran()
  expect_error_free(pandoc2.0())
})

test_that("session info is omitted / included", {
  skip_on_cran()
  if (rlang::is_installed("sessioninfo")) {
    regex <- "^sessioninfo::session_info"
  } else {
    regex <- "^sessionInfo"
  }
  input <- c("(y <- 1:4)", "mean(y)")
  ret <- reprex(input = input)
  expect_false(any(grepl(regex, ret)))
  ret <- reprex(input = input, session_info = TRUE)
  expect_match(ret, regex, all = FALSE)
})

test_that("session info is folded for `venue = 'gh'`", {
  skip_on_cran()
  input <- c("(y <- 1:4)", "mean(y)")
  ret <- reprex(input = input, session_info = TRUE, venue = "gh")
  expect_match(ret, "<details.*>", all = FALSE)
  expect_match(ret, "</details>", fixed = TRUE, all = FALSE)
})

# test_that("reprex: clipboard input works")
# This test was removed:
#   * Feels like I'm just testing clipr, which seems silly.
#   * Because clipr and reprex have erected so many safeguards against
#     clipboard access in a noninteractive session, for CRAN reasons, this test
#     requires a great deal of gymnastics to bypass all of that.
#   * Normal usage will absolutely and immediately reveal clipboard problems.

test_that("reprex: expression input works", {
  expect_snapshot(cli::cat_line(
    reprex({x <- 1:5; mean(x)}, render = FALSE)
  ))
})

## https://github.com/tidyverse/reprex/issues/241
test_that("reprex: expression input preserves `!!`", {
  res <- reprex(
    {f <- function(c6d573e) rlang::qq_show(how_many(!!rlang::enquo(c6d573e)))},
    render = FALSE
  )
  expect_match(res, "!!rlang::enquo(c6d573e)", all = FALSE, fixed = TRUE)
})

test_that("reprex: character input works", {
  expect_snapshot(cli::cat_line(
    reprex(input = c("x <- 5:1", "mean(x)"), render = FALSE)
  ))
})

test_that("reprex: file input works", {
  local_temp_wd()
  write_lines(c("x <- 6:10", "mean(x)"), "foo.R")
  expect_snapshot(cli::cat_line(
    reprex(input = "foo.R", render = FALSE)
  ))
})

test_that("reprex: file input in a subdirectory works", {
  local_temp_wd()
  dir_create("foo")
  write_lines(c("x <- 11:15", "mean(x)"), path("foo", "foo.R"))
  expect_snapshot(cli::cat_line(
    reprex(input = path("foo", "foo.R"), render = FALSE)
  ))
})

test_that("Circular use is detected before source file written", {
  skip_on_cran()
  ret <- reprex(exp(1), venue = "gh")
  expect_snapshot(error = TRUE, reprex(input = ret, render = FALSE))
  ret <- reprex(exp(1), venue = "r")
  expect_snapshot(error = TRUE, reprex(input = ret, render = FALSE))
  ret <- reprex(exp(1), venue = "html")
  expect_snapshot(error = TRUE, reprex(input = ret, render = FALSE))
})

test_that("Leading prompts are removed", {
  skip_on_cran()

  input <- c("x <- 1:3", "median(x)")
  res <- reprex(input = input, render = FALSE)
  input2 <- paste0(getOption("prompt"), input)

  local_reprex_loud()
  expect_snapshot(
    res2 <- reprex(input = input2, render = FALSE)
  )
  expect_identical(res, res2)
})

test_that("newlines in code are protected and uniformly so across venues", {
  # NOTE: use of single vs double quotes is counter-intuitive, but deliberate
  input <- 'paste(letters[1:3], collapse = "\n")\n'
  chr_input <- reprex(input = input, render = FALSE)

  input_file <- path_temp("foo.R")
  withr::local_file(input_file)
  write_lines(
    escape_newlines('paste(letters[1:3], collapse = "\n")'),
    input_file
  )
  path_input <- reprex(input = input_file, render = FALSE)

  expr_input <- reprex(paste(letters[1:3], collapse = "\n"), render = FALSE)

  expect_identical(chr_input, path_input)
  expect_identical(chr_input, expr_input)
})

test_that("venue = 'gh' works with/without leading prose", {
  skip_on_cran()
  input <- c(
    "#' Hello world",
    "## comment",
    "1:5"
  )
  output <- c(
    "Hello world",
    "",
    "``` r",
    "## comment",
    "1:5",
    "#> [1] 1 2 3 4 5",
    "```"
  )
  ret <- reprex(input = input, venue = "gh", advertise = FALSE)
  expect_identical(ret, output)

  input <- grep("Hello", input, invert = TRUE, value = TRUE)
  output <- grep("Hello", output, invert = TRUE, value = TRUE)
  output <- output[nzchar(output)]
  ret <- reprex(input = input, venue = "gh", advertise = FALSE)
  expect_identical(ret, output)
})

test_that("venue = 'R' works, regardless of case", {
  skip_on_cran()
  input <- c(
    "#' Hello world",
    "## comment",
    "1:5"
  )
  output <- c(
    "#' Hello world",
    "## comment",
    "1:5",
    "#> [1] 1 2 3 4 5"
  )
  ret <- reprex(input = input, venue = "R", advertise = FALSE)
  expect_identical(ret[nzchar(ret)], output)
  ret <- reprex(input = input, venue = "r", advertise = FALSE)
  expect_identical(ret[nzchar(ret)], output)
})

test_that("venues = 'ds' and 'so' are aliases for 'gh'", {
  skip_on_cran()
  input <- c(
    "#' Hello world",
    "## comment",
    "1:5"
  )
  ds <- reprex(input = input, venue = "ds", session_info = TRUE, advertise = FALSE)
  so <- reprex(input = input, venue = "so", session_info = TRUE, advertise = FALSE)
  gh <- reprex(input = input, venue = "gh", session_info = TRUE, advertise = FALSE)
  expect_identical(so, gh)
  expect_identical(ds, gh)
})

test_that("local image link is not interrupted by hard line break for 'gh'", {
  skip_on_cran()
  input <- c(
    "#+ setup, include = FALSE",
    "knitr::opts_knit$set(upload.fun = identity)",
    "",
    "#+ incredibly-long-chunk-name-to-make-image-path-also-incredibly-long",
    "plot(1:3)"
  )
  out <- reprex(input = input, venue = "gh")
  i <- grep("incredibly-long", out)
  expect_true(grepl("[)]", out[i]))
})

test_that("venue = 'html' works", {
  skip_on_cran()
  input <- c(
    "#' Hello world",
    "## comment",
    "1:5"
  )
  output <- c(
    "<head>",
    "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">",
    "</head>",
    "<p>Hello world</p>",
    "<pre class=\"r\"><code>## comment",
    "1:5",
    "#&gt; [1] 1 2 3 4 5</code></pre>"
  )
  ret <- reprex(input = input, venue = "html", advertise = FALSE)
  ret <- ret[nzchar(ret)]
  expect_identical(ret, output)
})

test_that("venue = 'slack' works", {
  skip_on_cran()
  input <- c(
    "#' Hello world",
    "## comment",
    "1:5"
  )
  output <- c(
    "Hello world",
    "```",
    "## comment",
    "1:5",
    "#> [1] 1 2 3 4 5",
    "```"
  )
  ret <- reprex(input = input, venue = "slack")
  ret <- ret[nzchar(ret)]
  expect_identical(ret, output)
})

test_that("stdout is captured", {
  skip_on_cran()
  out <- reprex(system2("echo", args = "blah"), std_out_err = TRUE)
  expect_match(out, "Standard output and standard error", all = FALSE)
  expect_match(out, "^blah$", all = FALSE)
})

test_that("stdout placeholder appears if nothing is captured", {
  skip_on_cran()
  out <- reprex(1:4, std_out_err = TRUE)
  expect_match(out, "Standard output and standard error", all = FALSE)
  expect_match(out, "nothing to show", all = FALSE)
})

test_that("stdout placeholder is absent if explicitly excluded", {
  skip_on_cran()
  out <- reprex(1:4, std_out_err = FALSE)
  expect_false(any(grepl("standard output and standard error", out)))
})

test_that("UTF-8 encoding, string input", {
  skip_on_cran()

  in_utf8 <- c(
    #       a-grave   e-diaeresis  Eth
    "x <- c('\u00C0', '\u00CB', '\u00D0')",
    "print(x)"
  )
  out_utf8 <- reprex(input = in_utf8)

  expect_true(all(Encoding(out_utf8) %in% c("unknown", "UTF-8")))

  i_in <-  grep("^x <-", in_utf8)
  i_out <- grep("^x <-", out_utf8)
  expect_identical(charToRaw(in_utf8[i_in]), charToRaw(out_utf8[i_out]))

  i_out <- grep("^#> \\[1\\]", out_utf8)
  expect_match(out_utf8[i_out], "[\u00C0]")
  expect_match(out_utf8[i_out], "[\u00CB]")
  expect_match(out_utf8[i_out], "[\u00D0]")

  in_latin1 <- iconv(in_utf8, from = "UTF-8", to = "latin1")
  out_latin1 <- reprex(input = in_latin1)

  expect_identical(
    charToRaw(paste0(out_utf8, collapse = "\n")),
    charToRaw(paste0(out_latin1, collapse = "\n"))
  )
})

test_that("reprex.current_venue is set", {
  skip_on_cran()
  input <- "getOption('reprex.current_venue')"
  ret <- reprex(input = paste0(input, "\n"))
  expect_match(ret, "gh", all = FALSE)
  ret <- reprex(input = paste0(input, "\n"), venue = "html")
  expect_match(ret, "html", all = FALSE)
})

test_that("`session_info` can be set via option", {
  skip_on_cran()
  withr::with_options(
    list(reprex.session_info = TRUE),
    out <- reprex(1, render = FALSE)
  )
  expect_match(out, "session_*[iI]nfo", all = FALSE)
})

test_that("`advertise` can be set via option", {
  skip_on_cran()
  withr::with_options(
    list(reprex.advertise = FALSE),
    out <- reprex(1, render = FALSE)
  )
  expect_false(any(grepl("#+ reprex-ad", out, fixed = TRUE)))
})

test_that("`comment` can be set via option", {
  skip_on_cran()
  withr::with_options(
    list(reprex.comment = "#? "),
    out <- reprex(rnorm(1))
  )
  expect_match(out, "^#\\?", all = FALSE)
})

test_that("`tidyverse_quiet` can be set via option", {
  skip_on_cran()
  withr::with_options(
    list(reprex.tidyverse_quiet = FALSE),
    out <- reprex(mean(1:3), render = FALSE)
  )
  expect_match(out, "tidyverse_quiet: FALSE", fixed = TRUE, all = FALSE)
})

test_that("`std_out_err` can be set via option", {
  skip_on_cran()
  withr::with_options(
    list(reprex.std_out_err = TRUE),
    out <- reprex(1, render = FALSE)
  )
  expect_match(out, "std_out_err", all = FALSE)
})

test_that("`comment` works", {
  skip_on_cran()
  out <- reprex(1, comment = "#?#")
  expect_match(out, "#?#", all = FALSE, fixed = TRUE)
})

test_that("reprex() suppresses tidyverse startup message by default", {
  skip_on_cran()
  skip_if_not_installed("tidyverse", minimum_version = "1.2.1")
  ret <- reprex(input = sprintf("library(%s)\n", "tidyverse"))
  expect_false(any(grepl("Attaching", ret)))
})

test_that("`tidyverse_quiet` works", {
  skip_on_cran()
  skip_if_not_installed("tidyverse", minimum_version = "1.2.1")

  ret <- reprex(
    input = "library(tidyverse)\n",
    tidyverse_quiet = TRUE
  )
  expect_false(any(grepl("Attaching", ret)))

  ret <- reprex(
    input = "library(tidyverse)\n",
    tidyverse_quiet = FALSE
  )
  expect_match(ret, "Attaching", all = FALSE)
})

test_that("`tidyverse_quiet` works for tidymodels", {
  skip_on_cran()
  skip_if_not_installed("tidymodels")

  ret <- reprex(
    input = "library(tidymodels)\n",
    tidyverse_quiet = TRUE
  )
  expect_false(any(grepl("Attaching", ret)))

  ret <- reprex(
    input = "library(tidymodels)\n",
    tidyverse_quiet = FALSE
  )
  expect_match(ret, "Attaching", all = FALSE)
})


test_that("`style` works", {
  skip_on_cran()
  skip_if_not_installed("styler")
  ret <- reprex(input = c("a<-function( x){", "1+1}           "), style = TRUE)
  i <- grep("^a", ret)
  expect_identical(
    ret[i + 0:2],
    c("a <- function(x) {", "  1 + 1", "}")
  )
})

test_that("bang bang bang is not mangled with parentheses", {
  skip_on_cran()
  skip_if_not_installed("styler")
  input <- c(
    'nameshift <- c(SL = "Sepal.Length")',
    "head(dplyr::rename(iris[, 1:2], !!!nameshift), 3)"
  )
  ret <- reprex(input = input, style = TRUE)
  ret <- grep("dplyr::rename", ret, value = TRUE)
  expect_match(ret, "!!!")
})

is_windows <- function() {
  .Platform$OS.type == "windows"
}

is_path <- function(x) {
  length(x) == 1 && is.character(x) && !grepl("\n$", x)
}

isFALSE <- function(x) identical(x, FALSE)

is_rstudio_server <- function() {
  if (rstudioapi::hasFun("versionInfo")) {
    rstudioapi::versionInfo()$mode == "server"
  } else {
    FALSE
  }
}

in_rstudio <- function() {
  .Platform$GUI == "RStudio"
}

trim_ws <- function(x) {
  sub("\\s*$", "", sub("^\\s*", "", x))
}

trim_common_leading_ws <- function(x) {
  m <- regexpr("^(\\s*)", x)
  n_chars <- nchar(x)
  n_spaces <- attr(m, which = "match.length")
  num <- min(n_spaces[n_chars > 0])
  substring(x, num + 1)
}

escape_regex <- function(x) {
  chars <- c("*", ".", "?", "^", "+", "$", "|", "(", ")", "[", "]", "{", "}", "\\")
  gsub(paste0("([\\", paste(chars, collapse = "\\"), "])"), "\\\\\\1", x, perl = TRUE)
}

escape_newlines <- function(x) {
  gsub("\n", "\\\\n", x, perl = TRUE)
}

roxygen_comment <- function(x) paste0("#' ", x)

r_chunk <- function(code, label = NULL) {
  c(sprintf("```{r %s}", label %||% ""), label, code, "```")
}

details <- function(txt, desc = "Details") {
  c(
    "<details style=\"margin-bottom:10px;\">",
    glue("<summary>{desc}</summary>"),
    txt,
    "</details>"
  )
}

backtick <- function(x) encodeString(x, quote = "`")

newline <- function(x) paste0(x, "\n")

is_testing <- function() {
  identical(Sys.getenv("TESTTHAT"), "true")
}

#' Render a reprex in a specific locale
#'
#' Render a [reprex()], with control over the localization of error messages and
#' aspects of the locale. Note that these are related but distinct issues!
#' Typical usage is for someone on a Spanish system to create a reprex that is
#' easier for an English-speaking audience to follow.
#'
#' @section `language`:
#' Use the `language` argument to express the preferred language of error
#' messages. The output of `dir(system.file(package = "translations"))` may
#' provide some helpful ideas. The `language` should generally follow "XPG
#' syntax": a two-letter language code, optionally followed by other modifiers.
#'
#' Examples: `"en"`, `"de"`, `"en_GB"`, `"pt_BR"`.
#'
#' @section `locale`:
#' Use the `locale` argument only if you want to affect something like how
#' day-of-the-week or month is converted to character. You are less likely to
#' need to set this than the `language` argument. You may have more success
#' setting specific categories, such as `"LC_TIME"`, than multi-category
#' shortcuts like `"LC_ALL"` or `"LANG"`. The `locale` values must follow the
#' format dictated by your operating system and the requested locale must be
#' installed. On *nix systems, `locale -a` is a good way to see which locales
#' are installed. Note that the format for `locale` and `language` are different
#' from each other on Windows.
#'
#' Examples: `"en_CA.UTF-8"` (macOS), `"French_France.1252"` (Windows).
#'
#' @param ... Inputs passed through to [reprex()].
#' @param language A string specifying the preferred language for messages. It
#'   is enacted via the `LANGUAGE` environment variable, for the duration of the
#'   `reprex()` call. Examples: `"en"` for English and `"fr"` for French. See
#'   Details for more.
#' @param locale A named character vector, specifying aspects of the locale, in
#'   the [Sys.setlocale()] sense. It is enacted by setting one or more
#'   environment variables, for the duration of the `reprex()` call. See Details
#'   for more.
#'
#' @seealso
#'   * The [Locale
#'   Names](https://www.gnu.org/software/libc/manual/html_node/Locale-Names.html)
#'   section of the GNU C docs, for more about XPG syntax
#'   * The [Internationalization and
#'   Localization](https://cran.r-project.org/doc/manuals/r-patched/R-admin.html#Internationalization)
#'   section of the R Installation and Administration manual
#'
#' @return Character vector of rendered reprex, invisibly.
#' @examples
#' \dontrun{
#'
#' # if all you want to do is make sure messages are in English
#' reprex_locale("a" / 2)
#'
#' # change messages to a specific language
#' reprex_locale(
#'   {
#'     "a" / 2
#'   },
#'   language = "it"
#' )
#'
#' reprex_locale(
#'   {
#'     "a" / 2
#'   },
#'   language = "fr_CA"
#' )
#'
#' reprex_locale(
#'   {
#'     "a" / 2
#'   },
#'   language = "pt_BR"
#' )
#'
#' # get day-of-week and month to print in French (not Windows)
#' reprex_locale(
#'   {
#'     format(as.Date(c("2019-01-01", "2019-02-01")), "%a %b %d")
#'   },
#'   locale = c(LC_TIME = "fr_FR")
#' )
#'
#' # get day-of-week and month to print in French (Windows)
#' # assumes that the relevant language is installed on the system
#' # LC_TIME can also be specified as "French" or "French_France" here
#' reprex_locale(
#'   {
#'     format(as.Date(c("2019-01-01", "2019-02-01")), "%a %b %d")
#'   },
#'   locale = c(LC_TIME = "French_France.1252")
#' )
#' }
#' @export
reprex_locale <- function(...,
                          language = "en",
                          locale = NULL) {
  withr::local_envvar(c(LANGUAGE = language))
  if (!is.null(locale)) {
    # If we use withr::local_locale(), the new locale is NOT inherited by the
    # reprexing child process. Whereas it is if we use an env var approach.
    withr::local_envvar(locale)
  }
  reprex(...)
}

## input is quote()'ed expression
## reprex() takes care of that or, to use directly:
## x <- quote({a + b})

stringify_expression <- function(x) {
  if (is.null(x)) {
    return(NULL)
  }

  .srcref <- utils::getSrcref(x)

  if (is.null(.srcref)) {
    return(enc2utf8(deparse(x)))
  }

  ## Construct a new srcref with the first_line, first_byte, etc. from the
  ## first expression and the last_line, last_byte, etc. from the last one.
  first_src <- .srcref[[1]]
  last_src <- .srcref[[length(.srcref)]]

  .srcfile <- attr(first_src, "srcfile")

  src <- srcref(
    .srcfile,
    c(
      first_src[[1]], first_src[[2]],
      last_src[[3]], last_src[[4]],
      first_src[[5]], last_src[[6]],
      first_src[[7]], last_src[[8]]
    )
  )

  lines <- enc2utf8(as.character(src, useSource = TRUE))

  ## remove the first brace and line if the brace is the only thing on the line
  lines[[1L]] <- sub("^[{]", "", lines[[1L]])
  if (!nzchar(lines[[1L]])) {
    lines <- lines[-1L]
  }

  ## identify the last source line affiliated with an expression
  n <- utils::getSrcLocation(last_src, which = "line", first = FALSE)

  ## rescue trailing comment on (current) last surviving line
  last_source_line <- getSrcLines(.srcfile, n, n) ## "raw"
  last_line <- lines[length(lines)] ## srcref'd
  m <- regexpr(last_line, last_source_line, fixed = TRUE)
  rescue_me <- substring(last_source_line, m + attr(m, "match.length"))
  if (grepl("^\\s*#", rescue_me)) {
    lines[length(lines)] <- paste0(last_line, rescue_me)
  }

  ## rescue trailing comment lines
  tail_lines <- getSrcLines(.srcfile, n + 1, Inf)
  closing_bracket_line <- max(grep("^\\s*[}]", tail_lines), 0)
  tail_lines <- utils::head(tail_lines, closing_bracket_line - 1)

  trim_common_leading_ws(c(lines, tail_lines))
}

test_that("simple statements are stringified", {
  expect_identical(stringify_expression(1:5), "1:5")
  expect_identical(stringify_expression({1:5}), "1:5")
  expect_identical(stringify_expression(quote(mean(x))), "mean(x)")
})

## it is very difficult to create quoted multi-line expressions in tests
## that mimic what a user can create interactively re: the srcrefs
## therefore, I executed this interactively to create expressions.rds
if (FALSE) {
  e <- new.env()
  e$e01 <- quote({
    1:5
  })
  e$e02 <- quote({1:5
  })
  e$e03 <- quote({
    1:5})
  e$e04 <- quote({1:3;4:6})
  e$e05 <- quote({
    #' Leading comment
    x <- rnorm(3)
    #' Embedded comment
    mean(x)
    #' Trailing comment
  })
  e$e06 <- quote({mean(1:4) # comment
  })
  e$e07 <- quote({
    #' Leading comment
    y <- 1:4 # comment
    #' Trailing comment
  }
  )
  e$e08 <- quote({
x <- 1:2
{x + 3:4} %>% sum()
  })
  saveRDS(
    e,
    rprojroot::find_testthat_root_file("expressions.rds"),
    version = 2
  )
}

e <- readRDS(rprojroot::find_testthat_root_file("expressions.rds"))

test_that("one statement, brackets, multiple lines, take 1", {
  # quote({
  #   1:5
  # })
  expect_identical(
    stringify_expression(e$e01),
    "1:5"
  )
})

test_that("one statement, brackets, multiple lines, take 2", {
  # expr <- quote({1:5
  # })
  expect_identical(
    stringify_expression(e$e02),
    "1:5"
  )
})

test_that("one statement, brackets, multiple lines, take 3", {
  # expr <- quote({
  #   1:5})
  expect_identical(
    stringify_expression(e$e03),
    "1:5"
  )
})

test_that("multiple statements, brackets, semicolon", {
  # quote({1:3;4:6})
  expect_identical(
    stringify_expression(e$e04),
    "1:3;4:6"
  )
})

test_that("leading, embedded, trailing comment, #89", {
  # expr <- quote({
  #   #' Leading comment
  #   x <- rnorm(3)
  #   #' Embedded comment
  #   mean(x)
  #   #' Trailing comment
  # })
  out <- c(
    "#' Leading comment",
    "x <- rnorm(3)",
    "#' Embedded comment",
    "mean(x)",
    "#' Trailing comment"
  )
  expect_identical(
    stringify_expression(e$e05),
    out
  )
})

test_that("trailing inline comment, #91", {
  # expr <- quote({mean(1:4) # comment
  # })
  out <- "mean(1:4) # comment"
  expect_identical(
    stringify_expression(e$e06),
    out
  )
})

test_that("trailing inline comment AND trailing comment line", {
  # expr <- quote({
  #   #' Leading comment
  #   y <- 1:4 # comment
  #   #' Trailing comment
  # }
  out <- c(
    "#' Leading comment",
    "y <- 1:4 # comment",
    "#' Trailing comment"
  )
  expect_identical(
    stringify_expression(e$e07),
    out
  )
})

test_that("leading bracket that should not be removed", {
  # e$e08 <- quote({
  #   x <- 1:2
  #   {x + 3:4} %>% sum()
  # })
  out <- c(
    "x <- 1:2",
    "{x + 3:4} %>% sum()"
  )
  expect_identical(
    stringify_expression(e$e08),
    out
  )
})

reprex_quiet <- function() {
  as.logical(Sys.getenv("REPREX_QUIET", unset = "NA"))
}

local_reprex_quiet <- function(reprex_quiet = "TRUE", env = parent.frame()) {
  withr::local_envvar(c(REPREX_QUIET = reprex_quiet), .local_envir = env)
}

local_reprex_loud <- function(env = parent.frame()) {
  local_reprex_quiet("FALSE", env = env)
}

reprex_alert <- function(text,
                         type = c("success", "info", "warning", "danger"),
                         .envir = parent.frame()) {
  quiet <- reprex_quiet() %|% is_testing()
  if (quiet) {
    return(invisible())
  }
  cli_fun <- switch(type,
    success = cli::cli_alert_success,
    info    = cli::cli_alert_info,
    warning = cli::cli_alert_warning,
    danger  = cli::cli_alert_danger,
    cli::cli_alert
  )
  cli_fun(text = text, wrap = TRUE, .envir = .envir)
}

reprex_success <- function(text, .envir = parent.frame()) {
  reprex_alert(text, type = "success", .envir = .envir)
}

reprex_info <- function(text, .envir = parent.frame()) {
  reprex_alert(text, type = "info", .envir = .envir)
}

reprex_warning <- function(text, .envir = parent.frame()) {
  reprex_alert(text, type = "warning", .envir = .envir)
}

reprex_danger <- function(text, .envir = parent.frame()) {
  reprex_alert(text, type = "danger", .envir = .envir)
}

# TODO: if a better built-in solution arises in the semantic UI, use it
# https://github.com/r-lib/cli/issues/211
reprex_path <- function(header, path, type = "success", .envir = parent.frame()) {
  quiet <- reprex_quiet() %|% is_testing()
  if (quiet) {
    return(invisible())
  }
  reprex_alert(header, type = type, .envir = .envir)
  cli::cli_div(theme = list(.alert = list(`margin-left` = 2, before = "")))
  cli::cli_alert("{.path {path}}")
  cli::cli_end()
}

message <- function(...) {
  cli::cli_abort(
    "Inside {.pkg reprex}, we use our own UI functions, not {.fun message}.",
    .internal = TRUE
  )
}

test_that("reprex_quiet() defaults to NA", {
  expect_true(is.na(reprex_quiet()))
})

test_that("reprex_alert() and friends work", {
  local_reprex_loud()

  expect_snapshot({
    reprex_alert("alert", type = "")
    reprex_success("success")
    reprex_info("info")
    reprex_warning("warning")
    reprex_danger("danger")
  })
})

test_that("reprex_alert() is under the control of REPREX_QUIET env var", {
  local_reprex_quiet()
  expect_snapshot(reprex_alert("alert", type = ""))

  local_reprex_loud()
  expect_snapshot(reprex_alert("alert", type = ""))
})

test_that("reprex_path() works and respects REPREX_QUIET", {
  local_reprex_quiet()
  expect_snapshot(reprex_path("Something descriptive:", "path/to/file"))

  local_reprex_loud()
  expect_snapshot({
    reprex_path("Something descriptive:", "path/to/file")
    x <- "path/to/file"
    reprex_path("Something descriptive:", x)
    y <- c("path", "to", "file")
    reprex_path("Something descriptive:", path_join(y))
  })
})

#' Un-render a reprex
#'
#' @description
#' Recover clean, runnable code from a reprex captured in the wild and write it
#' to user's clipboard. The code is also returned invisibly and optionally
#' written to file. Three different functions address various forms of
#' wild-caught reprex.
#'
#' @inheritParams reprex
#' @param input Character. If has length one and lacks a terminating newline,
#'   interpreted as the path to a file containing the reprex. Otherwise,
#'   assumed to hold the reprex as a character vector. If not provided, the
#'   clipboard is consulted for input. If the clipboard is unavailable and
#'   we're in RStudio, the current selection is used.
#' @param comment regular expression that matches commented output lines
#' @param prompt character, the prompt at the start of R commands
#' @param continue character, the prompt for continuation lines
#' @return Character vector holding just the clean R code, invisibly
#' @name un-reprex
NULL

#' @describeIn un-reprex Attempts to reverse the effect of [reprex()]. When
#'   `venue = "r"`, this just calls `reprex_clean()`.
#' @inheritParams reprex
#' @export
#' @examples
#' \dontrun{
#' # a roundtrip: R code --> rendered reprex, as gfm --> R code
#' original <- file.path(tempdir(), "original.R")
#' writeLines(glue::glue("
#'   #' Some text
#'   #+ chunk-label-and-options-cannot-be-recovered, message = TRUE
#'   (x <- 1:4)
#'   #' More text
#'   y <- 2:5
#'   x + y"), con = original)
#' reprex(input = original, html_preview = FALSE, advertise = FALSE)
#' reprexed <- sub("[.]R$", "_reprex.md", original)
#' writeLines(readLines(reprexed))
#' unreprexed <- reprex_invert(input = reprexed)
#' writeLines(unreprexed)
#'
#' # clean up
#' file.remove(
#'   list.files(dirname(original), pattern = "original", full.names = TRUE)
#' )
#' }
reprex_invert <- function(input = NULL,
                          wd = NULL,
                          venue = c("gh", "r"),
                          comment = opt("#>"),
                          outfile = deprecated()) {
  venue <- tolower(venue)
  venue <- match.arg(venue)

  if (venue == "r") {
    return(
      reprex_clean(input, wd = wd, comment = comment, outfile = outfile)
    )
  }

  reprex_undo(input, wd = wd, is_md = TRUE, comment = comment, outfile = outfile)
}

#' @describeIn un-reprex Assumes R code is top-level, possibly interleaved with
#'   commented output, e.g., a displayed reprex copied from GitHub or the direct
#'   output of `reprex(..., venue = "R")`. This function removes commented
#'   output.
#' @export
#' @examples
#' \dontrun{
#' # a roundtrip: R code --> rendered reprex, as R code --> original R code
#' code_in <- c(
#'   "# a regular comment, which is retained",
#'   "(x <- 1:4)",
#'   "median(x)"
#' )
#' reprexed <- reprex(input = code_in, venue = "r", advertise = FALSE)
#' writeLines(reprexed)
#' code_out <- reprex_clean(input = reprexed)
#' writeLines(code_out)
#' identical(code_in, code_out)
#' }
reprex_clean <- function(input = NULL,
                         wd = NULL,
                         comment = opt("#>"),
                         outfile = deprecated()) {
  reprex_undo(input, wd = wd, is_md = FALSE, comment = comment, outfile = outfile)
}

#' @describeIn un-reprex Assumes R code lines start with a prompt and that
#'   printed output is top-level, e.g., what you'd get from copy/paste from the
#'   R Console. Removes lines of output and strips prompts from lines holding R
#'   commands.
#' @export
#' @examples
#' \dontrun{
#' # rescue a reprex that was copied from a live R session
#' from_r_console <- c(
#'   "> # a regular comment, which is retained",
#'   "> (x <- 1:4)",
#'   "[1] 1 2 3 4",
#'   "> median(x)",
#'   "[1] 2.5"
#' )
#' rescued <- reprex_rescue(input = from_r_console)
#' writeLines(rescued)
#' }
reprex_rescue <- function(input = NULL,
                          wd = NULL,
                          prompt = getOption("prompt"),
                          continue = getOption("continue"),
                          outfile = deprecated()) {
  reprex_undo(
    input,
    wd = wd,
    is_md = FALSE,
    prompt = paste(escape_regex(prompt), escape_regex(continue), sep = "|"),
    outfile = outfile
  )
}

reprex_undo <- function(input = NULL,
                        wd = NULL,
                        is_md = FALSE,
                        comment = NULL, prompt = NULL,
                        outfile = deprecated()) {
  where <- locate_input(input)
  src <- switch(where,
    clipboard = ingest_clipboard(),
    path      = read_lines(input),
    input     = escape_newlines(sub("\n$", "", input)),
    selection = rstudio_selection(),
    NULL
  )
  comment <- arg_option(comment)

  undo_files <- plan_files(
    infile = if (where == "path") input else NULL,
    wd = wd, outfile = outfile
  )
  r_file <- r_file_clean(undo_files$filebase)
  if (would_clobber(r_file)) {
    return(invisible())
  }

  if (is_md) {                             # reprex_invert
    out <- convert_md_to_r(src, comment = comment, drop_output = TRUE)
  } else if (is.null(prompt)) {            # reprex_clean
    out <- src[!grepl(comment, src)]
  } else {                                 # reprex_rescue
    regex <- paste0("^\\s*", prompt)
    out <- src[grepl(regex, src)]
    out <- sub(regex, "", out)
  }

  if (undo_files$chatty) {
    reprex_path("Writing clean code as {.code .R} script:", r_file)
  }
  write_lines(out, r_file)
  expose_reprex_output(r_file)
  invisible(out)
}

convert_md_to_r <- function(lines, comment = "#>", drop_output = FALSE) {
  lines_info <- classify_fenced_lines(lines, comment = comment)
  lines_out <- ifelse(lines_info == "prose" & nzchar(lines), roxygen_comment(lines), lines)
  drop_classes <- c("bt", if (drop_output) "output")
  lines_out[!lines_info %in% drop_classes]
}

# Classify lines in the presence of fenced code blocks.
# Specifically, blocks fenced by three backticks.
# This is true of the output from reprex() with venue "gh" (+ "so", "ds", "slack")
# Classifies each line like so:
#   * bt     = backticks
#   * code   = code inside a fenced block
#   * output = commented output inside a fenced block
#   * prose  = outside a fenced block
classify_fenced_lines <- function(x, comment = "^#>") {
  x_shift <- c("", utils::head(x, -1))
  cumulative_fences <- cumsum(grepl("^```", x_shift))
  wut <- ifelse(grepl("^```", x), "bt",
    ifelse(cumulative_fences %% 2 == 1, "code", "prose")
  )
  wut <- ifelse(wut == "code" & grepl(comment, x), "output", wut)
  wut
}

## intentionally forcing continuation lines
input <- c(
  "## a comment",
  "x <- 1:4",
  "#' hi",
  "y <- c(2,",
  "  3, 4,",
  "  5)",
  "x + y"
)

test_that("round trip, venue = 'gh': reprex() --> reprex_invert()", {
  skip_on_cran()
  output <- reprex(input = input, advertise = FALSE)
  res <- reprex_invert(output)
  expect_identical(input, res[nzchar(res)])
})

test_that("round trip, venue = 'r': reprex() --> reprex_invert()", {
  skip_on_cran()
  output <- reprex(input = input, advertise = FALSE, venue = "r")
  res <- reprex_clean(output)
  expect_identical(input, res[nzchar(res)])
})

test_that("reprex_rescue() rescues code from R Console copy/paste", {
  skip_on_cran()
  console <- c(
    "> ## a regular comment, which is retained",
    "> (y <- c(2,",
    "+         3, 4,",
    "+         5))",
    "[1] 2 3 4 5",
    "> median(y)",
    "[1] 3.5"
  )
  output <- c(
    "## a regular comment, which is retained",
    "(y <- c(2,",
    "        3, 4,",
    "        5))",
    "median(y)"
  )
  expect_identical(reprex_rescue(console), output)
})

test_that("reprex_rescue()'s prompt argument works", {
  skip_on_cran()
  code <- c(
    ":-) ## a regular comment, which is retained",
    ":-) (x <- 1:4)",
    "[1] 1 2 3 4",
    ":-) median(x)",
    "[1] 2.5"
  )
  output <- c(
    "## a regular comment, which is retained",
    "(x <- 1:4)",
    "median(x)"
  )
  expect_identical(reprex_rescue(code, prompt = ":-) "), output)
})

test_that("reprex_rescue()'s continue argument works", {
  skip_on_cran()
  code <- c(
    "> ## a regular comment, which is retained",
    "> (y <- c(2,",
    "yes, and?         3, 4,",
    "yes, and?         5))",
    "[1] 2 3 4 5",
    "> median(y)",
    "[1] 3.5"
  )
  output <- c(
    "## a regular comment, which is retained",
    "(y <- c(2,",
    "        3, 4,",
    "        5))",
    "median(y)"
  )
  expect_identical(reprex_rescue(code, continue = "yes, and? "), output)
})

test_that("reprex_rescue() can cope with leading whitespace", {
  skip_on_cran()
  console <- c(
    "> ## a regular comment, which is retained",
    " > (x <- 1:4)",
    "   [1] 1 2 3 4",
    "   > median(x)",
    "2.5"
  )
  output <- c(
    "## a regular comment, which is retained",
    "(x <- 1:4)",
    "median(x)"
  )
  expect_identical(reprex_rescue(console), output)
})

test_that("reprex_invert() can write to specific outfile", {
  skip_on_cran()
  local_temp_wd()

  code <- c("x <- 1:3", "median(x)")
  invert_me <- reprex(input = code, advertise = FALSE)
  out <- reprex_invert(input = invert_me, outfile = "foo")
  expect_identical(read_lines("foo_clean.R"), out)
})

test_that("reprex_invert() can name its own outfile", {
  skip_on_cran()
  local_temp_wd()

  code <- c("x <- 1:3", "median(x)")
  invert_me <- reprex(input = code, advertise = FALSE)

  local_reprex_loud()
  msg <- capture_messages(
    out <- reprex_invert(input = invert_me, outfile = NA)
  )
  msg <- sub("\n$", "", msg)
  outfile <- regmatches(msg, regexpr("reprex(.*)", msg))
  expect_identical(read_lines(outfile), out)
})

test_that("reprex_invert() can name outfile based on input filepath", {
  skip_on_cran()
  local_temp_wd()

  code <- c("x <- 1:3", "median(x)")
  reprex(input = code, advertise = FALSE, outfile = "a")
  out <- reprex_invert(input = "a_reprex.md", outfile = NA)
  expect_identical(read_lines("a_reprex_clean.R"), out)
})

test_that("reprex_invert(venue = 'gh') doesn't strip leading ws", {
  skip_on_cran()
  local_temp_wd()

  input <- c("head(", "    letters)")
  reprexed <- reprex(
    input = input, venue = "gh", advertise = FALSE
  )
  inverted <- reprex_invert(reprexed, venue = "gh")
  expect_match(inverted, input[2], all = FALSE, fixed = TRUE)
})

## intentionally forcing continuation lines
input <- c(
  "## a comment",
  "x <- 1:4",
  "#' hi",
  "y <- c(2,",
  "  3, 4,",
  "  5)",
  "x + y"
)

test_that("round trip, venue = 'gh': reprex() --> reprex_invert()", {
  skip_on_cran()
  output <- reprex(input = input, advertise = FALSE)
  res <- reprex_invert(output)
  expect_identical(input, res[nzchar(res)])
})

test_that("round trip, venue = 'r': reprex() --> reprex_invert()", {
  skip_on_cran()
  output <- reprex(input = input, advertise = FALSE, venue = "r")
  res <- reprex_clean(output)
  expect_identical(input, res[nzchar(res)])
})

test_that("reprex_rescue() rescues code from R Console copy/paste", {
  skip_on_cran()
  console <- c(
    "> ## a regular comment, which is retained",
    "> (y <- c(2,",
    "+         3, 4,",
    "+         5))",
    "[1] 2 3 4 5",
    "> median(y)",
    "[1] 3.5"
  )
  output <- c(
    "## a regular comment, which is retained",
    "(y <- c(2,",
    "        3, 4,",
    "        5))",
    "median(y)"
  )
  expect_identical(reprex_rescue(console), output)
})

test_that("reprex_rescue()'s prompt argument works", {
  skip_on_cran()
  code <- c(
    ":-) ## a regular comment, which is retained",
    ":-) (x <- 1:4)",
    "[1] 1 2 3 4",
    ":-) median(x)",
    "[1] 2.5"
  )
  output <- c(
    "## a regular comment, which is retained",
    "(x <- 1:4)",
    "median(x)"
  )
  expect_identical(reprex_rescue(code, prompt = ":-) "), output)
})

test_that("reprex_rescue()'s continue argument works", {
  skip_on_cran()
  code <- c(
    "> ## a regular comment, which is retained",
    "> (y <- c(2,",
    "yes, and?         3, 4,",
    "yes, and?         5))",
    "[1] 2 3 4 5",
    "> median(y)",
    "[1] 3.5"
  )
  output <- c(
    "## a regular comment, which is retained",
    "(y <- c(2,",
    "        3, 4,",
    "        5))",
    "median(y)"
  )
  expect_identical(reprex_rescue(code, continue = "yes, and? "), output)
})

test_that("reprex_rescue() can cope with leading whitespace", {
  skip_on_cran()
  console <- c(
    "> ## a regular comment, which is retained",
    " > (x <- 1:4)",
    "   [1] 1 2 3 4",
    "   > median(x)",
    "2.5"
  )
  output <- c(
    "## a regular comment, which is retained",
    "(x <- 1:4)",
    "median(x)"
  )
  expect_identical(reprex_rescue(console), output)
})

test_that("reprex_invert() can write to specific outfile", {
  skip_on_cran()
  local_temp_wd()

  write_lines(c("x <- 1:3", "median(x)"), "foo.R")
  reprex(input = "foo.R", advertise = FALSE)
  out <- reprex_invert(input = "foo_reprex.md")
  expect_identical(read_lines("foo_reprex_clean.R"), out)
})

test_that("reprex_invert() can name its own outfile", {
  skip_on_cran()
  local_temp_wd()

  code <- c("x <- 1:3", "median(x)")
  invert_me <- reprex(input = code, advertise = FALSE)
  out <- reprex_invert(input = invert_me, wd = ".")
  r_file <- dir_ls(regexp = "_clean[.]R$")
  expect_identical(read_lines(r_file), out)
})

test_that("reprex_invert(venue = 'gh') doesn't strip leading ws", {
  skip_on_cran()
  local_temp_wd()

  src <- c("head(", "    letters)")
  write_lines(src, "whitespace.R")
  invert_me <- reprex(input = "whitespace.R", venue = "gh", advertise = FALSE)
  inverted <- reprex_invert(input = "whitespace_reprex.md", venue = "gh")
  expect_equal(inverted, src)
})

#' Render a reprex, conveniently
#'
#' @description `reprex_addin()` opens an [RStudio
#'   gadget](https://shiny.rstudio.com/articles/gadgets.html) and
#'   [addin](http://rstudio.github.io/rstudioaddins/) that allows you to say
#'   where the reprex source is (clipboard? current selection? active file?
#'   other file?) and to control a few other arguments. Appears as "Render
#'   reprex" in the RStudio Addins menu.
#'
#' @description `reprex_selection()` is an
#'   [addin](http://rstudio.github.io/rstudioaddins/) that reprexes the current
#'   selection, optionally customised by options. Appears as "Reprex selection"
#'   in the RStudio Addins menu. Heavy users might want to create a keyboard
#'   shortcut, which is described in
#'   `https://support.rstudio.com/hc/en-us/articles/206382178-Customizing-Keyboard-Shortcuts`.
#'   Suggested shortcut: Cmd + Shift + R (macOS) or Ctrl + Shift + R (Windows).
#'
#' @export
reprex_addin <- function() { # nocov start

  check_installed(
    c("shiny", "miniUI"),
    "in order to use the reprex addin"
  )
  resource_path <- path_package("reprex", "addins")
  shiny::addResourcePath("reprex_addins", resource_path)

  ui <- miniUI::miniPage(
    shiny::tags$head(shiny::includeCSS(path(resource_path, "reprex.css"))),
    miniUI::gadgetTitleBar(
      shiny::p(
        "Use",
        shiny::a(href = "https://reprex.tidyverse.org", "reprex"),
        "to render a bit of code"
      ),
      right = miniUI::miniTitleBarButton("done", "Render", primary = TRUE)
    ),
    miniUI::miniContentPanel(
      shiny::radioButtons(
        "source",
        "Where is reprex source?",
        c(
          "on the clipboard" = if (reprex_clipboard()) "clipboard",
          "current selection" = "cur_sel",
          "current file" = "cur_file",
          "another file" = "input_file"
        )
      ),
      shiny::conditionalPanel(
        condition = "input.source == 'input_file'",
        shiny::fileInput(
          inputId = "source_file",
          label = "Source file"
        )
      ),
      shiny::radioButtons(
        "venue",
        "Target venue:",
        c(
          "GitHub or Stack Overflow" = "gh",
          "R script (output appears as comments)" = "r",
          "HTML" = "html",
          "Rich Text Format" = "rtf",
          "Slack message" = "slack"
        ),
        selected = getOption("reprex.venue", "gh")
      ),
      shiny::tags$hr(),
      shiny::checkboxInput(
        "session_info",
        "Append session info",
        getOption("reprex.session_info", FALSE)
      ),
      shiny::checkboxInput(
        "html_preview",
        "Preview HTML",
        getOption("reprex.html_preview", TRUE)
      )
    )
  )

  server <- function(input, output, session) {
    shiny::observeEvent(input$done, {
      shiny::stopApp(reprex_guess(
        input$source,
        input$venue,
        input$source_file,
        as.logical(input$session_info),
        as.logical(input$html_preview)
      ))
    })
  }

  app <- shiny::shinyApp(ui, server, options = list(quiet = TRUE))
  shiny::runGadget(app, viewer = shiny::dialogViewer("Render reprex"))
}

reprex_guess <- function(source, venue = "gh", source_file = NULL,
                         session_info = FALSE, html_preview = FALSE) {
  reprex_input <- switch(source,
    clipboard = NULL,
    cur_sel = rstudio_selection(),
    cur_file = rstudio_file(),
    input_file = source_file$datapath
  )

  reprex(
    input = reprex_input,
    venue = venue,
    session_info = session_info,
    html_preview = html_preview
  )
}

#' @export
#' @rdname reprex_addin
#' @inheritParams reprex
reprex_selection <- function(venue = getOption("reprex.venue", "gh")) {
  reprex(input = rstudio_selection(), venue = venue)
}

# RStudio helpers ---------------------------------------------------------

rstudio_file <- function(context = rstudio_context()) {
  rstudio_text_tidy(context$contents)
}

rstudio_selection <- function(context = rstudio_context()) {
  text <- rstudioapi::primary_selection(context)[["text"]]
  rstudio_text_tidy(text)
}

rstudio_context <- function() {
  rstudioapi::getSourceEditorContext()
}
# nocov end

rstudio_text_tidy <- function(x) {
  if (identical(x, "")) {
    return(character())
  }
  Encoding(x) <- "UTF-8"
  if (length(x) == 1) {
    ## rstudio_selection() returns catenated text
    x <- strsplit(x, "\n")[[1]]
  }

  n <- length(x)
  if (!grepl("\n$", x[[n]])) {
    x[[n]] <- newline(x[[n]])
  }
  x
}

ensure_not_empty <- function(x) {
  if (length(x) > 0) {
    x
  } else {
    read_lines(path_package("reprex", "templates", "BETTER_THAN_NOTHING.R"))
  }
}

ensure_not_dogfood <- function(x) {
  looks_like_fenced_md <- any(grepl("^```", x))
  if (looks_like_fenced_md) {
    lines <- paste0("  ", x[1:3])
    ## I negate yep(), instead of using nope(), to get desired behaviour in
    ## a non-interactive call
    if (!yep(
      "First three lines of putative code are:\n",
      glue_collapse(lines, sep = "\n"), "\n",
      "which doesn't look like R code.\n",
      "Are we going in circles? Did you just run reprex()?\n",
      "In that case, the clipboard or selection now holds the *rendered* result.\n",
      "Carry on with this reprex?"
    )) {
      cli::cli_abort("Cancelling.", call = quote(reprex()))
    }
  }

  looks_like_r <- any(grepl("^#>", x))
  if (looks_like_r) {
    if (!yep(
      "Putative code contains lines that start with `#>`.\n",
      "Are we going in circles? Did you just run `reprex(..., venue = \"r\")`?\n",
      "In that case, the clipboard or selection now holds the *rendered* result.\n",
      "Carry on with this reprex?"
    )) {
      cli::cli_abort("Cancelling.", call = quote(reprex()))
    }
  }

  html_start <- grep("^<pre class=\"r\">", x)
  if (length(html_start) > 0) {
    lines <- paste0("  ", x[html_start + 0:2])
    if (!yep(
      "First three lines of putative code are:\n",
      glue_collapse(lines, sep = "\n"), "\n",
      "which looks like html, not R code.\n",
      "Are we going in circles? Did you just run `reprex(..., venue = \"html\")`?\n",
      "In that case, the clipboard or selection now holds the *rendered* result.\n",
      "Carry on with this reprex?"
    )) {
      cli::cli_abort("Cancelling.", call = quote(reprex()))
    }
  }

  x
}

ensure_no_prompts <- function(x, prompt = getOption("prompt")) {
  regex <- paste0("^", escape_regex(prompt))
  prompts <- grepl(regex, x)
  if (any(prompts)) {
    reprex_info("Removing leading prompts from reprex source.")
  }
  sub(regex, "", x)
}

#' Render a document in a new R session
#'
#' @description
#' This is a wrapper around [rmarkdown::render()] that enforces the "reprex"
#' mentality. Here's a simplified version of what happens:
#' ```
#' callr::r(
#'   function(input) {
#'     rmarkdown::render(input, envir = globalenv(), encoding = "UTF-8")
#'   },
#'   args = list(input = input),
#'   spinner = is_interactive(),
#'   stdout = std_file, stderr = std_file
#' )
#' ```
#' Key features to note
#' * [rmarkdown::render()] is executed in a new R session, by using
#'   [callr::r()]. The goal is to eliminate the leakage of objects, attached
#'   packages, and other aspects of session state from the current session into
#'   the rendering session. Also, the system and user-level `.Rprofile`s are
#'   ignored.
#' * Code is evaluated in the `globalenv()` of this new R session, which means
#'   that method dispatch works the way most people expect it to.
#' * The input file is assumed to be UTF-8, which is a knitr requirement as of
#'   v1.24.
#' * If the YAML frontmatter includes `std_err_out: TRUE`, standard output and
#'   error of the rendering R session are captured in `std_file`, which is
#'   then injected into the rendered result.
#'
#' `reprex_render()` is designed to work with the [reprex_document()] output
#' format, typically through a call to [reprex()]. `reprex_render()` may work
#' with other R Markdown output formats, but it is not well-tested.
#'
#' @param input The input file to be rendered. This can be a `.R` script or a
#'   `.Rmd` R Markdown document.
#' @inheritParams reprex
#' @param encoding The encoding of the input file. Note that the only acceptable
#'   value is "UTF-8", which is required by knitr as of v1.24. This is exposed
#'   as an argument purely for technical convenience, relating to the "Knit"
#'   button in the RStudio IDE.
#'
#' @return The output of [rmarkdown::render()] is passed through, i.e. the path
#'   of the output file.
#' @export
#'
#' @examples
#' \dontrun{
#' reprex_render("input.Rmd")
#' }
reprex_render <- function(input,
                          html_preview = NULL,
                          encoding = "UTF-8") {
  if (!identical(encoding, "UTF-8")) {
    cli::cli_abort("The {.arg input} file must have UTF-8 encoding.")
  }
  reprex_render_impl(
    input,
    new_session = TRUE,
    html_preview = html_preview
  )
}

prex_render <- function(input,
                        html_preview = NULL) {
  reprex_render_impl(
    input,
    new_session = FALSE,
    html_preview = html_preview
  )
}

reprex_render_impl <- function(input,
                               new_session = TRUE,
                               html_preview = NULL) {
  yaml_opts <- reprex_document_options(input)

  venue   <- yaml_opts[["venue"]] %||% "gh"
  comment <- yaml_opts[["comment"]] %||% "#>"

  html_preview <-
    html_preview %||% yaml_opts[["html_preview"]] %||% is_interactive_ish()
  stopifnot(is_bool(html_preview))

  std_out_err <- new_session && (yaml_opts[["std_out_err"]] %||% FALSE)
  if (tolower(path_ext(input)) == "rmd") {
    input <- file_copy(input, rmd_file(input), overwrite = TRUE)
  }
  std_file <- std_out_err_path(input, std_out_err)

  if (new_session) {
    # if callr::r() picks up a local .Rprofile, it should be local to
    # where the the reprex work is happening, not the session where reprex()
    # was called
    withr::with_dir(
      path_dir(input),
      out <- tryCatch(
        callr::r(
          function(input) {
            rmarkdown::render(
              input,
              quiet = TRUE, envir = globalenv(), encoding = "UTF-8"
            )
          },
          args = list(input = path_file(input)),
          spinner = is_interactive(),
          stdout = if (is.null(std_file)) NULL else path_file(std_file),
          stderr = if (is.null(std_file)) NULL else path_file(std_file)
        ),
        error = function(e) e
      )
    )

    # reprex has crashed rmarkdown::render()
    if (is.null(out)) {
      cli::cli_abort("
        This reprex appears to halt execution of {.fun rmarkdown::render}.",
        call = quote(reprex_render())
      )
    }

    # reprex has crashed R
    if (inherits(out, "error")) {
      if (!inherits(out, "callr_status_error")) {
        cli::cli_abort(
          "Unhandled error from {.fun rmarkdown::render} in the external process.",
          .internal = TRUE
        )
      }
      if (!isTRUE(std_out_err)) {
        cli::cli_abort("
          This reprex appears to crash R.
          Call {.fun reprex} again with {.code std_out_err = TRUE} to get \\
          more info.",
          call = quote(reprex_render())
        )
      }
      md_lines <- c(
        "This reprex appears to crash R.",
        "See standard output and standard error for more details.",
        "",
        std_out_err_stub(input, venue %in% c("gh", "html"))
      )
      md_file <- md_file(input)
      write_lines(md_lines, md_file)
    } else {
      md_file <- out
    }

    if (!is.null(std_file)) {
      inject_file(md_file, std_file)
    }
  } else {
    md_file <- rmarkdown::render(
      input,
      quiet = TRUE, envir = globalenv(), encoding = "UTF-8",
      knit_root_dir = getwd()
    )
  }

  # we can almost use the post_processor of output_format, but sadly we cannot
  # we can't inject std_out_err until the connection to std_file is closed
  # and we can't post process until the injection is done
  reprex_file <- switch(venue,
    r     = pp_md_to_r(md_file, comment = comment),
    rtf   = pp_highlight(pp_md_to_r(md_file, comment = comment)),
    slack = pp_slackify(md_file),
    html  = pp_html_render(md_file),
    md_file
  )

  # also something that would naturally go in a post_processor, but can't
  # (see above)
  if (html_preview) {
    preview(md_file)
  }
  invisible(reprex_file)
}

# heavily influenced by the post_processor() function of github_document()
preview <- function(input) {
  css <- rmarkdown::pandoc_path_arg(
    path_package(
      "rmarkdown",
      "rmarkdown/templates/github_document/resources/github.css"
    )
  )
  css <- glue("github-markdown-css:{css}")
  template <- rmarkdown::pandoc_path_arg(
    path_package(
      "rmarkdown",
      "rmarkdown/templates/github_document/resources/preview.html"
    )
  )
  args <- c(
    "--standalone", "--self-contained",
    "--highlight-style", "pygments",
    "--template", template,
    "--variable", css,
    "--metadata", "pagetitle=PREVIEW",
    "--quiet"
  )

  # important considerations re: HTML preview
  # 1. where it lives matters, i.e. RStudio's decision to display it within
  #    the app (vs. using an external browser) hinges on it being located below
  #    session temp dir or RMARKDOWN_PREVIEW_DIR
  # 2. best not to co-locate with input, because (a) the user really shouldn't
  #    ever see such a preview file and (b) there's the potential for confusion
  #    with the actual reprex file when `venue = "html"` (although we do use
  #    a '_preview" suffix to disambiguate)
  preview_file <- preview_file(input)
  rmarkdown::pandoc_convert(
    input = input, to = "html", from = "gfm", output = preview_file,
    options = args, verbose = FALSE
  )

  # can be interesting re: detecting how we were called and what we should
  # do re: getting the html open
  # cat("\nRSTUDIO: ", Sys.getenv("RSTUDIO", unset = NA), file = stderr())
  # cat("\n.Platform$GUI: ", .Platform$GUI, file = stderr())
  # cat("\nis_interactive(): ", is_interactive(), file = stderr())
  # cat("\nRMARKDOWN_PREVIEW_DIR: ", Sys.getenv("RMARKDOWN_PREVIEW_DIR", NA), file = stderr())
  # cat("\ntempdir(): ", tempdir(), file = stderr())
  # cat("\n")

  preview_dir <- Sys.getenv("RMARKDOWN_PREVIEW_DIR", unset = tempdir())
  preview_file <- file_move(preview_file, preview_dir)

  if (is_interactive()) {
    viewer <- getOption("viewer") %||% utils::browseURL
    viewer(preview_file)
  } else {
    # a rudimentary proxy for:
    # "hey, we got here via the 'Knit' button"
    # so, morally, the session IS still interactive
    # this magic utterance causes RStudio to preview the file because of:
    # https://github.com/rstudio/rstudio/blob/1f998005fcafe3372413e9eb0c0b0567c46056ce/src/cpp/session/modules/rmarkdown/SessionRMarkdown.cpp#L188
    cat("\nPreview created: ", preview_file, file = stderr())
  }

  invisible(preview_file)
}

# passes is_interactive() through EXCEPT for a specific set of conditions
# that are intended to detect reprex_render() executed via RStudio's "Knit"
# button
is_interactive_ish <- function() {
  if (is_interactive()) {
    return(TRUE)
  }

  Sys.getenv("RSTUDIO", unset = "0") == "1" &&
    !is.na(Sys.getenv("RMARKDOWN_PREVIEW_DIR", unset = NA))
}

reprex_document_options <- function(input) {
  yaml_input <- input
  if (tolower(path_ext(input)) == "r") {
    yaml_input <- knitr::spin(input, knit = FALSE)
    withr::defer(file_delete(yaml_input))
  }
  yaml <- rmarkdown::yaml_front_matter(yaml_input)
  tryCatch(
    yaml[["output"]][["reprex::reprex_document"]],
    error = function(e) list()
  )
}

std_out_err_path <- function(input, std_out_err) {
  if (isTRUE(std_out_err)) {
    std_file(input)
  } else {
    NULL
  }
}

inject_file <- function(path, inject_path) {
  regex <- glue("(`)(.*)({inject_path})(`)")

  lines <- read_lines(path)
  inject_locus <- grep(regex, lines)

  # a user should never see this, but it can happen during development
  if (length(inject_locus) > 1) {
    reprex_warning("multiple placeholders for std_out_err! taking the last")
    inject_locus <- inject_locus[length(inject_locus)]
  }

  if (length(inject_locus)) {
    inject_lines <- read_lines(inject_path)
    if (length(inject_lines) == 0) {
      inject_lines <- "-- nothing to show --"
    }
    inject_lines <- c("``` sh", inject_lines, "```")
    regex <- glue("(.*){regex}(.*)")
    lines <- c(
      lines[seq_len(inject_locus - 1)],
      sub(regex, "\\1", lines[inject_locus]),
      inject_lines,
      sub(regex, "\\6", lines[inject_locus]),
      lines[-seq_len(inject_locus)]
    )
    write_lines(lines, path)
  }
  path
}

# used when venue is "r" or "rtf"
pp_md_to_r <- function(input, comment = "#>") {
  output_lines <- read_lines(md_file(input))
  output_lines <- convert_md_to_r(output_lines, comment = comment)
  rout_file <- r_file_rendered(input)
  write_lines(output_lines, rout_file)
  rout_file
}

# used when venue is "slack"
# https://www.markdownguide.org/tools/slack/
pp_slackify <- function(input) {
  output_lines <- read_lines(md_file(input))
  output_lines <- remove_info_strings(output_lines)
  output_lines <- simplify_image_links(output_lines)
  slack_file <- md_file_slack(input)
  write_lines(output_lines, slack_file)
  slack_file
}

# remove "info strings" from opening code fences, e.g. ```r
# https://spec.commonmark.org/0.29/#info-string
remove_info_strings <- function(x) {
  sub("^```[^`]*$", "```", x, perl = TRUE)
}

# input:  ![](https://i.imgur.com/woc4vHs.png)
# output: https://i.imgur.com/woc4vHs.png
simplify_image_links <- function(x) {
  sub("(^!\\[\\]\\()(.+)(\\)$)", "\\2", x, perl = TRUE)
}

# used when venue is "rtf"
pp_highlight <- function(input) {
  rtf_file <- rtf_file(input)
  reprex_highlight(r_file_rendered(input), rtf_file)
  rtf_file
}

# used when venue is "html"
pp_html_render <- function(input) {
  output_file <- rmarkdown::render(
    md_file(input),
    output_format = rmarkdown::html_fragment(
      self_contained = FALSE,
      pandoc_args = "--quiet"
    ),
    clean = FALSE,
    quiet = TRUE,
    encoding = "UTF-8"
  )
  output_file <- file_move(output_file, html_file(input))
  # the html_fragment() output is a bit too minimal
  # I add an encoding specification
  # I think this is positive-to-neutral for the reprex output and, if I don't,
  # viewing the fragment in the browser results in mojibake
  output_lines <- read_lines(output_file)
  output_lines <- c(
    "<head>",
    "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">",
    "</head>",
    output_lines
  )
  write_lines(output_lines, output_file)
  output_file
}

# https://github.com/tidyverse/reprex/issues/349
test_that("reprex additions are added to a *copy* of an Rmd input file", {
  skip_on_cran()
  fixture_path <- path_abs(test_path("fixtures/a-reprex-document.Rmd"))
  local_temp_wd()

  file_copy(fixture_path, "foo.Rmd")
  n_before <- read_lines("foo.Rmd")
  reprex_render("foo.Rmd")
  n_after <- read_lines("foo.Rmd")
  expect_equal(n_before, n_after)
})

test_that("remove_info_strings() gets rid of 'info strings'", {
  f <- function(x) strsplit(glue::glue(x), split = "\n")[[1]]
  # examples from https://spec.commonmark.org/0.29/#info-string
  x <- f("
    ```
    <
     >
    ```")
  expect_equal(x, remove_info_strings(x))

  x <- f("
    ```ruby
    def foo(x)
      return 3
    end
    ```")
  expect_equal(sub("ruby", "", x), remove_info_strings(x))

  x <- f("
    ``` ruby startline=3 $%@#$
    def foo(x)
      return 3
    end
    ```")
  expect_equal(c("```", tail(x, -1)), remove_info_strings(x))
})

test_that("preview() works with the 'Knit' button", {
  skip_on_cran()

  local_temp_wd()
  write_lines("3 * 5\n", "foo.R")
  reprex(input = "foo.R")

  withr::local_envvar(c(RMARKDOWN_PREVIEW_DIR = "."))
  rlang::local_interactive(FALSE)

  msg <- capture.output(
    preview_file <- preview("foo_reprex.md"),
    type = "message"
  )

  expect_true(file_exists("foo_reprex_preview.html"))
  expect_equal(path_file(preview_file), "foo_reprex_preview.html")
  expect_messages_to_include(
    msg,
    c("^Preview created:", "foo_reprex_preview.html$")
  )
})

test_that("preview() calls viewer() in the interactive RStudio scenario", {
  skip_on_cran()

  local_temp_wd()
  write_lines("10 / 5\n", "foo.R")
  reprex(input = "foo.R")

  rlang::local_interactive(TRUE)
  local_options(viewer = function(...) cat("viewer!", file = stderr()))

  msg <- capture.output(
    preview_file <- preview("foo_reprex.md"),
    type = "message"
  )

  expect_true(file_exists(preview_file))
  expect_equal(path_file(preview_file), "foo_reprex_preview.html")
  expect_messages_to_include(msg, "viewer!")
})

.onLoad <- function(libname, pkgname) {
  withr::with_preserve_seed({
    # create a new random permutation every time we load
    adjective_animal <<- sample(adjective_animal, size = length(adjective_animal))
  })
  invisible()
}

#' @keywords internal
#' @aliases reprex-package
"_PACKAGE"

## usethis namespace: start
#' @import fs
#' @import rlang
#' @importFrom glue glue
#' @importFrom glue glue_collapse
#' @importFrom lifecycle deprecated
## usethis namespace: end
NULL

# Work around some bug in R.cache and/or styler that affects CI ----
# 2024-01: Since this went in, I have seen yet another CI failure due to what I
# assume is this directory not existing, presumably because some code in styler
# deleted it, after we created it here.
# The problem seemed to go away upon further investigation, so it seems
# somewhat stochastic.
# See https://github.com/tidyverse/reprex/pull/455.
# If we have to debug this again, these are some thoughts:
# * When we forcibly create the directory, also put a file in it. That might
#   keep styler from deleting it.
# * Figure out how to deactivate all styler caching for reprex, at least on CI.
#   Seems to require `options(styler.cache_name = NULL)`.
if (getRversion() >= "4.0.0" && identical(Sys.getenv("CI"), "true")) {
  dir.create(
    tools::R_user_dir("R.cache", which = "cache"),
    recursive = TRUE,
    showWarnings = FALSE
  )
}

expect_messages_to_include <- function(haystack, needles) {
  lapply(
    needles,
    function(x) expect_match(haystack, x, all = FALSE)
  )
  invisible()
}

# 1. creates a subdirectory within session temp
# 2. makes that the current working directory
# 3. schedules these cleanup actions for when env goes out of scope:
#    - restore original working directory
#    - delete the directory
local_temp_wd <- function(pattern = "reprextests",
                          env = parent.frame()) {
  old_wd <- getwd()
  tmp <- withr::local_tempdir(pattern = pattern, .local_envir = env)
  withr::local_dir(tmp, .local_envir = env)
  reprex_path("Switching to temporary working directory:", tmp)
  withr::defer(
    reprex_path("Restoring original working directory:", old_wd),
    envir = env
  )
  invisible(tmp)
}

test_that("expected outfiles are written and messaged, venue = 'gh'", {
  skip_on_cran()
  local_temp_wd()
  local_reprex_loud()

  msg <- trimws(capture_messages(
    ret <- reprex(1:5, wd = ".")
  ))

  outfiles <- dir_ls()
  expect_setequal(
    gsub("[a-z-]+(_reprex.+)", "\\1", outfiles),
    c("_reprex.R", "_reprex.md")
  )
  r_file <- grep("_reprex[.]R", outfiles, value = TRUE)
  expect_match(read_lines(r_file), "1:5", all = FALSE)
  md_file <- grep("_reprex[.]md", outfiles, value = TRUE)
  expect_equal(ret, read_lines(md_file))

  expect_messages_to_include(
    msg,
    c("Preparing reprex as .*.R.* file", "Writing reprex file", outfiles)
  )
})

test_that("expected outfiles are written and messaged, venue = 'R'", {
  skip_on_cran()
  local_temp_wd()
  local_reprex_loud()

  msg <- trimws(capture_messages(
    ret <- reprex(1:5, wd = ".", venue = "R")
  ))

  outfiles <- dir_ls()
  expect_setequal(
    gsub("[a-z-]+(_reprex.+)", "\\1", outfiles),
    c("_reprex.R", "_reprex.md", "_reprex_r.R")
  )
  rout_file <- grep("_reprex_r[.]R", outfiles, value = TRUE)
  expect_equal(ret, read_lines(rout_file))

  expect_messages_to_include(
    msg,
    c("Preparing reprex as .*R.* file", "Writing reprex file", rout_file)
  )
})

test_that("expected outfiles are written and messaged, venue = 'html'", {
  skip_on_cran()
  local_temp_wd()
  local_reprex_loud()

  msg <- trimws(capture_messages(
    ret <- reprex(1:5, wd = ".", venue = "html")
  ))

  outfiles <- dir_ls()
  # punting on the issue of the `utf8.md` file and folder of files
  expect_true(all(
    c("_reprex.R", "_reprex.md", "_reprex.html") %in%
      gsub("[a-z-]+(_reprex.+)", "\\1", outfiles)
  ))
  html_file <- grep("_reprex[.]html", outfiles, value = TRUE)
  expect_equal(ret, read_lines(html_file))

  expect_messages_to_include(
    msg,
    c("Preparing reprex as .*R.* file", "Writing reprex file", html_file)
  )
})

test_that(".R outfile doesn't clobber .R infile", {
  skip_on_cran()
  local_temp_wd()

  write_lines("1:5", "foo.R")
  ret <- reprex(input = "foo.R")
  expect_identical("1:5", read_lines("foo.R"))
})

test_that("infile can have path components", {
  skip_on_cran()
  local_temp_wd()
  local_reprex_loud()

  dir_create("aaa")
  write_lines("1:5", "aaa/bbb.R")
  msg <- capture_messages(
    ret <- reprex(input = "aaa/bbb.R")
  )
  expect_messages_to_include(
    msg,
    c(
      "Preparing reprex as .*.R.* file", "aaa/bbb_reprex.R",
      "Writing reprex file", "aaa/bbb_reprex.md"
    )
  )
})

test_that("pre-existing xyz_reprex.R doesn't get clobbered w/o user's OK", {
  skip_on_cran()
  local_temp_wd()

  write_lines("5:1", "xyz.R")
  ret <- reprex(input = "xyz.R")
  expect_match(read_lines("xyz_reprex.md"), "5:1", all = FALSE, fixed = TRUE)
  write_lines("max(4:6)", "xyz.R")
  reprex(input = "xyz.R")
  expect_match(read_lines("xyz_reprex.md"), "5:1", all = FALSE, fixed = TRUE)
})

test_that(".Rprofile local to reprex target directory is consulted & messaged", {
  local_temp_wd("reprextests-aaa-")
  cat("x <- 'aaa'\n", file = ".Rprofile")
  cat("x\n", file = "foo.R")
  aaa_foo <- path_abs("foo.R")

  local_temp_wd("reprextests-bbb-")
  cat("x <- 'bbb'\n", file = ".Rprofile")

  local_reprex_loud()
  msg <- capture_messages(
    out <- reprex(x, wd = ".", advertise = FALSE)
  )
  expect_match(out, "bbb", all = FALSE)
  expect_messages_to_include(
    msg,
    c("Local '[.]Rprofile' detected", "bbb")
  )

  msg <- capture_messages(
    out <- reprex(input = aaa_foo, wd = ".", advertise = FALSE)
  )
  expect_match(out, "aaa", all = FALSE)
  expect_messages_to_include(
    msg,
    c("Local '[.]Rprofile' detected", "aaa")
  )
})

test_that("local .Rprofile reporting responds to venue", {
  expect_snapshot(rprofile_alert("gh"))
  expect_snapshot(rprofile_alert("r"))
})

test_that("local .Rprofile not reported when it's not there", {
  local_reprex_loud()
  msg <- capture_messages(
    reprex(1 + 1, advertise = FALSE)
  )
  expect_false(any(grepl(".Rprofile", msg, fixed = TRUE)))
})

readLines <- function(...) {
  cli::cli_abort(
    "Inside {.pkg reprex}, we use {.fun read_lines}, not {.fun readLines}, for UTF-8 reasons.",
    .internal = TRUE
  )
}

writeLines <- function(...) {
  cli::cli_abort(
    "Inside {.pkg reprex}, we use {.fun write_lines}, not {.fun writeLines}, for UTF-8 reasons.",
    .internal = TRUE
  )
}

read_lines <- function(path, n = -1L) {
  if (is.null(path)) {
    return(NULL)
  }
  base::readLines(path, n = n, encoding = "UTF-8", warn = FALSE)
}

write_lines <- function(text, path, sep = "\n") {
  path <- file(path, open = "wb")
  withr::defer(close(path))
  base::writeLines(enc2utf8(text), con = path, sep = sep, useBytes = TRUE)
}

locate_input <- function(input) {
  if (is.null(input)) {
    if (reprex_clipboard()) {
      return("clipboard")
    }
    if (in_rstudio()) {
      return("selection")
    } else {
      return(NULL)
    }
  }

  if (is_path(input)) {
    "path"
  } else {
    "input"
  }
}

retrofit_files <- function(infile = NULL, wd = NULL, outfile = deprecated()) {
  if (!lifecycle::is_present(outfile)) {
    return(list(infile = infile, wd = wd))
  }
  # `outfile` was specified

  if (!is.null(wd)) {
    lifecycle::deprecate_warn(
      when = "2.0.0",
      what = "reprex(outfile)",
      with = "reprex(wd)"
    )
    return(list(infile = infile, wd = wd))
  }
  # `wd` was not specified

  # cases to consider
  # infile      outfile
  # NULL        NA
  # "foo.R"     NA
  # "foo/bar.R" NA
  # NULL        "foo"
  # NULL        "foo/bar"
  # "foo/bar.R" "blah"

  if (is.na(outfile)) {
    # historically, this was a good way to say "reprex in working directory"
    if (is.null(infile)) {
      lifecycle::deprecate_warn(
        when = "2.0.0",
        what = "reprex(outfile)",
        details = 'Use `reprex(wd = ".")` instead of `reprex(outfile = NA)`.'
      )
      return(list(infile = NULL, wd = "."))
    }
    lifecycle::deprecate_warn(
      when = "2.0.0",
      what = "reprex(outfile)",
      details =  "Working directory will be derived from `input`."
    )
    return(list(infile = infile, wd = NULL))
  }
  # `outfile` is string

  if (is.null(infile)) {
    lifecycle::deprecate_warn(
      when = "2.0.0",
      what = "reprex(outfile)",
      details =  c(
        "To control output filename, provide a filepath to `input`.",
        "Only taking working directory from `outfile`."
      )
    )
    return(list(infile = NULL, wd = path_dir(outfile)))
  }
  # both `infile` and `outfile` are strings

  lifecycle::deprecate_warn(
    when = "2.0.0",
    what = "reprex(outfile)",
    details = "Working directory and output filename will be determined from `input`."
  )
  list(infile = infile, wd = NULL)
}

plan_files <- function(infile = NULL, wd = NULL, outfile = deprecated()) {
  tmp <- retrofit_files(infile, wd, outfile)
  infile <- tmp$infile
  wd <- tmp$wd
  chatty <- !is.null(infile) || !is.null(wd) || lifecycle::is_present(outfile)

  if (!is.null(infile) && !is.null(wd)) {
    reprex_warning(
      "Ignoring {.code wd}, working directory is determined by {.code input}"
    )
    wd <- NULL
  }

  list(
    chatty = chatty,
    filebase = make_filebase(infile, wd)
  )
}

# we'll index into the (shuffled) adjective-animal list with this
aa_i <- (function() {
  i <- 0
  function() {
    i <<- i + 1
    i
  }
})()

reprex_aa <- function() adjective_animal[[aa_i()]]

reprex_default_filebase <- function(in_temp_dir) {
  # ugly but (probably) unique
  ugly_dir <- file_temp("reprex-")
  # human-friendly and unique within an R session, at least for first n reprexes
  aa <- reprex_aa()
  if (in_temp_dir) {
    # wd not specified --> reprex in sub-directory of session temp directory
    # example: /private/var/.../.../.../reprex-98183d9c49-prior-boa/prior-boa
    target_dir <- path_real(dir_create(glue("{ugly_dir}-{aa}")))
    path(target_dir, aa)
  } else {
    # no infile, wd is specified
    # example: prior-boa
    aa
  }
}

make_filebase <- function(infile = NULL, wd = NULL) {
  if (is.null(infile)) {
    if (is.null(wd)) {
      reprex_default_filebase(in_temp_dir = TRUE)
    } else {
      if (wd == ".") {
        reprex_default_filebase(in_temp_dir = FALSE)
      } else {
        path(wd, reprex_default_filebase(in_temp_dir = FALSE))
      }
    }
  } else {
    path_ext_remove(infile)
  }
}

add_suffix <- function(x, suffix = "reprex") {
  if (!nzchar(suffix)) {
    return(x)
  }

  orig_ext <- path_ext(x)
  filebase <- path_ext_remove(x)
  suffix <- paste0("_", suffix)
  if (endsWith(filebase, suffix)) {
    return(x)
  }
  path_ext_set(paste0(filebase, suffix), orig_ext)
}

path_mutate <- function(path, suffix = "", ext = NULL) {
  if (nzchar(suffix)) {
    path <- add_suffix(path, suffix)
  }
  if (!is.null(ext)) {
    path <- path_ext_set(path, ext)
  }
  path
}

r_file <- function(path) {
  path_mutate(path, suffix = "reprex", ext = "R")
}

r_file_clean <- function(path) {
  path_mutate(path, suffix = "clean", ext = "R")
}

r_file_rendered <- function(path) {
  path_mutate(path, suffix = "r", ext = "R")
}

md_file <- function(path) {
  path_mutate(path, ext = "md")
}

md_file_slack <- function(path) {
  path_mutate(path, suffix = "slack", ext = "md")
}

std_file <- function(path) {
  path_mutate(path, suffix = "std_out_err", ext = "txt")
}

html_file <- function(path) {
  path_mutate(path, ext = "html")
}

rtf_file <- function(path) {
  path_mutate(path, ext = "rtf")
}

rmd_file <- function(path) {
  path_mutate(path, suffix = "reprex", ext = "Rmd")
}

preview_file <- function(path) {
  path_mutate(path, suffix = "preview", ext = "html")
}

would_clobber <- function(path) {
  if (!file_exists(path)) {
    return(FALSE)
  }
  if (!is_interactive()) {
    return(TRUE)
  }
  reprex_path("Oops, file already exists:", path, type = "warning")
  nope("Carry on and overwrite it?")
}

# goals in order of preference:
# 1. put reprex output on clipboard
# 2. open file for manual copy
expose_reprex_output <- function(reprex_file, rtf = FALSE) {
  if (reprex_clipboard()) {
    if (rtf && is_windows()) {
      write_clip_windows_rtf(reprex_file)
    } else {
      clipr::write_clip(read_lines(reprex_file))
    }
    reprex_success("Reprex output is on the clipboard.")
    return(invisible())
  }

  if (!is_interactive()) {
    return(invisible())
  }

  if (rtf) {
    reprex_path("Attempting to open RTF output file:", reprex_file)
    utils::browseURL(reprex_file)
    return(invisible())
  }

  reprex_path("Opening output file for manual copy:", reprex_file)
  if (in_rstudio()) {
    rstudio_open_and_select_all(reprex_file)
  } else {
    withr::defer_parent(utils::file.edit(reprex_file))
  }
  invisible()
}

rstudio_open_and_select_all <- function(path) {
  rstudioapi::navigateToFile(path)
  # navigateToFile() is not synchronous, hence the while loop & sleep
  # it takes an indeterminate amount of time for the active source file to
  # actually be 'path'
  #
  # DO NOT fiddle with this unless you also do thorough manual tests,
  # including on RSP, Cloud, using reprex() and the addin and the gadget
  ct <- rstudioapi::getSourceEditorContext()
  i <- 0
  while (ct$path == "" || path_real(ct$path) != path_real(path)) {
    if (i > 4) break
    i <- i + 1
    Sys.sleep(1)
    ct <- rstudioapi::getSourceEditorContext()
  }
  rg <- rstudioapi::document_range(
    start = rstudioapi::document_position(1, 1),
    end   = rstudioapi::document_position(Inf, Inf)
  )
  rstudioapi::setSelectionRanges(rg, id = ct$id)
  invisible()
}

test_that("make_filebase() defaults to 'reprex' inside a dir inside tempdir", {
  x <- make_filebase(outfile = NULL, infile = NULL)
  expect_equal(fs::path_file(x), "reprex")
  expect_match(fs::path_file(fs::path_dir(x)), "^reprex")
  temp <- fs::path_real(path_temp())
  expect_identical(fs::path_common(c(x, temp)), temp)
})

test_that("make_filebase(outfile = NA) fabricates filebase in wd", {
  x <- make_filebase(outfile = NA, infile = NULL)
  expect_match(fs::path_file(x), "^reprex")
  expect_equal(fs::path_dir(x), ".")
})

test_that("make_filebase() works from relative infile, outfile", {
  expect_equal(make_filebase(outfile = NA, infile = "abcde"), "abcde")
  expect_equal(make_filebase(outfile = "abcde"), "abcde")
})

test_that("make_filebase() works from absolute infile, outfile", {
  x <- make_filebase(outfile = NA, infile = fs::path_temp("abcde"))
  expect_match(fs::path_file(x), "^abcde")
  expect_equal(fs::path_dir(x), as.character(path_temp()))

  x <- make_filebase(outfile = fs::path_temp("abcde"))
  expect_match(fs::path_file(x), "^abcde")
  expect_equal(fs::path_dir(x), as.character(path_temp()))
})

## https://github.com/tidyverse/reprex/issues/152
test_that("keep.source is TRUE inside the reprex()", {
  skip_on_cran()
  ret <- reprex(input = "getOption('keep.source')\n")
  expect_match(ret, "TRUE", all = FALSE)
})

test_that("reprex() works with code that deals with srcrefs", {
  skip_on_cran()
  ret <- reprex(
    input = "utils::getParseData(parse(text = 'a'))\n",
    advertise = FALSE
  )
  expect_snapshot_output(print(ret))
})

## https://github.com/tidyverse/reprex/issues/183
test_that("reprex() doesn't leak files by default", {
  skip_on_cran()
  reprex(base::writeLines("test", "test.txt"), advertise = FALSE)
  ret <- reprex(base::readLines("test.txt"), advertise = FALSE)
  expect_match(ret, "cannot open file 'test.txt'", all = FALSE)
})

test_that("rmarkdown::render() context is trimmed from rlang backtrace", {
  skip_on_cran()
  input <- c(
    "f <- function() rlang::abort('foo')",
    "f()",
    "rlang::last_error()",
    "rlang::last_trace()"
  )
  ret <- reprex(input = input, advertise = FALSE)
  expect_false(any(grepl("tryCatch", ret)))
  expect_false(any(grepl("rmarkdown::render", ret)))
})

test_that("rlang::last_error() and last_trace() work", {
  skip_on_cran()

  input <- c(
    "f <- function() rlang::abort('foo')",
    "f()",
    # as of rlang 1.0.0 (2022-01-26)
    # https://github.com/r-lib/rlang/blame/0e2718639d7b87effbf47cf17d6e0288a69454e6/NEWS.md#L350-L354
    "#'", # currently, this must be in a new chunk
    "rlang::last_error()",
    "rlang::last_trace()"
  )
  ret <- reprex(input = input, advertise = FALSE)
  m <- match("rlang::last_error()", ret)
  expect_false(grepl("Error", ret[m + 1]))
  m <- match("rlang::last_trace()", ret)
  expect_false(grepl("Error", ret[m + 1]))
})

test_that("reprex() works even if user uses fancy quotes", {
  skip_on_cran()
  withr::local_options(list(useFancyQuotes = TRUE))
  # use non-default venue to force some quoted yaml to be written
  expect_error_free(reprex(1, venue = "R"))
})

test_that("reprex() errors for an R crash, by default", {
  # TODO: consider switching to expect_snapshot() after switch to 3e
  code <- 'utils::getFromNamespace("crash", "callr")()\n'
  expect_error(reprex(input = code), "crash")
})

test_that("reprex() copes with an R crash, when `std_out_err = TRUE`", {
  # TODO: consider switching to expect_snapshot() after switch to 3e
  code <- 'utils::getFromNamespace("crash", "callr")()\n'
  expect_error_free(
    out <- reprex(input = code, std_out_err = TRUE)
  )
  skip_on_os("windows")
  expect_match(out, "crash", all = FALSE)
  expect_match(out, "segfault", all = FALSE)
  expect_match(out, "Traceback", all = FALSE)
})

test_that("retrofit_files() works", {
  local_reprex_loud()
  withr::local_options(lifecycle_verbosity = "warning")

  # when `outfile` is not specified, there's nothing to do
  expect_equal(
    retrofit_files(infile = NULL, wd = NULL),
    list(infile = NULL, wd = NULL)
  )
  expect_equal(
    retrofit_files(infile = "foo.R", wd = "whatever"),
    list(infile = "foo.R", wd = "whatever")
  )

  # `wd` takes precedence over `outfile` and we say something
  expect_snapshot_warning(
    x <- retrofit_files(wd = "this", outfile = "that")
  )
  expect_equal(x, list(infile = NULL, wd = "this"))

  # `outfile = NA` morphs into `wd = "."`, if no `infile`
  expect_snapshot_warning(
    x <- retrofit_files(outfile = NA)
  )
  expect_equal(x, list(infile = NULL, wd = "."))

  # only `wd` is salvaged from `outfile = some/path/blah` and we mention `input`
  expect_snapshot_warning(
    x <- retrofit_files(outfile = "some/path/blah")
  )
  expect_equal(x, list(infile = NULL, wd = "some/path"))

  # `infile` takes over much of `outfile`'s previous role
  expect_snapshot_warning(
    x <- retrofit_files(infile = "a/path/foo.R", outfile = NA)
  )
  expect_equal(x, list(infile = "a/path/foo.R", wd = NULL))
  expect_snapshot_warning(
    x <- retrofit_files(infile = "a/path/foo.R", outfile = "other/path/blah")
  )
    expect_equal(x, list(infile = "a/path/foo.R", wd = NULL))
})

# root cause of
# https://github.com/tidyverse/reprex/issues/379
test_that("we don't add a suffix more than once", {
  x <- "blah_r.R"
  expect_equal(x, add_suffix(x, suffix = "r"))
})

test_that("make_filebase() works with no input", {
  x <- make_filebase(infile = NULL, wd = NULL)

  filebase_base <- path_file(x)
  # adjective-animal
  expect_match(filebase_base, "^[a-z]+[-][a-z]+$")

  filebase_parent <- path_file(path_dir(x))
  # reprex-[hexademical from tempfile()]-adjective-animal
  expect_match(filebase_parent, "^reprex-[[:xdigit:]]+[-][a-z]+[-][a-z]+$")

  temp <- path_real(path_temp())
  expect_identical(path_common(c(x, temp)), temp)
})

test_that("make_filebase(wd = '.') works", {
  x <- make_filebase(infile = NULL, wd = ".")
  expect_equal(path_dir(x), ".")
  # adjective-animal
  expect_match(x, "^[a-z]+[-][a-z]+$")
})

test_that("make_filebase(wd = 'blah') works", {
  wd <- path_temp("xyz")
  x <- make_filebase(infile = NULL, wd = wd)
  expect_equal(path_file(path_dir(x)), "xyz")
  # adjective-animal
  expect_match(path_file(x), "^[a-z]+[-][a-z]+$")
})

test_that("make_filebase(infile = 'blah') works", {
  # relative path
  expect_equal(make_filebase(infile = "foo.R"), "foo")
  expect_equal(make_filebase(infile = "blah/foo.R"), "blah/foo")

  # `wd` should be ignored
  expect_equal(make_filebase(infile = "foo.R", wd = "wut"), "foo")
  expect_equal(make_filebase(infile = "blah/foo.R", wd = "wut"), "blah/foo")

  # absolute path
  infile <- path_temp("abcde.R")
  x <- make_filebase(infile = infile)
  expect_equal(path_file(x), "abcde")
  expect_equal(path_dir(x), path_dir(infile))
})

reprex_highlight <- function(rout_file, reprex_file, arg_string = NULL) {
  arg_string <- arg_string %||% highlight_args()
  cmd <- paste0(
    "highlight ",
    " -i ", shQuote(rout_file),
    " --out-format=rtf --no-trailing-nl --encoding=UTF-8",
    arg_string,
    " -o ", shQuote(reprex_file)
  )
  if (is_windows()) {
    res <- shell(cmd)
  } else {
    res <- system(cmd)
  }
  if (res > 0) {
    # I am OK with a non-exported function appearing in this error.
    # This whole feature is "use at your own risk".
    cli::cli_abort(
      "Call to the {.pkg highlight} command line tool was unsuccessful."
    )
  }
  res
}

rtf_requires_highlight <- function(venue) {
  if (venue == "rtf" && !highlight_found()) {
    # I am OK with a non-exported function appearing in this error.
    # This whole feature is "use at your own risk".
    cli::cli_abort(c(
      "The {.pkg highlight} command line tool doesn't appear to be installed.",
      '{.code venue = "rtf"} is only supported if R can find {.pkg highlight}.'
    ))
  }
  invisible(venue)
}

highlight_found <- function() Sys.which("highlight") != ""

highlight_args <- function() {
  hl_style  <-         getOption("reprex.highlight.hl_style", "darkbone")
  font      <- shQuote(getOption("reprex.highlight.font", "Courier Regular"))
  font_size <-         getOption("reprex.highlight.font_size", 50)
  other     <-         getOption("reprex.highlight.other", "")

  paste0(
    " --style ",     hl_style,
    " --font ",      font,
    " --font-size ", font_size,
    " ", other
  )
}

#' reprex options
#'
#' @description
#' Some [reprex()] behaviour can be controlled via an option, providing a way
#' for the user to set personal defaults. The pattern for such option names is
#' `reprex.<arg>`, where `<arg>` is an argument of [reprex()]. Here are the main
#' ones:
#'   * `reprex.advertise`
#'   * `reprex.session_info` (previously, `reprex.si`)
#'   * `reprex.style`
#'   * `reprex.html_preview` (previously, `reprex.show`)
#'   * `reprex.comment`
#'   * `reprex.tidyverse_quiet`
#'   * `reprex.std_out_err`
#'
#' A few more options exist, but are only relevant to specific situations:
#'   * `reprex.venue`: Can be used to control the `venue` used by the
#'   [reprex_selection()] addin.
#'   * `reprex.current_venue`: Read-only option that is set during
#'   [reprex_render()]. Other packages that want to generate reprex-compatible
#'   output can consult it via `getOption("reprex.current_venue")`, if they want
#'   to tailor their output to the `venue`.
#'   * `reprex.clipboard`: When `FALSE`, reprex makes no attempt to access the
#'   user's clipboard, ever. This exists mostly for internal use, i.e. we set it
#'   to `FALSE` when we detect use from RStudio Server. But a user could set
#'   this to `FALSE` to explicitly opt-out of clipboard functionality. A Linux
#'   user with no intention of installing `xclip` or `xsel` might also do this.
#'   * `reprex.highlight.hl_style`: Only relevant to `venue = "rtf`. Details are
#'     in the article
#'     [reprex venue RTF](https://reprex.tidyverse.org/articles/articles/rtf.html).
#'   * `reprex.highlight.font`: See above.
#'   * `reprex.highlight.font_size`: See above.
#'   * `reprex.highlight.other`: See above.
#'
#' Here's code you could put in `.Rprofile` to set reprex options. It would be
#' rare to want non-default behaviour for all of these! We only do so here for
#' the sake of exposition:
#' ```
#' options(
#'   reprex.advertise       = FALSE,
#'   reprex.session_info    = TRUE,
#'   reprex.style           = TRUE,
#'   reprex.html_preview    = FALSE,
#'   reprex.comment         = "#;-)",
#'   reprex.tidyverse_quiet = FALSE,
#'   reprex.std_out_err     = TRUE,
#'   reprex.venue           = "html", # NOTE: only affects reprex_selection()!
#'   reprex.highlight.hl_style  = "acid", # NOTE: only affects RTF venue
#'   reprex.highlight.font      = "Andale Mono Regular",
#'   reprex.highlight.font_size = 35,
#'   reprex.highlight.other     = "--line-numbers"
#' )
#' ```
#' The function `usethis::edit_r_profile()` is handy for creating and/or opening
#' your `.Rprofile`.
#'
#' @section Explaining the `opt()` helper:
#' Arguments that appear like so in [reprex()]:
#' ```
#' reprex(..., arg = opt(DEFAULT), ...)
#' ````
#' get their value according to this logic:
#' ```
#' user-specified value or, if not given,
#'   getOption("reprex.arg") or, if does not exist,
#'     DEFAULT
#' ```
#' It's shorthand for:
#' ```
#' f(..., arg = getOption("reprex.arg", DEFAULT), ...)
#' ```
#' This is not an exported function and should not be called directly.
#'
#' @name reprex_options
#' @aliases opt
NULL

optionally <- function(x, opt_name = NA_character_) {
  if (!is.na(opt_name)) {
    attr(x, "opt_name") <- opt_name
  }
  attr(x, "optional") <- TRUE
  x
}

opt <- optionally

arg_option <- function(arg) {
  arg_expr <- enexpr(arg)
  if (!is_symbol(arg_expr)) {
    cli::cli_abort(
      "{.fun arg_option} expects a symbol.",
      .internal = TRUE
    )
  }

  opt_name <- attr(arg, "opt_name") %||% make_opt_name(as_string(arg_expr))

  if (is_optional(arg)) {
    getOption(opt_name) %||% de_opt(arg)
  } else {
    arg
  }
}

is_optional <- function(x) isTRUE(attr(x, "optional"))

de_opt <- function(x) {
  attr(x, "optional") <- NULL
  attr(x, "opt_name") <- NULL
  x
}

make_opt_name <- function(x) {
  pkg_name <- tryCatch(ns_env_name(), error = function(e) NULL)
  paste(c(pkg_name, x), collapse = ".")
}

---
title: "Magic reprex"
author: "[Nicholas Tierney](http://www.njtierney.com)"
---

> This article has been adapted from a post on [Nick Tierney's blog](https://www.njtierney.com/post/2017/01/11/magic-reprex/) to help spread the good word about reprex! :) - **Nick Tierney**

Making reproducible examples can be hard.

There's a lot of things you need to consider. Like, making sure your environment is clean, the right packages are loaded, the code is formatted nicely, and images are the right resolution and dimension.

Getting all of these ducks lined up can sometimes take a couple of minutes, if you have a nice tightly defined problem. Other times, it can take much, much longer. Say for example, if you had a variable loaded in your environment that you forgot how to create, and you didn't realise it was so important and OH GOD HOW DID I EVEN DO THIS? Ahem.

So, after some sensible swearing and frustration, you might ask yourself something like:

> Surely there's an easy, reproducible way to ... make reproducible examples?

Enter the R package reprex (on [CRAN](https://cran.r-project.org/package=reprex), on [GitHub](https://github.com/tidyverse/reprex)), by [Jenny Bryan](http://www.github.com/jennybc).

In the fashion of the times right now, I'll illustrate reprex with a gif, which shows the following things:

1. Take your code, copy it to the clipboard.
2. Run `reprex::reprex()`, wait a moment for the code to execute.
3. `reprex::reprex()` creates a html preview of your code.
4. A markdown chunk is also copied to the clipboard.

<p>
<div style="height: auto">
<img src="https://raw.githubusercontent.com/tidyverse/reprex/main/img/reprex-copy.gif" alt="Gif of reprex copying code, running reprex, and demonstrating the html preview of the code text" style="width: 100%;max-height: 100%" />
</div>
</p>


This means you can copy this to say, GitHub (`reprex(venue = "gh")`) (the default) to help you file an issue:


<p>
<div style="height: auto">
<img src="https://raw.githubusercontent.com/tidyverse/reprex/main/img/reprex-github.gif" alt="Gif of reprex copying code, running reprex, and demonstrating the html preview of the code text" style="width: 100%;max-height: 100%" />
</div>
</p>

There is also an option for Stack Overflow (`reprex(venue = "so")`).

How does this work? `reprex()` takes the section of copied code and then runs `rmarkdown::render()` on it, which starts a fresh environment, and runs the code.
This means that `reprex()` will fail hard and fail early if your examples aren't reproducible. If you don't have the correct library loaded, it will show the error messages.

<p>
<div style="height: auto">
<img src="https://raw.githubusercontent.com/tidyverse/reprex/main/img/reprex-fail-1.gif" alt="Gif of reprex copying code, running reprex, and demonstrating the html preview of the code text" style="width: 100%;max-height: 100%" />
</div>
</p>

But then when you fix that, by adding library calls to [`visdat`](https://github.com/njtierney/visdat), [`ggplot2`](https://github.com/tidyverse/ggplot2) and [`naniar`](https://github.com/njtierney/naniar), and you get what you want:

<p>
<div style="height: auto">
<img src="https://raw.githubusercontent.com/tidyverse/reprex/main/img/reprex-success-1.gif" alt="Gif of reprex copying code, running reprex, and demonstrating the html preview of the code text using visdat, naniar, and ggplot2 packages" style="width: 100%;max-height: 100%" />
</div>
</p>

So you know that when you run `reprex()`, and it gives you a reproducible example, it will work!

Also worth mentioning is that rather magically, `reprex()` also handles images. I didn't even notice this at first. It was just one of those things where "it just works". `reprex()` handles this by uploading them to imgur using the `knitr::imgur_upload()` function, which creates markdown links to the images.

so

```r

library(tidyverse)

# run a lm

lm_fit <- lm(Sepal.Length ~ . , data = iris)

# use broom to clean it up

library(broom)

lm_augmented <- augment(lm_fit)

head(lm_augmented)

# Explore the fit of the model

ggplot(data = lm_augmented,
   aes(x = .fitted,
       y = Sepal.Length)) +
  geom_point()

```

gets turned into


```
library(tidyverse)
#> Loading tidyverse: ggplot2
#> Loading tidyverse: tibble
#> Loading tidyverse: tidyr
#> Loading tidyverse: readr
#> Loading tidyverse: purrr
#> Loading tidyverse: dplyr
#> Conflicts with tidy packages ----------------------------------------------
#> filter(): dplyr, stats
#> lag():    dplyr, stats

# run a lm

lm_fit <- lm(Sepal.Length ~ . , data = iris)

# use broom to clean it up

library(broom)

lm_augmented <- augment(lm_fit)

head(lm_augmented)
#>   Sepal.Length Sepal.Width Petal.Length Petal.Width Species  .fitted
#> 1          5.1         3.5          1.4         0.2  setosa 5.004788
#> 2          4.9         3.0          1.4         0.2  setosa 4.756844
#> 3          4.7         3.2          1.3         0.2  setosa 4.773097
#> 4          4.6         3.1          1.5         0.2  setosa 4.889357
#> 5          5.0         3.6          1.4         0.2  setosa 5.054377
#> 6          5.4         3.9          1.7         0.4  setosa 5.388886
#>      .se.fit      .resid       .hat    .sigma      .cooksd  .std.resid
#> 1 0.04479188  0.09521198 0.02131150 0.3077918 3.570856e-04  0.31367294
#> 2 0.05514933  0.14315645 0.03230694 0.3076565 1.251718e-03  0.47429637
#> 3 0.04690495 -0.07309695 0.02336968 0.3078349 2.317688e-04 -0.24106926
#> 4 0.05135928 -0.28935683 0.02801904 0.3069173 4.396118e-03 -0.95656076
#> 5 0.04736842 -0.05437691 0.02383379 0.3078627 1.309299e-04 -0.17937436
#> 6 0.05592364  0.01111427 0.03322050 0.3078956 7.772793e-06  0.03684044

# Explore the fit of the model, looking

ggplot(data = lm_augmented,
       aes(x = .fitted,
           y = Sepal.Length)) +
  geom_point()

![](http://i.imgur.com/h72ZEKG.png)
```

You can even show the results of the `session_info()`, for maximum reproducibility, by adding the argument: `reprex::reprex(session_info = TRUE)`, which adds a super nifty drop down box for the session info.


<p>
<div style="height: auto">
<img src="https://raw.githubusercontent.com/tidyverse/reprex/main/img/reprex-si.gif" alt="Gif of reprex copying code, running reprex, showing option SI = TRUE" style="width: 100%;max-height: 100%" />
</div>
</p>

I really like these sorts of small tools in R. Squinting at a distance you might make the mistake of thinking:

> so what, it does this __one little thing__

But then when you're actually up close, you realise this means that you don't have to bend your brain around doing this thing, which can range from mildly annoying to mostly infuriating.

You save time and energy, and can focus more on **doing what you want to do**, and less on **fighting against interruptions**. This comic by [Jason Heeris](http://heeris.id.au/2013/this-is-why-you-shouldnt-interrupt-a-programmer/) illustrates this point nicely, I think:

<p>
<div style="height: auto">
<img src="https://raw.githubusercontent.com/tidyverse/reprex/main/img/ProgrammerInterrupted.png" alt="Why you shouldn't interrupt a programmer" style="max-height: 100%"/>
</div>
</p>


Once again thank you to Jenny Bryan for making our lives easier. Jenny will be joining the RStudio team this year and I am really looking forward to seeing what she gets up to.

Read more about reprex on github [here](http://www.github.com/tidyverse/reprex#readme).

---
title: "Reprex do's and don'ts"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Reprex do's and don'ts}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

If you're asking for R help, reporting a bug, or requesting a new feature, you're more likely to succeed if you include a good reprex.

## Main requirements

**Use the smallest, simplest, most [built-in data](https://stat.ethz.ch/R-manual/R-patched/library/datasets/html/00Index.html) possible.**

  - Think: `iris` or `mtcars`. Bore me.
  - If you must make some objects, minimize their size and complexity.
  - Many of the functions and packages you already use offer a way to create a small data frame "inline":
    - `read.table()` and friends have a `text` argument. Example:

      ```{r eval = FALSE}
      read.csv(text = "a,b\n1,2\n3,4")
      #>   a b
      #> 1 1 2
      #> 2 3 4
      ```
      
    - `tibble::tribble()` lets you use a natural and readable layout. Example:
    
      ```{r eval = FALSE}
      tibble::tribble(
        ~ a, ~ b,
          1,   2,
          3,   4
      )
      #> # A tibble: 2 x 2
      #>       a     b
      #>   <dbl> <dbl>
      #> 1     1     2
      #> 2     3     4
      ```
            
  - Get just a bit of something with `head()` or by indexing with the result of `sample()`. If anything is random, consider using `set.seed()` to make it repeatable.
  - The [datapasta package](https://milesmcbain.github.io/datapasta/) can generate code for `data.frame()`, `tibble::tribble()`, or `data.table::data.table()` based on an existing R data frame. For example, a call to `tribble_format(head(ChickWeight, 3))` leaves this on the clipboard, ready to paste into your reprex:
  
    ```{r eval = FALSE}
    tibble::tribble(
     ~weight, ~Time, ~Chick, ~Diet,
          42,     0,    "1",   "1",
          51,     2,    "1",   "1",
          59,     4,    "1",   "1"
    )
    ```
  - `dput()` is a decent last resort, i.e. if you simply cannot make do with built-in or simulated data or inline data creation in a more readable format. But `dput()` output is not very human-readable. Avoid if at all possible.
  - Look at official examples and try to write in that style. Consider adapting one.

**Include commands on a strict "need to run" basis.**

  - Ruthlessly strip out anything unrelated to the specific matter at hand.
  - Include every single command that is required, e.g. loading specific packages via `library(foo)`.

**Consider including so-called "session info"**, i.e. your OS and versions of R and add-on packages, if it's conceivable that it matters.

  - Use `reprex(..., session_info = TRUE)` for this.

**Whitespace rationing is not in effect.**

  - Use good [coding style](https://style.tidyverse.org).
  - Use `reprex(..., style = TRUE)` to request automated styling of your code.

**Pack it in, pack it out, and don't take liberties with other people's computers.** You are asking people to run this code!

  - Don't start with `rm(list = ls())`. It is anti-social to clobber other people's workspaces.
  - Don't start with `setwd("C:\Users\jenny\path\that\only\I\have")`, because it won't work on anyone else's computer.
  - Don't mask built-in functions, i.e. don't define a new function named `c` or `mean`.
  - If you change options, store original values at the start, do your thing, then restore them:
  
    ```{r eval = FALSE}
    opar <- par(pch = 19)
    <blah blah blah>
    par(opar)
    ```
  - If you create files, delete them when you're done:
  
    ```{r eval = FALSE}
    write(x, "foo.txt")
    <blah blah blah>
    file.remove("foo.txt")
    ```
  - Don't delete files or objects that you didn't create in the first place.
  - Take advantage of R's built-in ability to create temporary files and directories. Read up on [`tempfile()` and `tempdir()`](https://stat.ethz.ch/R-manual/R-patched/library/base/html/tempfile.html).
  
## This seems like a lot of work!
  
Yes, creating a great reprex requires work. You are asking other people to do work too. It's a partnership.

80% of the time you will solve your own problem in the course of writing an excellent reprex. YMMV.
  
The remaining 20% of the time, you will create a reprex that is more likely to elicit the desired behavior in others.

## Further reading:

[How to make a great R reproducible example?](https://stackoverflow.com/questions/5963269/how-to-make-a-great-r-reproducible-example/16532098) thread on StackOverflow

## Package philosophy

The reprex code:

  * Must run and, therefore, should be run **by the person posting**. No faking it.

  * Should be easy for others to digest, so **they don't necessarily have to run it**. You are encouraged to include selected bits of output. :scream:

  * Should be easy for others to copy + paste + run, **if and only if they so choose**. Don't let inclusion of output break executability.

Accomplished like so:

  * Use `rmarkdown::render()` to run the code and capture output that you would normally see on your screen. This is done in a separate R process, via [callr](https://callr.r-lib.org), to guarantee it is self-contained.

  * Use chunk option `comment = "#>"` to include the output while retaining executability.

---
output:
  reprex::reprex_document:
    venue: "gh"
    advertise: FALSE
    session_info: TRUE
    style: TRUE
    comment: "#;-)"
    tidyverse_quiet: FALSE
    std_out_err: TRUE
knit: reprex::reprex_render
---

This template demonstrates many of the bells and whistles of the `reprex::reprex_document()` output format. The YAML sets many options to non-default values, such as using `#;-)` as the comment in front of output.

## Code style

Since `style` is `TRUE`, this difficult-to-read code (look at the `.Rmd` source file) will be restyled according to the Tidyverse style guide when it's rendered. Whitespace rationing is not in effect!

```{r}
x=1;y=2;z=x+y;z
```

## Quiet tidyverse

The tidyverse meta-package is quite chatty at startup, which can be very useful in exploratory, interactive work. It is often less useful in a reprex, so by default, we suppress this.

However, when `tidyverse_quiet` is `FALSE`, the rendered result will include a tidyverse startup message about package versions and function masking.

```{r, eval = requireNamespace("tidyverse", quietly = TRUE)}
library(tidyverse)
```

## Chunks in languages other than R

Remember: knitr supports many other languages than R, so you can reprex bits of code in Python, Ruby, Julia, C++, SQL, and more. Note that, in many cases, this still requires that you have the relevant external interpreter installed.

Let's try Python!

```{python, eval = Sys.which("python") != "", python.reticulate = requireNamespace("reticulate", quietly = TRUE)}
x = 'hello, python world!'
print(x.split(' '))
```

And bash!

```{bash, eval = Sys.which("bash") != ""}
echo "Hello Bash!";
pwd;
ls | head;
```

Write a function in C++, use Rcpp to wrap it and ...

```{Rcpp, eval = requireNamespace("Rcpp", quietly = TRUE)}
#include <Rcpp.h>
using namespace Rcpp;

// [[Rcpp::export]]
NumericVector timesTwo(NumericVector x) {
  return x * 2;
}
```

then immediately call your C++ function from R!

```{r, eval = requireNamespace("Rcpp", quietly = TRUE)}
timesTwo(1:4)
```

## Standard output and error

Some output that you see in an interactive session is not actually captured by rmarkdown, when that same code is executed in the context of an `.Rmd` document. When `std_out_err` is `TRUE`, `reprex::reprex_render()` uses a feature of `callr:r()` to capture such output and then injects it into the rendered result.

Look for this output in a special section of the rendered document (and notice that it does not appear right here).

```{r}
system2("echo", args = "Output that would normally be lost")
```

## Session info

Because `session_info` is `TRUE`, the rendered result includes session info, even though no such code is included here in the source document. 

---
title: "reprex venue RTF"
---

Use `reprex_rtf()` to get output in [Rich Text Format](https://en.wikipedia.org/wiki/Rich_Text_Format).
It is a convenience wrapper around `reprex(venue = "rtf")`.
This is useful for creating syntax highlighted code snippets -- rendered or not -- to paste into software such as Keynote, PowerPoint, or Word.

Why is RTF special?
Although Pandoc can produce RTF and, therefore, so can `rmarkdown::render()` and `reprex::reprex()`, Pandoc's syntax highlighting is unavailable for RTF output.
For now, we've got to get the functionality from somewhere else.
Syntax highlighting is a non-issue for `venue = "gh"` (GitHub) and `venue = "so"` (Stack Overflow), because the target sites take care of that when the Markdown content is displayed.

<p>
<div style="height: auto">
<img src="https://raw.githubusercontent.com/tidyverse/reprex/main/img/reprex_rtf_jennybryan.gif" alt="Gif of setting options to control highlight and using reprex with the RTF venue" style="width: 100%;max-height: 100%" />
</div>
</p>

The "rtf" venue was introduced in reprex v0.2.1 and is experimental.
The [highlight](http://www.andre-simon.de/doku/highlight/en/highlight.php) command line tool must (1) be installed and (2) be on your `PATH`.
People have successfully used `reprex_rtf()` on macOS and Windows (specifically, Windows 10 + Windows PowerShell 5.1).
We consider it experimental because of the rather hacky way we shell out to highlight and, long-term, it would be preferable to get syntax-highlighted RTF through a proper R package.
The existing [highlight R package](https://cran.r-project.org/package=highlight) can't do this (yet?).

### How to install highlight

The highlight website offers downloads in various forms, for various OSes:

* <http://www.andre-simon.de/zip/download.php>

For macOS users who use [homebrew](https://brew.sh), this works:

```{sh eval = FALSE}
brew install highlight
```

For Windows users who use [chocolatey](https://chocolatey.org/), this works:

```{sh eval = FALSE}
choco install highlight
```

On Windows, the chocolatey method did not add highlight to my `PATH` and this could affect other methods of installation as well.
In this case, you'll [need to do this manually](https://www.maketecheasier.com/what-is-the-windows-path/).
In my case, I needed to add `C:\Program Files\Highlight` to `PATH`.
A second issue we've seen is with the scoop package manager for Windows, which uses a shim-based method of making the executables it installs findable.
The shim method appears to be less resilient to the different ways of calling highlight from R.

`reprex_rtf()` should work when `Sys.which("highlight")`, executed in R, correctly returns the path to your highlight executable.

Here's how that looks for me on macOS:

```{r eval = FALSE}
Sys.which("highlight")
#>                  highlight 
#> "/usr/local/bin/highlight" 
```

and on Windows:

```{r eval = FALSE}
Sys.which("highlight")
#>                              highlight 
#> "C:\\PROGRA~1\\HIGHLI~1\\HIGHLI~2.EXE" 
```

Here is the highlight call implied by the default behaviour of `reprex_rtf()`:

```{sh eval = FALSE}
highlight -i foo_reprex.R --out-format rtf --no-trailing-nl --encoding=UTF-8 --style darkbone --font 'Courier Regular' --font-size 50 -o foo_reprex.rtf
```

## Basic usage

Let's reprex some code and stick it on a Keynote slide with other non-R "features".

```{r eval = FALSE}
reprex({
  (x <- rnorm(3))
  mean(x)
}, venue = "rtf")
```

```{r echo = FALSE, out.width = "75%"}
knitr::include_graphics("img/keynote-w00t.png")
```

## highlight options

If you don't like the defaults, control highlight command line arguments by setting `reprex.highlight.*` options.
Why options?
I didn't want to burden `reprex()` with even more arguments and this is usually something you want to set globally for yourself or for an entire project or talk or course.

Set these options with code like this:

```{r eval = FALSE}
options(
  reprex.highlight.hl_style  = "darkbone",
  reprex.highlight.font      = "Source Code Pro",
  reprex.highlight.font_size = 50
)
```

The supported options are `reprex.highlight.hl_style`, `reprex.highlight.font`, `reprex.highlight.font_size` (all shown above), and `reprex.highlight.other` (a catchall for anything else, shown below).

You could set options for all your reprexing by doing this in your user-level `.Rprofile`. Create or open this file via `usethis::edit_r_profile()`.

You could also set options for a specific Project in `.Rprofile` inside the Project. Create or open this file via `usethis::edit_r_profile("project")`.

Finally, you might collect all of the R snippets destined for inclusion in, e.g. some talk slides, in a `.R` file.
It would make sense to set the `reprex.highlight.*` options at the start of that file.

`hl_style` must be one of your existing highlight themes and `font` must be one of your installed fonts.

List available themes:

```{sh eval = FALSE}
highlight --list-scripts=themes
```

But what do these themes *look like*?
Here are some galleries that probably have high overlap with your themes:

<http://www.andre-simon.de/doku/highlight/en/theme-samples.php>  
<https://rclickhandbuch.files.wordpress.com/2014/09/knitrthemesoverview.pdf>

You can also override defaults on-the-fly.
Here's how to do that for a single call using withr and, as a bonus, how to use `reprex.highlight.other` to send arbitrary command line arguments to highlight:

```{r dusk-fira-35-line-numbers, eval = FALSE}
withr::with_options(
  new = c(
    reprex.highlight.hl_style  = "dusk",
    reprex.highlight.font      = "Fira Code Regular",
    reprex.highlight.font_size = 35,
    reprex.highlight.other     = "--line-numbers --line-number-length=2 --zeroes"
  ),
  reprex::reprex(input = c("(x <- rnorm(3))", "mean(x)"), venue = "rtf")
)
```

```{r echo = FALSE, out.width = "60%"}
knitr::include_graphics("img/dusk-fira-35-line-numbers.png")
```

Here's how to set highlight options for part of an R session:

```{r anotherdark-andale-60-line-numbers, eval = FALSE}
op <- options(
  reprex.highlight.hl_style  = "anotherdark",
  reprex.highlight.font      = "Andale Mono Regular",
  reprex.highlight.font_size = 60,
  reprex.highlight.other     = "--line-numbers"
)

reprex::reprex(input = c("(x <- rnorm(3))", "mean(x)"), venue = "rtf")

options(op)
```

```{r echo = FALSE, out.width = "85%"}
knitr::include_graphics("img/anotherdark-andale-60-line-numbers.png")
```

Experience has shown that it is hard to predict which aspects of the RTF will be honored when you paste into the target document.
We're talking about things like the font size or background color.
This seems to vary greatly across applications and operating systems and you'll just have to experiment to find a workflow that is practical for you.

Note that it is even possible to create a custom highlight theme, e.g. to match the theme of a Keynote or Powerpoint slide deck with respect to background color, font, etc.
A specific example of this is documented in these [how-to notes](https://github.com/jennybc/debugging/blob/master/how-to/highlight-custom-theme.md).

## Run ... or not

If you just want to highlight code, but not run it, set chunk option `eval = FALSE` option, i.e. put a special comment as the first line of your reprex source:

```{r eval-false, eval = FALSE}
#+ eval = FALSE
(x <- rnorm(3))
mean(x)
```

The above source, when `reprex()`ed, will not include the usual output.
This is not specific to `venue = "rtf"`, but it's more likely to come up in this setting.

```{r echo = FALSE, out.width = "20%"}
knitr::include_graphics("img/eval-false.png")
```

## What if each snippet can't be self-contained?

### `reprex:::prex()` and friends

When preparing bits of code for a presentation, it can be impractical to make each snippet completely self-contained.
You really need to run the snippets in the current workspace and session. But `reprex()` does not allow this.

The **unexported** function `prex()` addresses this use case.
It's like `reprex()`, but much less reproducible!

  * Code is evaluated in the global environment of the current R session.
  * Current working directory is used.
  * `advertise = FALSE` is the default.

`prex()` also has "venue as suffix" variants, so `reprex:::prex_rtf(...)` is short for `reprex:::prex(..., venue = "rtf")`.

This code defines `x` and `y` in the current workspace and writes the file `stuff.txt` to the current working directory.

```{r eval = FALSE}
x <- 1
y <- 4
writeLines("blah blah", file.path(getwd(), "stuff.txt"))
```

Now we can show the difference between `prex()` and `reprex()`.

```{r eval = FALSE}
prex({
  ls()
  x + y
  readLines("stuff.txt")  
})
```

yields

```{r eval = FALSE}
ls()
#> [1] "x" "y"
x + y
#> [1] 5
readLines("stuff.txt")
#> [1] "blah blah"
```

`x` and `y` can be found, even though they aren't defined in the snippet, and `stuff.txt` can be read.

This is very different from executing the same code via `reprex()`:

```{r eval = FALSE}
reprex({
  ls()
  x + y
  readLines("stuff.txt")  
})
```

which yields:

```{r eval = FALSE}
ls()
#> character(0)
x + y
#> Error in eval(expr, envir, enclos): object 'x' not found
readLines("stuff.txt")
#> Warning in file(con, "r"): cannot open file 'stuff.txt': No such file or
#> directory
#> Error in file(con, "r"): cannot open the connection
```

`x` and `y` can't be found, because `reprex()` works in a clean session, and `stuff.txt` can't be found, because `reprex()` works in a temp directory, by default.

If you're going to `prex()` a lot, you'll want to do something like this in your `.R` file of snippets or in the project-specific or user-level `.Rprofile`:

```{r eval = FALSE}
prex <- reprex:::prex
prex_rtf <- reprex:::prex_rtf
```

## Selectively hide and reveal code

*Before `prex()` existed, you had to use a more cumbersome technique to produce small snippets, by hiding code.
We leave this here in case it is still useful in some situations.*

You can use knitr chunk options to selectively hide code, i.e. to run a large snippet, but only include a small bit in the output.

This is easiest to show by example.
If you `reprex()` this code, only the last three lines and their output will appear in the output.
But the other lines will be run, i.e. `str_reverse()` will be defined, as will the `bizzaro()` generic and default method.

```{r bizarro-selective-reveal, eval = FALSE}
#+ include = FALSE
str_reverse <- function(x) {
  vapply(
    strsplit(x, ""),
    FUN = function(z) paste(rev(z), collapse = ""),
    FUN.VALUE = "")
}

#+ include = FALSE
bizarro <- function(x) {
  UseMethod("bizarro")
}
bizarro.default <- function(x) {
  stop(
    "Don't know how to make bizzaro <",
    class(x)[[1]], ">",
    call. = FALSE
  )
}

#+ include = TRUE
bizarro.character <- function(x) str_reverse(x)

bizarro(c("abc", "def"))

bizarro(1:5)
```

```{r echo = FALSE, out.width = "70%"}
knitr::include_graphics("img/bizarro-selective-reveal.png")
```

---
output: reprex::reprex_document
knit: reprex::reprex_render
---

Describe your issue very briefly here. Then show it with a minimal, self-contained example in the following R chunk.

```{r}
x <- 1
y <- "2"
x + y
```

---
title: "Suppress package startup messages"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Suppress package startup messages}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

Sometimes your reprex uses packages that emit messages and warnings at startup (dplyr is a very common culprit). In general, these are worth reading! They can alert you to the root cause of your problem, such as a function in one package masking a function in another. But in many cases, this is just distracting, startup noise.

How can you silence this chatter, specifically? We don't want to suppress messages and warnings, in general, because they are an important part of the reprex.

## TL;DR

Here's a quick look at various techniques. They are described in more detail below.

Call `library()` with `warn.conflicts = FALSE`.

```{r, eval = FALSE}
library(dplyr, warn.conflicts = FALSE)
```

Surround a chatty `library()` call with `suppressPackageStartupMessages()`.

```{r, eval = FALSE}
suppressPackageStartupMessages(library(dplyr))
```

Break your reprex into "chunks", in the `.Rmd` sense, and use a special `#+` comment to silence messages and/or warnings for the chunk that holds a chatty `library()` call. Note that the second `#+` comment is very important, so you don't silence messages and warnings for the entire reprex.

```{r, eval = FALSE}
#+ message = FALSE, warning = FALSE
library(dplyr)

#+
slice(iris, 1)
```

If you're using one or more tidyverse packages, consider using the tidyverse metapackage, literally. `reprex::reprex()` has an argument `tidyverse_quiet`, which defaults to `TRUE` and silences the startup messages.

```{r, eval = FALSE}
library(tidyverse)

slice(iris, 1)
```

`tidyverse_quiet` also silences startup messages from the [tidymodels](https://www.tidymodels.org) meta-package.

## dplyr is chatty at startup

dplyr is a common culprit for noisy startup, so we use it as an example. Note this messaging as a baseline.

```{r}
library(dplyr)
```

```{r include = FALSE}
unloadNamespace("dplyr")
```

## `warn.conflicts = FALSE`

To suppress warnings about conflicts, set the `warn.conflicts` argument of `library()` to `FALSE`.

```{r}
library(dplyr, warn.conflicts = FALSE)

slice(iris, 1)
```

```{r include = FALSE}
unloadNamespace("dplyr")
```

## `suppressPackageStartupMessages()`

Surround `library()` with `suppressPackageStartupMessages()`.

```{r}
suppressPackageStartupMessages(library(dplyr))

slice(iris, 1)
```

```{r include = FALSE}
unloadNamespace("dplyr")
```

## Set `message = FALSE` and `warning = FALSE` for a chunk

If we were working in R Markdown, we could suppress messages and warnings in the chunk containing `library()` calls, then put our "real code" in a different chunk:

<pre><code>```{r, message = FALSE, warning = FALSE}  
library(dplyr)  
```

Some text.

`r ''````{r}     
slice(iris, 1)
```</code></pre>

We can do the same in plain R code, suitable for `reprex()`ing, by using special comments that start with `#+`. Note that the second `#+` is significant, because it begins a new chunk capable of emitting messages and warnings.


```{r eval = FALSE}
#+ message = FALSE, warning = FALSE
library(dplyr)
message("You CANNOT hear me!")

#+ 
message("You can hear me!")
slice(iris, 1)
```

## reprex knows about `tidyverse_quiet`

The `reprex::reprex()` function has a `tidyverse_quiet` argument that defaults to `TRUE`. If your reprex uses one or more tidyverse packages, consider attaching the tidyverse metapackage, instead of individual packages, in order to enjoy a quiet startup.

<!-- faking this so I don't have to install tidyverse just for pkgdown -->
```{r eval = FALSE}
library(tidyverse) # instead of library(dplyr)

slice(iris, 1)
```

```{r echo = FALSE}
suppressPackageStartupMessages(library(dplyr))

slice(iris, 1)
```

Note that this default behaviour can be overridden by setting `tidyverse_quiet = FALSE` in a specific `reprex()` call or by setting the option `reprex.tidyverse_quiet = FALSE` in the `.Rprofile` startup file. The `tidyverse_quiet` argument and `reprex.tidyverse_quiet` option also affect startup messages from the [tidymodels](https://www.tidymodels.org) meta-package.

---
title: "Using datapasta with reprex"
author: "Mara Averick"
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

While using a built-in dataset for a reprex is ideal, sometimes you need to troubleshoot code with a bit of outside data. Meet [datapasta](https://github.com/MilesMcBain/datapasta), a package by [Miles McBain](https://twitter.com/milesmcbain) for "reducing resistance associated with copying and pasting data to and from R." Like reprex, datapasta has addin functionality, so you can select commands from the RStudio Addins and/or associate keyboard shortcuts with them for easy use.

## Scenario 1: From spreadsheet to reprex with `tribble_paste()`

It's easy to copy and paste nicely formatted data from a spreadsheet with datapasta's `tribble_paste()` functionality. I've found the simplest method to be: 

1. Copy data from a source onto your clipboard.  
2. Click **Paste as tribble** from the datapasta section of the RStudio Addins drop-down.  
3. Be sure to assign your data frame to a name, so you can use it elsewhere in your reprex.

<div class="figure" style="height: auto">
<img src="https://raw.githubusercontent.com/tidyverse/reprex/main/img/datapasta_w_reprex_sheet_to_tribble.gif" alt="Gif of using datapasta::tribble_paste() to get data from a spreadsheet as R code for use in a reprex" style="width: 100%;max-height: 100%" />
<p class = "caption"><code>datapasta::tribble_paste()</code></p>
</div>

While the output of `tribble_paste()` doesn't have the same level of detail it would with [`dput()`](https://stat.ethz.ch/R-manual/R-patched/library/base/html/dput.html), it works well enough for most scenarios, with the added benefit of being easy to read on the screen.

## Scenario 2: From R object to tibble with `dpasta()`

Now let’s say you have a data frame in R that you want to use in a reprex. You can use datapasta's `dpasta()` function to render your object into the same format as the result of `tribble_paste()`, above.

<div class="figure" style="height: auto">
<img src="https://raw.githubusercontent.com/tidyverse/reprex/main/img/dpasta_datapasta_reprex.gif" alt="Gif of using datapasta::dpasta() to convert an R data frame into the R code needed to define it in a reprex" style="width: 100%;max-height: 100%" />
<p class = "caption"><code>datapasta::dpasta()</code></p>
</div>

## Fin

There's more to datapasta than discussed here, so be sure to check out its [documentation](https://github.com/MilesMcBain/datapasta). Don't forget, when it comes to reprex, less is always more, so wield the power of datapasta wisely.

---
title: "How to use reprex"
---

Materials from an [RStudio webinar](https://resources.rstudio.com/webinars) delivered September 2018:

### Help me help you: Creating reproducible examples with reprex  

What is a reprex? It’s a **repr**oducible **ex**ample. Making a great reprex is both an art and a science and this webinar will cover both aspects. A reprex makes a conversation about code more efficient and pleasant for all. This comes up whenever you ask someone for help, report a bug in software, or propose a new feature. The reprex package (https://reprex.tidyverse.org) makes it especially easy to prepare R code as a reprex, in order to share on sites such as https://community.rstudio.com, https://github.com, or https://stackoverflow.com. The habit of making little, rigorous, self-contained examples also has the great side effect of making you think more clearly about your programming problems.

Webinar page: <https://resources.rstudio.com/webinars/help-me-help-you-creating-reproducible-examples-jenny-bryan>

<script src="https://fast.wistia.com/embed/medias/eu1gge6c34.jsonp" async></script><script src="https://fast.wistia.com/assets/external/E-v1.js" async></script><div class="wistia_responsive_padding" style="padding:56.25% 0 0 0;position:relative;"><div class="wistia_responsive_wrapper" style="height:100%;left:0;position:absolute;top:0;width:100%;"><div class="wistia_embed wistia_async_eu1gge6c34 videoFoam=true" style="height:100%;position:relative;width:100%"><div class="wistia_swatch" style="height:100%;left:0;opacity:0;overflow:hidden;position:absolute;top:0;transition:opacity 200ms;width:100%;"><img src="https://fast.wistia.com/embed/medias/eu1gge6c34/swatch" style="filter:blur(5px);height:100%;object-fit:contain;width:100%;" alt="Video of September 2018 RStudio webinar about reprex" onload="this.parentNode.style.opacity=1;" /></div></div></div></div>

Slides on SpeakerDeck: 

<https://speakerdeck.com/jennybc/reprex-reproducible-examples-with-r>

<script async class="speakerdeck-embed" data-id="a5eedf90acb648fab7a3f1f0a5bb8885" data-ratio="1.77777777777778" src="//speakerdeck.com/assets/embed.js"></script>

<!-- 
Video: <https://youtu.be/MmTPhGQWPUo>

<iframe width="560" height="315" src="https://www.youtube.com/embed/MmTPhGQWPUo?rel=0" frameborder="0" allow="autoplay; encrypted-media" allowfullscreen></iframe>
-->

# ad responds to venue

    Code
      ad("gh")
    Output
      <sup>Created on `r Sys.Date()` with [reprex v`r utils::packageVersion("reprex")`](https://reprex.tidyverse.org)</sup>

---

    Code
      ad("slack")
    Output
      Created on `r Sys.Date()` with [reprex v`r utils::packageVersion("reprex")`](https://reprex.tidyverse.org)

---

    Code
      ad("r")
    Output
      Created on `r Sys.Date()` with reprex v`r utils::packageVersion("reprex")` https://reprex.tidyverse.org

# local .Rprofile reporting responds to venue

    Code
      rprofile_alert("gh")
    Output
      [1] "```{r, results = 'asis', echo = FALSE, include = file.exists('.Rprofile'), eval = file.exists('.Rprofile')}"
      [2] "cat(sprintf(\"*Local `.Rprofile` detected at `%s`*\", normalizePath(\".Rprofile\")))"                       
      [3] "```"                                                                                                        

---

    Code
      rprofile_alert("r")
    Output
      [1] "```{r, results = 'asis', echo = FALSE, include = file.exists('.Rprofile'), eval = file.exists('.Rprofile')}"
      [2] "cat(sprintf(\"Local .Rprofile detected at %s\", normalizePath(\".Rprofile\")))"                             
      [3] "```"                                                                                                        

# reprex: expression input works

    Code
      cli::cat_line(reprex({
        x <- 1:5
        mean(x)
      }, render = FALSE))
    Output
      #' ---
      #' output: reprex::reprex_document
      #' ---
      
      x <- 1:5
      mean(x)

# reprex: character input works

    Code
      cli::cat_line(reprex(input = c("x <- 5:1", "mean(x)"), render = FALSE))
    Output
      #' ---
      #' output: reprex::reprex_document
      #' ---
      
      x <- 5:1
      mean(x)

# reprex: file input works

    Code
      cli::cat_line(reprex(input = "foo.R", render = FALSE))
    Output
      #' ---
      #' output: reprex::reprex_document
      #' ---
      
      x <- 6:10
      mean(x)

# reprex: file input in a subdirectory works

    Code
      cli::cat_line(reprex(input = path("foo", "foo.R"), render = FALSE))
    Output
      #' ---
      #' output: reprex::reprex_document
      #' ---
      
      x <- 11:15
      mean(x)

# Circular use is detected before source file written

    Code
      reprex(input = ret, render = FALSE)
    Condition
      Error in `reprex()`:
      ! Cancelling.

---

    Code
      reprex(input = ret, render = FALSE)
    Condition
      Error in `reprex()`:
      ! Cancelling.

---

    Code
      reprex(input = ret, render = FALSE)
    Condition
      Error in `reprex()`:
      ! Cancelling.

# Leading prompts are removed

    Code
      res2 <- reprex(input = input2, render = FALSE)
    Message
      i Removing leading prompts from reprex source.

---
output:
  reprex::reprex_document:
    venue: "gh"
    advertise: TRUE
    session_info: TRUE
    std_out_err: TRUE
knit: reprex::reprex_render
---

Used in a test to make sure we don't add the reprex advertisement and stubs for session info and standard output/error to a source `.Rmd` document.
Potentially over and over again.

```{r}
x <- rnorm(10)
quantile(x)
```

# reprex_alert() and friends work

    Code
      reprex_alert("alert", type = "")
    Message
      > alert
    Code
      reprex_success("success")
    Message
      v success
    Code
      reprex_info("info")
    Message
      i info
    Code
      reprex_warning("warning")
    Message
      ! warning
    Code
      reprex_danger("danger")
    Message
      x danger

# reprex_alert() is under the control of REPREX_QUIET env var

    Code
      reprex_alert("alert", type = "")

---

    Code
      reprex_alert("alert", type = "")
    Message
      > alert

# reprex_path() works and respects REPREX_QUIET

    Code
      reprex_path("Something descriptive:", "path/to/file")

---

    Code
      reprex_path("Something descriptive:", "path/to/file")
    Message
      v Something descriptive:
        'path/to/file'
    Code
      x <- "path/to/file"
      reprex_path("Something descriptive:", x)
    Message
      v Something descriptive:
        'path/to/file'
    Code
      y <- c("path", "to", "file")
      reprex_path("Something descriptive:", path_join(y))
    Message
      v Something descriptive:
        'path/to/file'

# retrofit_files() works

    The `outfile` argument of `reprex()` is deprecated as of reprex 2.0.0.
    i Please use the `wd` argument instead.

---

    The `outfile` argument of `reprex()` is deprecated as of reprex 2.0.0.
    i Use `reprex(wd = ".")` instead of `reprex(outfile = NA)`.

---

    The `outfile` argument of `reprex()` is deprecated as of reprex 2.0.0.
    i To control output filename, provide a filepath to `input`.
    i Only taking working directory from `outfile`.

---

    The `outfile` argument of `reprex()` is deprecated as of reprex 2.0.0.
    i Working directory will be derived from `input`.

---

    The `outfile` argument of `reprex()` is deprecated as of reprex 2.0.0.
    i Working directory and output filename will be determined from `input`.

# reprex() works with code that deals with srcrefs

    [1] "``` r"                                                    
    [2] "utils::getParseData(parse(text = 'a'))"                   
    [3] "#>   line1 col1 line2 col2 id parent  token terminal text"
    [4] "#> 1     1    1     1    1  1      3 SYMBOL     TRUE    a"
    [5] "#> 3     1    1     1    1  3      0   expr    FALSE"     
    [6] "```"                                                      

# reprex() errors for an R crash, by default

    Code
      code <- "rlang::node_car(0)\n"
      reprex(input = code)
    Condition
      Error in `reprex_render()`:
      ! This reprex appears to crash R. Call `reprex()` again with `std_out_err = TRUE` to get more info.

# reprex() copes with an R crash, when `std_out_err = TRUE`

    Code
      out
    Output
      [1] "This reprex appears to crash R."
      [2] "See standard output and standard error for more details."
      [3] ""
      [4] "#### Standard output and error"
      [5] ""
      [6] "``` sh"
      [7] ""
      [8] " *** caught segfault ***"
      [9] "address ADDRESS, cause 'CAUSE'"
      [10] ""
      [11] "Traceback:"

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reprex-addin.R
\name{reprex_addin}
\alias{reprex_addin}
\alias{reprex_selection}
\title{Render a reprex, conveniently}
\usage{
reprex_addin()

reprex_selection(venue = getOption("reprex.venue", "gh"))
}
\arguments{
\item{venue}{Character. Must be one of the following (case insensitive):
\itemize{
\item "gh" for \href{https://github.github.com/gfm/}{GitHub-Flavored Markdown}, the
default
\item "r" for a runnable R script, with commented output interleaved. Also useful
for \href{https://slack.com/intl/en-ca/slack-tips/share-code-snippets}{Slack code snippets};
select "R" from the "Type" drop-down menu to enjoy nice syntax
highlighting.
\item "rtf" for
\href{https://en.wikipedia.org/wiki/Rich_Text_Format}{Rich Text Format}
(not supported for un-reprexing)
\item "html" for an HTML fragment suitable for inclusion in a larger HTML
document (not supported for un-reprexing)
\item "slack" for pasting into a Slack message. Optimized for people who opt out
of Slack's WYSIWYG interface. Go to
\strong{Preferences > Advanced > Input options} and select "Format messages with
markup". (If there is demand for a second Slack venue optimized for use
with WYSIWYG, please open an issue to discuss.)
\item "so" for
\href{https://stackoverflow.com/editing-help#syntax-highlighting}{Stack Overflow Markdown}.
Note: this is just an alias for "gh", since Stack Overflow started to
support CommonMark-style fenced code blocks in January 2019.
\item "ds" for Discourse, e.g.,
\href{https://community.rstudio.com}{community.rstudio.com}. Note: this is
currently just an alias for "gh".
}}
}
\description{
\code{reprex_addin()} opens an \href{https://shiny.rstudio.com/articles/gadgets.html}{RStudio gadget} and
\href{http://rstudio.github.io/rstudioaddins/}{addin} that allows you to say
where the reprex source is (clipboard? current selection? active file?
other file?) and to control a few other arguments. Appears as "Render
reprex" in the RStudio Addins menu.

\code{reprex_selection()} is an
\href{http://rstudio.github.io/rstudioaddins/}{addin} that reprexes the current
selection, optionally customised by options. Appears as "Reprex selection"
in the RStudio Addins menu. Heavy users might want to create a keyboard
shortcut, which is described in
\verb{https://support.rstudio.com/hc/en-us/articles/206382178-Customizing-Keyboard-Shortcuts}.
Suggested shortcut: Cmd + Shift + R (macOS) or Ctrl + Shift + R (Windows).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/venues.R
\name{reprex_venue}
\alias{reprex_venue}
\alias{reprex_html}
\alias{reprex_r}
\alias{reprex_rtf}
\alias{reprex_slack}
\title{Venue-specific shortcuts}
\usage{
reprex_html(...)

reprex_r(...)

reprex_rtf(...)

reprex_slack(...)
}
\arguments{
\item{...}{Passed along to \code{\link[=reprex]{reprex()}}.}
}
\description{
These are thin wrappers around \code{reprex()} that incorporate the target \code{venue}
as a suffix in the function name, for easier access via auto-completion.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reprex-options.R
\name{reprex_options}
\alias{reprex_options}
\alias{opt}
\title{reprex options}
\description{
Some \code{\link[=reprex]{reprex()}} behaviour can be controlled via an option, providing a way
for the user to set personal defaults. The pattern for such option names is
\verb{reprex.<arg>}, where \verb{<arg>} is an argument of \code{\link[=reprex]{reprex()}}. Here are the main
ones:
\itemize{
\item \code{reprex.advertise}
\item \code{reprex.session_info} (previously, \code{reprex.si})
\item \code{reprex.style}
\item \code{reprex.html_preview} (previously, \code{reprex.show})
\item \code{reprex.comment}
\item \code{reprex.tidyverse_quiet}
\item \code{reprex.std_out_err}
}

A few more options exist, but are only relevant to specific situations:
\itemize{
\item \code{reprex.venue}: Can be used to control the \code{venue} used by the
\code{\link[=reprex_selection]{reprex_selection()}} addin.
\item \code{reprex.current_venue}: Read-only option that is set during
\code{\link[=reprex_render]{reprex_render()}}. Other packages that want to generate reprex-compatible
output can consult it via \code{getOption("reprex.current_venue")}, if they want
to tailor their output to the \code{venue}.
\item \code{reprex.clipboard}: When \code{FALSE}, reprex makes no attempt to access the
user's clipboard, ever. This exists mostly for internal use, i.e. we set it
to \code{FALSE} when we detect use from RStudio Server. But a user could set
this to \code{FALSE} to explicitly opt-out of clipboard functionality. A Linux
user with no intention of installing \code{xclip} or \code{xsel} might also do this.
\item \code{reprex.highlight.hl_style}: Only relevant to \verb{venue = "rtf}. Details are
in the article
\href{https://reprex.tidyverse.org/articles/articles/rtf.html}{reprex venue RTF}.
\item \code{reprex.highlight.font}: See above.
\item \code{reprex.highlight.font_size}: See above.
\item \code{reprex.highlight.other}: See above.
}

Here's code you could put in \code{.Rprofile} to set reprex options. It would be
rare to want non-default behaviour for all of these! We only do so here for
the sake of exposition:

\if{html}{\out{<div class="sourceCode">}}\preformatted{options(
  reprex.advertise       = FALSE,
  reprex.session_info    = TRUE,
  reprex.style           = TRUE,
  reprex.html_preview    = FALSE,
  reprex.comment         = "#;-)",
  reprex.tidyverse_quiet = FALSE,
  reprex.std_out_err     = TRUE,
  reprex.venue           = "html", # NOTE: only affects reprex_selection()!
  reprex.highlight.hl_style  = "acid", # NOTE: only affects RTF venue
  reprex.highlight.font      = "Andale Mono Regular",
  reprex.highlight.font_size = 35,
  reprex.highlight.other     = "--line-numbers"
)
}\if{html}{\out{</div>}}

The function \code{usethis::edit_r_profile()} is handy for creating and/or opening
your \code{.Rprofile}.
}
\section{Explaining the \code{opt()} helper}{

Arguments that appear like so in \code{\link[=reprex]{reprex()}}:

\if{html}{\out{<div class="sourceCode">}}\preformatted{reprex(..., arg = opt(DEFAULT), ...)
}\if{html}{\out{</div>}}

get their value according to this logic:

\if{html}{\out{<div class="sourceCode">}}\preformatted{user-specified value or, if not given,
  getOption("reprex.arg") or, if does not exist,
    DEFAULT
}\if{html}{\out{</div>}}

It's shorthand for:

\if{html}{\out{<div class="sourceCode">}}\preformatted{f(..., arg = getOption("reprex.arg", DEFAULT), ...)
}\if{html}{\out{</div>}}

This is not an exported function and should not be called directly.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reprex_document.R
\name{reprex_document}
\alias{reprex_document}
\title{reprex output format}
\usage{
reprex_document(
  venue = c("gh", "r", "rtf", "html", "slack", "so", "ds"),
  advertise = NULL,
  session_info = opt(FALSE),
  style = opt(FALSE),
  comment = opt("#>"),
  tidyverse_quiet = opt(TRUE),
  std_out_err = opt(FALSE),
  pandoc_args = NULL
)
}
\arguments{
\item{venue}{Character. Must be one of the following (case insensitive):
\itemize{
\item "gh" for \href{https://github.github.com/gfm/}{GitHub-Flavored Markdown}, the
default
\item "r" for a runnable R script, with commented output interleaved. Also useful
for \href{https://slack.com/intl/en-ca/slack-tips/share-code-snippets}{Slack code snippets};
select "R" from the "Type" drop-down menu to enjoy nice syntax
highlighting.
\item "rtf" for
\href{https://en.wikipedia.org/wiki/Rich_Text_Format}{Rich Text Format}
(not supported for un-reprexing)
\item "html" for an HTML fragment suitable for inclusion in a larger HTML
document (not supported for un-reprexing)
\item "slack" for pasting into a Slack message. Optimized for people who opt out
of Slack's WYSIWYG interface. Go to
\strong{Preferences > Advanced > Input options} and select "Format messages with
markup". (If there is demand for a second Slack venue optimized for use
with WYSIWYG, please open an issue to discuss.)
\item "so" for
\href{https://stackoverflow.com/editing-help#syntax-highlighting}{Stack Overflow Markdown}.
Note: this is just an alias for "gh", since Stack Overflow started to
support CommonMark-style fenced code blocks in January 2019.
\item "ds" for Discourse, e.g.,
\href{https://community.rstudio.com}{community.rstudio.com}. Note: this is
currently just an alias for "gh".
}}

\item{advertise}{Logical. Whether to include a footer that describes when and
how the reprex was created. If unspecified, the option \code{reprex.advertise}
is consulted and, if that is not defined, default is \code{TRUE} for venues
\code{"gh"}, \code{"html"}, \code{"so"}, \code{"ds"} and \code{FALSE} for \code{"r"}, \code{"rtf"}, \code{"slack"}.}

\item{session_info}{Logical. Whether to include
\code{\link[sessioninfo:session_info]{sessioninfo::session_info()}}, if available, or \code{\link[=sessionInfo]{sessionInfo()}} at the end
of the reprex. When \code{venue} is "gh", the session info is wrapped in a
collapsible details tag. Read more about \code{\link[=opt]{opt()}}.}

\item{style}{Logical. Whether to set the knitr chunk option \code{tidy = "styler"}, which re-styles code with the \href{https://styler.r-lib.org}{styler package}. Read more about \code{\link[=opt]{opt()}}.}

\item{comment}{Character. Prefix with which to comment out output, defaults
to \code{"#>"}. Read more about \code{\link[=opt]{opt()}}.}

\item{tidyverse_quiet}{Logical. Sets the options \code{tidyverse.quiet} and
\code{tidymodels.quiet}, which suppress (\code{TRUE}, the default) or include
(\code{FALSE}) the startup messages for the tidyverse and tidymodels packages.
Read more about \code{\link[=opt]{opt()}}.}

\item{std_out_err}{Logical. Whether to append a section for output sent to
stdout and stderr by the reprex rendering process. This can be necessary to
reveal output if the reprex spawns child processes or \code{system()} calls.
Note this cannot be properly interleaved with output from the main R
process, nor is there any guarantee that the lines from standard output and
standard error are in correct chronological order. See \code{\link[callr:r]{callr::r()}} for
more. Read more about \code{\link[=opt]{opt()}}.}

\item{pandoc_args}{Additional command line options to pass to pandoc}
}
\value{
An R Markdown output format to pass to \code{\link[rmarkdown:render]{rmarkdown::render()}}.
}
\description{
This is an R Markdown output format designed specifically for making
"reprexes", typically created via the \code{\link[=reprex]{reprex()}} function, which ultimately
renders the document with \code{\link[=reprex_render]{reprex_render()}}. It is a heavily modified version
of \code{\link[rmarkdown:md_document]{rmarkdown::md_document()}}. The arguments have different spheres of
influence:
\itemize{
\item \code{venue} potentially affects input preparation and \code{\link[=reprex_render]{reprex_render()}}.
\item Add content to the primary input, prior to rendering:
\itemize{
\item \code{advertise}
\item \code{session_info}
\item \code{std_out_err} (also consulted by \code{\link[=reprex_render]{reprex_render()}})
}
\item Influence knitr package or chunk options:
\itemize{
\item \code{style}
\item \code{comment}
\item \code{tidyverse_quiet}
}
}

RStudio users can create new R Markdown documents with the
\code{reprex_document()} format using built-in templates. Do
\emph{File > New File > R Markdown ... > From Template} and choose one of:
\itemize{
\item reprex (minimal)
\item reprex (lots of features)
}

Both include \code{knit: reprex::reprex_render} in the YAML, which causes the
RStudio "Knit" button to use \code{reprex_render()}. If you render these documents
yourself, you should do same.
}
\examples{
reprex_document()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reprex-package.R
\docType{package}
\name{reprex-package}
\alias{reprex-package}
\alias{_PACKAGE}
\title{reprex: Prepare Reproducible Example Code via the Clipboard}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Convenience wrapper that uses the 'rmarkdown' package to render small snippets of code to target formats that include both code and output. The goal is to encourage the sharing of small, reproducible, and runnable examples on code-oriented websites, such as \url{https://stackoverflow.com} and \url{https://github.com}, or in email. The user's clipboard is the default source of input code and the default target for rendered output. 'reprex' also extracts clean, runnable R code from various common formats, such as copy/paste from an R session.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://reprex.tidyverse.org}
  \item \url{https://github.com/tidyverse/reprex#readme}
  \item Report bugs at \url{https://github.com/tidyverse/reprex/issues}
}

}
\author{
\strong{Maintainer}: Jennifer Bryan \email{jenny@rstudio.com} (\href{https://orcid.org/0000-0002-6983-2759}{ORCID})

Authors:
\itemize{
  \item Jim Hester \email{james.hester@rstudio.com} (\href{https://orcid.org/0000-0002-2739-7082}{ORCID})
  \item David Robinson \email{admiral.david@gmail.com}
  \item Hadley Wickham \email{hadley@rstudio.com} (\href{https://orcid.org/0000-0003-4757-117X}{ORCID})
  \item Christophe Dervieux \email{cderv@rstudio.com} (\href{https://orcid.org/0000-0003-4474-2498}{ORCID})
}

Other contributors:
\itemize{
  \item RStudio [copyright holder, funder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reprex-locale.R
\name{reprex_locale}
\alias{reprex_locale}
\title{Render a reprex in a specific locale}
\usage{
reprex_locale(..., language = "en", locale = NULL)
}
\arguments{
\item{...}{Inputs passed through to \code{\link[=reprex]{reprex()}}.}

\item{language}{A string specifying the preferred language for messages. It
is enacted via the \code{LANGUAGE} environment variable, for the duration of the
\code{reprex()} call. Examples: \code{"en"} for English and \code{"fr"} for French. See
Details for more.}

\item{locale}{A named character vector, specifying aspects of the locale, in
the \code{\link[=Sys.setlocale]{Sys.setlocale()}} sense. It is enacted by setting one or more
environment variables, for the duration of the \code{reprex()} call. See Details
for more.}
}
\value{
Character vector of rendered reprex, invisibly.
}
\description{
Render a \code{\link[=reprex]{reprex()}}, with control over the localization of error messages and
aspects of the locale. Note that these are related but distinct issues!
Typical usage is for someone on a Spanish system to create a reprex that is
easier for an English-speaking audience to follow.
}
\section{\code{language}}{

Use the \code{language} argument to express the preferred language of error
messages. The output of \code{dir(system.file(package = "translations"))} may
provide some helpful ideas. The \code{language} should generally follow "XPG
syntax": a two-letter language code, optionally followed by other modifiers.

Examples: \code{"en"}, \code{"de"}, \code{"en_GB"}, \code{"pt_BR"}.
}

\section{\code{locale}}{

Use the \code{locale} argument only if you want to affect something like how
day-of-the-week or month is converted to character. You are less likely to
need to set this than the \code{language} argument. You may have more success
setting specific categories, such as \code{"LC_TIME"}, than multi-category
shortcuts like \code{"LC_ALL"} or \code{"LANG"}. The \code{locale} values must follow the
format dictated by your operating system and the requested locale must be
installed. On *nix systems, \code{locale -a} is a good way to see which locales
are installed. Note that the format for \code{locale} and \code{language} are different
from each other on Windows.

Examples: \code{"en_CA.UTF-8"} (macOS), \code{"French_France.1252"} (Windows).
}

\examples{
\dontrun{

# if all you want to do is make sure messages are in English
reprex_locale("a" / 2)

# change messages to a specific language
reprex_locale(
  {
    "a" / 2
  },
  language = "it"
)

reprex_locale(
  {
    "a" / 2
  },
  language = "fr_CA"
)

reprex_locale(
  {
    "a" / 2
  },
  language = "pt_BR"
)

# get day-of-week and month to print in French (not Windows)
reprex_locale(
  {
    format(as.Date(c("2019-01-01", "2019-02-01")), "\%a \%b \%d")
  },
  locale = c(LC_TIME = "fr_FR")
)

# get day-of-week and month to print in French (Windows)
# assumes that the relevant language is installed on the system
# LC_TIME can also be specified as "French" or "French_France" here
reprex_locale(
  {
    format(as.Date(c("2019-01-01", "2019-02-01")), "\%a \%b \%d")
  },
  locale = c(LC_TIME = "French_France.1252")
)
}
}
\seealso{
\itemize{
\item The \href{https://www.gnu.org/software/libc/manual/html_node/Locale-Names.html}{Locale Names}
section of the GNU C docs, for more about XPG syntax
\item The \href{https://cran.r-project.org/doc/manuals/r-patched/R-admin.html#Internationalization}{Internationalization and Localization}
section of the R Installation and Administration manual
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reprex.R
\name{reprex}
\alias{reprex}
\title{Render a reprex}
\usage{
reprex(
  x = NULL,
  input = NULL,
  wd = NULL,
  venue = c("gh", "r", "rtf", "html", "slack", "so", "ds"),
  render = TRUE,
  advertise = NULL,
  session_info = opt(FALSE),
  style = opt(FALSE),
  comment = opt("#>"),
  tidyverse_quiet = opt(TRUE),
  std_out_err = opt(FALSE),
  html_preview = opt(TRUE),
  outfile = deprecated(),
  show = deprecated(),
  si = deprecated()
)
}
\arguments{
\item{x}{An expression. If not given, \code{reprex()} looks for code in
\code{input}. If \code{input} is not provided, \code{reprex()} looks on the clipboard.

When the clipboard is structurally unavailable, e.g., on RStudio Server or
RStudio Cloud, \code{reprex()} consults the current selection instead of the
clipboard.}

\item{input}{Character. If has length one and lacks a terminating newline,
interpreted as the path to a file containing reprex code. Otherwise,
assumed to hold reprex code as character vector. When \code{input} specifies a
filepath, it also determines the reprex working directory and the location
of all resulting files.}

\item{wd}{An optional filepath that is consulted when \code{input} is not a
filepath. (By default, all work is done, quietly, in a subdirectory of the
session temp directory.)

The most common use of \code{wd} is to set \code{wd = "."}, which means "reprex right
HERE in the current working directory". Do this if you really must
demonstrate something with local files.}

\item{venue}{Character. Must be one of the following (case insensitive):
\itemize{
\item "gh" for \href{https://github.github.com/gfm/}{GitHub-Flavored Markdown}, the
default
\item "r" for a runnable R script, with commented output interleaved. Also useful
for \href{https://slack.com/intl/en-ca/slack-tips/share-code-snippets}{Slack code snippets};
select "R" from the "Type" drop-down menu to enjoy nice syntax
highlighting.
\item "rtf" for
\href{https://en.wikipedia.org/wiki/Rich_Text_Format}{Rich Text Format}
(not supported for un-reprexing)
\item "html" for an HTML fragment suitable for inclusion in a larger HTML
document (not supported for un-reprexing)
\item "slack" for pasting into a Slack message. Optimized for people who opt out
of Slack's WYSIWYG interface. Go to
\strong{Preferences > Advanced > Input options} and select "Format messages with
markup". (If there is demand for a second Slack venue optimized for use
with WYSIWYG, please open an issue to discuss.)
\item "so" for
\href{https://stackoverflow.com/editing-help#syntax-highlighting}{Stack Overflow Markdown}.
Note: this is just an alias for "gh", since Stack Overflow started to
support CommonMark-style fenced code blocks in January 2019.
\item "ds" for Discourse, e.g.,
\href{https://community.rstudio.com}{community.rstudio.com}. Note: this is
currently just an alias for "gh".
}}

\item{render}{Logical. Whether to call \code{\link[rmarkdown:render]{rmarkdown::render()}} on the templated
reprex, i.e. whether to actually run the code. Defaults to \code{TRUE}. Exists
primarily for the sake of internal testing.}

\item{advertise}{Logical. Whether to include a footer that describes when and
how the reprex was created. If unspecified, the option \code{reprex.advertise}
is consulted and, if that is not defined, default is \code{TRUE} for venues
\code{"gh"}, \code{"html"}, \code{"so"}, \code{"ds"} and \code{FALSE} for \code{"r"}, \code{"rtf"}, \code{"slack"}.}

\item{session_info}{Logical. Whether to include
\code{\link[sessioninfo:session_info]{sessioninfo::session_info()}}, if available, or \code{\link[=sessionInfo]{sessionInfo()}} at the end
of the reprex. When \code{venue} is "gh", the session info is wrapped in a
collapsible details tag. Read more about \code{\link[=opt]{opt()}}.}

\item{style}{Logical. Whether to set the knitr chunk option \code{tidy = "styler"}, which re-styles code with the \href{https://styler.r-lib.org}{styler package}. Read more about \code{\link[=opt]{opt()}}.}

\item{comment}{Character. Prefix with which to comment out output, defaults
to \code{"#>"}. Read more about \code{\link[=opt]{opt()}}.}

\item{tidyverse_quiet}{Logical. Sets the options \code{tidyverse.quiet} and
\code{tidymodels.quiet}, which suppress (\code{TRUE}, the default) or include
(\code{FALSE}) the startup messages for the tidyverse and tidymodels packages.
Read more about \code{\link[=opt]{opt()}}.}

\item{std_out_err}{Logical. Whether to append a section for output sent to
stdout and stderr by the reprex rendering process. This can be necessary to
reveal output if the reprex spawns child processes or \code{system()} calls.
Note this cannot be properly interleaved with output from the main R
process, nor is there any guarantee that the lines from standard output and
standard error are in correct chronological order. See \code{\link[callr:r]{callr::r()}} for
more. Read more about \code{\link[=opt]{opt()}}.}

\item{html_preview}{Logical. Whether to show rendered output in a viewer
(RStudio or browser). Always \code{FALSE} in a noninteractive session. Read more
about \code{\link[=opt]{opt()}}.}

\item{outfile}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} in favor of \code{wd} or
providing a filepath to \code{input}. To reprex in current working directory,
use \code{wd = "."} now, instead of \code{outfile = NA}.}

\item{show}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} in favor of \code{html_preview},
for greater consistency with other R Markdown output formats.}

\item{si}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} in favor of \code{session_info}.}
}
\value{
Character vector of rendered reprex, invisibly.
}
\description{
Run a bit of R code using \code{\link[rmarkdown:render]{rmarkdown::render()}} and write the rendered result
to user's clipboard. If the clipboard is unavailable, the file containing
the rendered result is opened for manual copy. The goal is to make it easy to
share a small reproducible example ("reprex"), e.g., in a GitHub issue.
Reprex source can be
\itemize{
\item read from clipboard
\item provided directly as expression, character vector, or string
\item read from file
\item read from current selection or active document in RStudio
}

reprex can also be used for syntax highlighting (with or without rendering);
see below for more.
}
\section{Details}{

The usual "code + commented output" is returned invisibly, written to file,
and, whenever possible, put on the clipboard. An HTML preview displays in
RStudio's Viewer pane, if available, or in the default browser, otherwise.
Leading \code{"> "} prompts, are stripped from the input code. Read more at
\url{https://reprex.tidyverse.org/}.

reprex sets specific \href{https://yihui.org/knitr/options/}{knitr options}:
\itemize{
\item Chunk options default to \code{collapse = TRUE}, \code{comment = "#>"},
\code{error = TRUE}. Note that \code{error = TRUE}, because a common use case is bug
reporting.
\item reprex also sets knitr's \code{upload.fun}. It defaults to
\code{\link[knitr:imgur_upload]{knitr::imgur_upload()}} so figures produced by the reprex appear properly
on GitHub, Stack Overflow, Discourse, and Slack. Note that \code{imgur_upload()}
requires the packages httr and xml2. When \code{venue = "r"}, \code{upload.fun} is
set to \code{identity()}, so that figures remain local. In that case, you may
also want to provide a filepath to \code{input} or set \code{wd}, to control where
the reprex files are written.
You can supplement or override these options with special comments in your
code (see examples).
}
}

\section{Error backtraces}{


reprex sets the rlang option \code{rlang_backtrace_on_error_report = "full"}.
Combined with the knitr option \code{error = TRUE}, this means rlang errors are
displayed with a full backtrace. This basically eliminates the need to call
\code{\link[rlang:last_error]{rlang::last_error()}} or \code{\link[rlang:last_error]{rlang::last_trace()}} explicitly, although these
functions can be used in a reprex.

Insert a line containing the special comment \verb{#'} in between the
error-causing code and the \code{last_error()} or \code{last_trace()} call, to fulfill
the requirement of being in separate chunks:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{f <- function() rlang::abort('foo')
f()
#'
rlang::last_error()
rlang::last_trace()
}\if{html}{\out{</div>}}

Read more in rlang's documentation: \href{https://rlang.r-lib.org/reference/rlang_backtrace_on_error.html#errors-in-rmarkdown}{Errors in RMarkdown}.
}

\section{Syntax highlighting}{


\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

A secondary use case for reprex is to produce syntax highlighted code
snippets, with or without rendering, to paste into applications like
Microsoft Word, PowerPoint, or Keynote. Use \code{venue = "rtf"} for this.

This feature is experimental and requires the installation of the
\href{http://www.andre-simon.de/doku/highlight/en/highlight.php}{highlight}
command line tool. The \code{"rtf"} venue is documented in \href{https://reprex.tidyverse.org/articles/articles/rtf.html}{its own article}
}

\examples{
\dontrun{
# put some code like this on the clipboard
# (y <- 1:4)
# mean(y)
reprex()

# provide code as an expression
reprex(rbinom(3, size = 10, prob = 0.5))
reprex({y <- 1:4; mean(y)})
reprex({y <- 1:4; mean(y)}, style = TRUE)

# note that you can include newlines in those brackets
# in fact, that is often a good idea
reprex({
  x <- 1:4
  y <- 2:5
  x + y
})

## provide code via character vector
reprex(input = c("x <- 1:4", "y <- 2:5", "x + y"))

## if just one line, terminate with '\n'
reprex(input = "rnorm(3)\n")

## customize the output comment prefix
reprex(rbinom(3, size = 10, prob = 0.5), comment = "#;-)")

# override a default chunk option
reprex({
  #+ setup, include = FALSE
  knitr::opts_chunk$set(collapse = FALSE)

  #+ actual-reprex-code
  (y <- 1:4)
  median(y)
})

# add prose, use general markdown formatting
reprex({
  #' # A Big Heading
  #'
  #' Look at my cute example. I love the
  #' [reprex](https://github.com/tidyverse/reprex#readme) package!
  y <- 1:4
  mean(y)
}, advertise = FALSE)

# read reprex from file and write resulting files to that location
tmp <- file.path(tempdir(), "foofy.R")
writeLines(c("x <- 1:4", "mean(x)"), tmp)
reprex(input = tmp)
list.files(dirname(tmp), pattern = "foofy")

# clean up
file.remove(list.files(dirname(tmp), pattern = "foofy", full.names = TRUE))

# write reprex to file AND keep figure local too, i.e. don't post to imgur
tmp <- file.path(tempdir(), "foofy")
dir.create(tmp)
reprex({
  #+ setup, include = FALSE
  knitr::opts_knit$set(upload.fun = identity)

  #+ actual-reprex-code
  #' Some prose
  ## regular comment
  (x <- 1:4)
  median(x)
  plot(x)
  }, wd = tmp)
list.files(dirname(tmp), pattern = "foofy")

# clean up
unlink(tmp, recursive = TRUE)

## target venue = R, also good for email or Slack snippets
ret <- reprex({
  x <- 1:4
  y <- 2:5
  x + y
}, venue = "R")
ret

## target venue = html
ret <- reprex({
  x <- 1:4
  y <- 2:5
  x + y
}, venue = "html")
ret

## include prompt and don't comment the output
## use this when you want to make your code hard to execute :)
reprex({
  #+ setup, include = FALSE
  knitr::opts_chunk$set(comment = NA, prompt = TRUE)

  #+ actual-reprex-code
  x <- 1:4
  y <- 2:5
  x + y
})

## leading prompts are stripped from source
reprex(input = c("> x <- 1:3", "> median(x)"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reprex-undo.R
\name{un-reprex}
\alias{un-reprex}
\alias{reprex_invert}
\alias{reprex_clean}
\alias{reprex_rescue}
\title{Un-render a reprex}
\usage{
reprex_invert(
  input = NULL,
  wd = NULL,
  venue = c("gh", "r"),
  comment = opt("#>"),
  outfile = deprecated()
)

reprex_clean(
  input = NULL,
  wd = NULL,
  comment = opt("#>"),
  outfile = deprecated()
)

reprex_rescue(
  input = NULL,
  wd = NULL,
  prompt = getOption("prompt"),
  continue = getOption("continue"),
  outfile = deprecated()
)
}
\arguments{
\item{input}{Character. If has length one and lacks a terminating newline,
interpreted as the path to a file containing the reprex. Otherwise,
assumed to hold the reprex as a character vector. If not provided, the
clipboard is consulted for input. If the clipboard is unavailable and
we're in RStudio, the current selection is used.}

\item{wd}{An optional filepath that is consulted when \code{input} is not a
filepath. (By default, all work is done, quietly, in a subdirectory of the
session temp directory.)

The most common use of \code{wd} is to set \code{wd = "."}, which means "reprex right
HERE in the current working directory". Do this if you really must
demonstrate something with local files.}

\item{venue}{Character. Must be one of the following (case insensitive):
\itemize{
\item "gh" for \href{https://github.github.com/gfm/}{GitHub-Flavored Markdown}, the
default
\item "r" for a runnable R script, with commented output interleaved. Also useful
for \href{https://slack.com/intl/en-ca/slack-tips/share-code-snippets}{Slack code snippets};
select "R" from the "Type" drop-down menu to enjoy nice syntax
highlighting.
\item "rtf" for
\href{https://en.wikipedia.org/wiki/Rich_Text_Format}{Rich Text Format}
(not supported for un-reprexing)
\item "html" for an HTML fragment suitable for inclusion in a larger HTML
document (not supported for un-reprexing)
\item "slack" for pasting into a Slack message. Optimized for people who opt out
of Slack's WYSIWYG interface. Go to
\strong{Preferences > Advanced > Input options} and select "Format messages with
markup". (If there is demand for a second Slack venue optimized for use
with WYSIWYG, please open an issue to discuss.)
\item "so" for
\href{https://stackoverflow.com/editing-help#syntax-highlighting}{Stack Overflow Markdown}.
Note: this is just an alias for "gh", since Stack Overflow started to
support CommonMark-style fenced code blocks in January 2019.
\item "ds" for Discourse, e.g.,
\href{https://community.rstudio.com}{community.rstudio.com}. Note: this is
currently just an alias for "gh".
}}

\item{comment}{regular expression that matches commented output lines}

\item{outfile}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} in favor of \code{wd} or
providing a filepath to \code{input}. To reprex in current working directory,
use \code{wd = "."} now, instead of \code{outfile = NA}.}

\item{prompt}{character, the prompt at the start of R commands}

\item{continue}{character, the prompt for continuation lines}
}
\value{
Character vector holding just the clean R code, invisibly
}
\description{
Recover clean, runnable code from a reprex captured in the wild and write it
to user's clipboard. The code is also returned invisibly and optionally
written to file. Three different functions address various forms of
wild-caught reprex.
}
\section{Functions}{
\itemize{
\item \code{reprex_invert()}: Attempts to reverse the effect of \code{\link[=reprex]{reprex()}}. When
\code{venue = "r"}, this just calls \code{reprex_clean()}.

\item \code{reprex_clean()}: Assumes R code is top-level, possibly interleaved with
commented output, e.g., a displayed reprex copied from GitHub or the direct
output of \code{reprex(..., venue = "R")}. This function removes commented
output.

\item \code{reprex_rescue()}: Assumes R code lines start with a prompt and that
printed output is top-level, e.g., what you'd get from copy/paste from the
R Console. Removes lines of output and strips prompts from lines holding R
commands.

}}
\examples{
\dontrun{
# a roundtrip: R code --> rendered reprex, as gfm --> R code
original <- file.path(tempdir(), "original.R")
writeLines(glue::glue("
  #' Some text
  #+ chunk-label-and-options-cannot-be-recovered, message = TRUE
  (x <- 1:4)
  #' More text
  y <- 2:5
  x + y"), con = original)
reprex(input = original, html_preview = FALSE, advertise = FALSE)
reprexed <- sub("[.]R$", "_reprex.md", original)
writeLines(readLines(reprexed))
unreprexed <- reprex_invert(input = reprexed)
writeLines(unreprexed)

# clean up
file.remove(
  list.files(dirname(original), pattern = "original", full.names = TRUE)
)
}
\dontrun{
# a roundtrip: R code --> rendered reprex, as R code --> original R code
code_in <- c(
  "# a regular comment, which is retained",
  "(x <- 1:4)",
  "median(x)"
)
reprexed <- reprex(input = code_in, venue = "r", advertise = FALSE)
writeLines(reprexed)
code_out <- reprex_clean(input = reprexed)
writeLines(code_out)
identical(code_in, code_out)
}
\dontrun{
# rescue a reprex that was copied from a live R session
from_r_console <- c(
  "> # a regular comment, which is retained",
  "> (x <- 1:4)",
  "[1] 1 2 3 4",
  "> median(x)",
  "[1] 2.5"
)
rescued <- reprex_rescue(input = from_r_console)
writeLines(rescued)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reprex_render.R
\name{reprex_render}
\alias{reprex_render}
\title{Render a document in a new R session}
\usage{
reprex_render(input, html_preview = NULL, encoding = "UTF-8")
}
\arguments{
\item{input}{The input file to be rendered. This can be a \code{.R} script or a
\code{.Rmd} R Markdown document.}

\item{html_preview}{Logical. Whether to show rendered output in a viewer
(RStudio or browser). Always \code{FALSE} in a noninteractive session. Read more
about \code{\link[=opt]{opt()}}.}

\item{encoding}{The encoding of the input file. Note that the only acceptable
value is "UTF-8", which is required by knitr as of v1.24. This is exposed
as an argument purely for technical convenience, relating to the "Knit"
button in the RStudio IDE.}
}
\value{
The output of \code{\link[rmarkdown:render]{rmarkdown::render()}} is passed through, i.e. the path
of the output file.
}
\description{
This is a wrapper around \code{\link[rmarkdown:render]{rmarkdown::render()}} that enforces the "reprex"
mentality. Here's a simplified version of what happens:

\if{html}{\out{<div class="sourceCode">}}\preformatted{callr::r(
  function(input) \{
    rmarkdown::render(input, envir = globalenv(), encoding = "UTF-8")
  \},
  args = list(input = input),
  spinner = is_interactive(),
  stdout = std_file, stderr = std_file
)
}\if{html}{\out{</div>}}

Key features to note
\itemize{
\item \code{\link[rmarkdown:render]{rmarkdown::render()}} is executed in a new R session, by using
\code{\link[callr:r]{callr::r()}}. The goal is to eliminate the leakage of objects, attached
packages, and other aspects of session state from the current session into
the rendering session. Also, the system and user-level \code{.Rprofile}s are
ignored.
\item Code is evaluated in the \code{globalenv()} of this new R session, which means
that method dispatch works the way most people expect it to.
\item The input file is assumed to be UTF-8, which is a knitr requirement as of
v1.24.
\item If the YAML frontmatter includes \code{std_err_out: TRUE}, standard output and
error of the rendering R session are captured in \code{std_file}, which is
then injected into the rendered result.
}

\code{reprex_render()} is designed to work with the \code{\link[=reprex_document]{reprex_document()}} output
format, typically through a call to \code{\link[=reprex]{reprex()}}. \code{reprex_render()} may work
with other R Markdown output formats, but it is not well-tested.
}
\examples{
\dontrun{
reprex_render("input.Rmd")
}
}
